!****h* src/interface/tens_comb
! NAME
!
!  Module tens_comb
!
! USAGE
!
!  use tens_comb
!
! DESCRIPTION
!
!  This module contains the routines necessary for the contraction
!  of the tensor coefficients as reconstructed by the module tens_rec
!  with the according tensor integrals.
!
!  Please, note that this module is generated by a script and should not
!  be modified manually. In order to make changes to this module rerun
!  the Python script
!
!  tool/tens_rec/tens.py
!
! USES
!
!  * precision_golem (src/module/precision_golem.f90)
!  * matrice_s (src/kinematics/matrice_s.f90)
!  * array (src/module/array.f90)
!  * form_factor_type (src/module/form_factor_type.f90)
!  * form_factor_2p (src/form_factor/form_factor_2p.f90)
!  * form_factor_3p (src/form_factor/form_factor_3p.f90)
!  * form_factor_4p (src/form_factor/form_factor_4p.f90)
!  * form_factor_5p (src/form_factor/form_factor_5p.f90)
!  * form_factor_6p (src/form_factor/form_factor_6p.f90)
!  * form_factor_higher_ranks (src/higherrank/form_factor_higher_ranks.f90)
!  * tens_rec (src/interface/tens_rec.f90)
!
!*****
module tens_comb
! This module has been generated using a script.
! Please, refrain from modifying it directly!
use precision_golem, only: ki
use matrice_s, only: b_ref, inv_s
use array, only: packb, unpackb, countb, pminus, punion
use form_factor_type, only: form_factor, operator(+), operator(-), &
    & operator(*), assignment(=)
use form_factor_1p, only: a10
use form_factor_2p, only: a20, a21, a22, b22
use form_factor_3p, only: a30, a31, a32, b32, a33, b33
use form_factor_4p, only: a40, a41, a42, b42, a43, b43, a44, b44, c44
use form_factor_5p, only: a50, a51, a52, b52, a53, b53, a54, b54, c54, a55, &
   & b55, c55
use form_factor_6p, only: a60, a61, a62, a63, a64, a65, a66
use form_factor_higher_ranks
use tens_rec, only: coeff_type_1, reconstruct1, coeff_type_2, reconstruct2, &
   & coeff_type_3, reconstruct3, coeff_type_4, reconstruct4, coeff_type_5, &
   & reconstruct5, coeff_type_6, reconstruct6, coeff_type_7, reconstruct7
implicit none
private :: ki, b_ref, unpackb, packb, pminus, form_factor
private :: coeff_type_1, reconstruct1, coeff_type_2, reconstruct2, &
   & coeff_type_3, reconstruct3, coeff_type_4, reconstruct4, coeff_type_5, &
   & reconstruct5, coeff_type_6, reconstruct6, coeff_type_7, reconstruct7
private :: a10
private :: a20, a21, a22, b22
private :: a30, a31, a32, b32, a33, b33
private :: a40, a41, a42, b42, a43, b43, a44, b44, c44
private :: a50, a51, a52, b52, a53, b53, a54, b54, c54, a55, b55, c55
private :: a60, a61, a62, a63, a64, a65, a66
real(ki), dimension(0:3), parameter, private :: null_vec = &
 & (/0.0_ki,0.0_ki,0.0_ki,0.0_ki/)
interface evaluate
   module procedure evaluate_b
   module procedure evaluate_s
end interface
private :: test_if_nonzero
contains


function test_if_nonzero (val)
   complex(ki),intent(in):: val
   logical :: test_if_nonzero
   test_if_nonzero= (real(val,ki)*real(val,ki) + aimag(val)*aimag(val))>1E-100_ki
end function test_if_nonzero


!****f* src/interface/tens_comb/evaluate_s
! NAME
!
! Function evaluate_s
!
! Accessible through interface evaluate
!
! USAGE
!
!  amp = evaluate(numeval, momenta, set, rank)
!
! DESCRIPTION
!
!  Evaluates a diagram by first reconstructing its tensor coefficients
!  and then contracting with the tensor integrals.
!
! INPUTS
!
!  * numeval -- the numerator function
!  * momenta -- real array of dimension(:,0:3) containing the
!               momenta r_i of the loop propagators
!  * set     -- the set of pinched propagators as integer array
!  * rank    -- optional integer, specifying the maximum rank of the diagram
!               If omitted, rank is assumed to be the equal to the number of
!               loop propagators.
! RETURN VALUE
!
!  Value of the diagram as a type(form_factor)
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     evaluate_s(numeval, momenta, set, rank) result(amp)
   ! generated by: write_function_evaluate
   implicit none
   interface
      function numeval(Q, mu2)
         use precision_golem, only: ki
         implicit none
         real(ki), dimension(0:3), intent(in) :: Q
         real(ki), intent(in) :: mu2
         complex(ki) :: numeval
      end function numeval
   end interface
   real(ki), dimension(:,:), intent(in) :: momenta
   integer, dimension(:), intent(in) :: set
   integer, intent(in), optional :: rank
   type(form_factor) :: amp
   if (present(rank)) then
      amp = evaluate_b(numeval, momenta, packb(set), rank)
   else
      amp = evaluate_b(numeval, momenta, packb(set))
   end if
end function evaluate_s
!****f* src/interface/tens_comb/evaluate_b
! NAME
!
! Function evaluate_b
!
! Accessible through interface evaluate
!
! USAGE
!
!  amp = evaluate(numeval, momenta, b_set, rank)
!
! DESCRIPTION
!
!  Evaluates a diagram by first reconstructing its tensor coefficients
!  and then contracting with the tensor integrals.
!
! INPUTS
!
!  * numeval -- the numerator function
!  * momenta -- real array of dimension(:,0:3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer (bit-set)
!  * rank    -- optional integer, specifying the maximum rank of the diagram
!               If omitted, rank is assumed to be the equal to the number of
!               loop propagators.
! RETURN VALUE
!
!  Value of the diagram as a type(form_factor)
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     evaluate_b(numeval, momenta, b_set, rank) result(amp)
   implicit none
   interface
      function numeval(Q, mu2)
         use precision_golem, only: ki
         implicit none
         real(ki), dimension(0:3), intent(in) :: Q
         real(ki), intent(in) :: mu2
         complex(ki) :: numeval
      end function numeval
   end interface
   real(ki), dimension(:,:), intent(in) :: momenta
   integer, intent(in) :: b_set
   integer, intent(in), optional :: rank
   type(form_factor) :: amp
   integer :: N, r
   complex(ki) :: coeffs0
   type(coeff_type_1) :: coeffs1, coeffs1x, coeffs1xx
   type(coeff_type_2) :: coeffs2, coeffs2x, coeffs2xx
   type(coeff_type_3) :: coeffs3, coeffs3x, coeffs3xx
   type(coeff_type_4) :: coeffs4, coeffs4x, coeffs4xx
   type(coeff_type_5) :: coeffs5, coeffs5x, coeffs5xx
   type(coeff_type_6) :: coeffs6, coeffs6x, coeffs6xx
   type(coeff_type_7) :: coeffs7, coeffs7x, coeffs7xx
   N = size(momenta,1) - countb(b_set)
   if (present(rank)) then
      r = rank
   else
      r = N
   end if
   select case(N)
   case(1)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         print*, "Tadpoles not implemented yet"
         stop
      case(1)
         call reconstruct1(numeval, coeffs1)
         print*, "Tadpoles not implemented yet"
         stop
      case(2)
         call reconstruct2(numeval, coeffs2, coeffs0)
         print*, "Tadpoles not implemented yet"
         stop
      case default
         print*, "Not yet implemented: N, r = ", 1, r
         stop
      end select
   case(2)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         amp = coeffs0 * a20(b_set)
      case(1)
         call reconstruct1(numeval, coeffs1)
         amp = contract2_1(coeffs1,momenta,b_set)
      case(2)
         call reconstruct2(numeval, coeffs2, coeffs0)
         amp = contract2_2(coeffs2,momenta,b_set)
         amp = amp + contract2_2s1(coeffs0,momenta,b_set)
      case(3)
         call reconstruct3(numeval, coeffs3, coeffs1)
         amp = contract2_3(coeffs3,momenta,b_set)
         amp = amp + contract2_3s1(coeffs1,momenta,b_set)
      case default
         print*, "Not yet implemented: N, r = ", 2, r
         stop
      end select
   case(3)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         amp = coeffs0 * a30(b_set)
      case(1)
         call reconstruct1(numeval, coeffs1)
         amp = contract3_1(coeffs1,momenta,b_set)
      case(2)
         call reconstruct2(numeval, coeffs2, coeffs0)
         amp = contract3_2(coeffs2,momenta,b_set)
         amp = amp + contract3_2s1(coeffs0,momenta,b_set)
      case(3)
         call reconstruct3(numeval, coeffs3, coeffs1)
         amp = contract3_3(coeffs3,momenta,b_set)
         amp = amp + contract3_3s1(coeffs1,momenta,b_set)
      case(4)
         call reconstruct4(numeval, coeffs4, coeffs2, coeffs2x)
         amp = contract3_4(coeffs4,momenta,b_set)
         amp = amp + contract3_4s1(coeffs2,momenta,b_set)
         amp = amp + contract3_4s2(coeffs2x,momenta,b_set)
      case default
         print*, "Not yet implemented: N, r = ", 3, r
         stop
      end select
   case(4)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         amp = coeffs0 * a40(b_set)
      case(1)
         call reconstruct1(numeval, coeffs1)
         amp = contract4_1(coeffs1,momenta,b_set)
      case(2)
         call reconstruct2(numeval, coeffs2, coeffs0)
         amp = contract4_2(coeffs2,momenta,b_set)
         amp = amp + contract4_2s1(coeffs0,momenta,b_set)
      case(3)
         call reconstruct3(numeval, coeffs3, coeffs1)
         amp = contract4_3(coeffs3,momenta,b_set)
         amp = amp + contract4_3s1(coeffs1,momenta,b_set)
      case(4)
         call reconstruct4(numeval, coeffs4, coeffs2, coeffs2x)
         amp = contract4_4(coeffs4,momenta,b_set)
         amp = amp + contract4_4s1(coeffs2,momenta,b_set)
         amp = amp + contract4_4s2(coeffs2x,momenta,b_set)
      case(5)
         call reconstruct5(numeval, coeffs5, coeffs3, coeffs3x)
         amp = contract4_5(coeffs5,momenta,b_set)
         amp = amp + contract4_5s1(coeffs3,momenta,b_set)
         amp = amp + contract4_5s2(coeffs3x,momenta,b_set)
      case default
         print*, "Not yet implemented: N, r = ", 4, r
         stop
      end select
   case(5)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         amp = coeffs0 * a50(b_set)
      case(1)
         call reconstruct1(numeval, coeffs1)
         amp = contract5_1(coeffs1,momenta,b_set)
      case(2)
         call reconstruct2(numeval, coeffs2)
         amp = contract5_2(coeffs2,momenta,b_set)
      case(3)
         call reconstruct3(numeval, coeffs3)
         amp = contract5_3(coeffs3,momenta,b_set)
      case(4)
         call reconstruct4(numeval, coeffs4)
         amp = contract5_4(coeffs4,momenta,b_set)
      case(5)
         call reconstruct5(numeval, coeffs5)
         amp = contract5_5(coeffs5,momenta,b_set)
      case(6)
         call reconstruct6(numeval, coeffs6, coeffs4, coeffs4x, coeffs4xx)
         amp = contract5_6(coeffs6,momenta,b_set)
         amp = amp + contract5_6s1(coeffs4,momenta,b_set)
         amp = amp + contract5_6s2(coeffs4x,momenta,b_set)
         amp = amp + contract5_6s3(coeffs4xx,momenta,b_set)
      case default
         print*, "Not yet implemented: N, r = ", 5, r
         stop
      end select
   case(6)
      select case(r)
      case(0)
         coeffs0 = numeval(null_vec, 0.0_ki)
         amp = coeffs0 * a60(b_set)
      case(1)
         call reconstruct1(numeval, coeffs1)
         amp = contract6_1(coeffs1,momenta,b_set)
      case(2)
         call reconstruct2(numeval, coeffs2)
         amp = contract6_2(coeffs2,momenta,b_set)
      case(3)
         call reconstruct3(numeval, coeffs3)
         amp = contract6_3(coeffs3,momenta,b_set)
      case(4)
         call reconstruct4(numeval, coeffs4)
         amp = contract6_4(coeffs4,momenta,b_set)
      case(5)
         call reconstruct5(numeval, coeffs5)
         amp = contract6_5(coeffs5,momenta,b_set)
      case(6)
         call reconstruct6(numeval, coeffs6)
         amp = contract6_6(coeffs6,momenta,b_set)
      case default
         print*, "Not yet implemented: N, r = ", 6, r
         stop
      end select
   case default
      print*, "Not yet implemented: N=", N
      stop
   end select
end function evaluate_b
!****f* src/interface/tens_comb/contract1_1
! NAME
!
! Function contract1_1
!
! USAGE
!
!  amp = contract1_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 1-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract1_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1
   complex(ki) :: tmp
   integer, dimension(1) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 1)
   l1 = unpinched(1)
   tmp = coeffs%c0
   amp = tmp * a10(b_set)
end function contract1_1
!****f* src/interface/tens_comb/contract1_2
! NAME
!
! Function contract1_2
!
! USAGE
!
!  amp = contract1_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 1-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract1_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1
   complex(ki) :: tmp
   integer, dimension(1) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 1)
   l1 = unpinched(1)
   tmp = coeffs%c0
   amp = tmp * a10(b_set)
end function contract1_2
!****f* src/interface/tens_comb/contract1_2s1
! NAME
!
! Function contract1_2s1
!
! USAGE
!
!  amp = contract1_2s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 1-point rank 2 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(complex(ki))
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract1_2s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   complex(ki), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1
   complex(ki) :: tmp
   integer, dimension(1) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 1)
   l1 = unpinched(1)
   amp = 0.0_ki
end function contract1_2s1
!****f* src/interface/tens_comb/contract2_1
! NAME
!
! Function contract2_1
!
! USAGE
!
!  amp = contract2_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 2-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract2_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2
   complex(ki) :: tmp
   integer, dimension(2) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 2)
   l1 = unpinched(1)
   l2 = unpinched(2)
   tmp = coeffs%c0
   amp = tmp * a20(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l2, b_set)
   end if
end function contract2_1
!****f* src/interface/tens_comb/contract2_2
! NAME
!
! Function contract2_2
!
! USAGE
!
!  amp = contract2_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 2-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract2_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2
   complex(ki) :: tmp
   integer, dimension(2) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 2)
   l1 = unpinched(1)
   l2 = unpinched(2)
   tmp = coeffs%c0
   amp = tmp * a20(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l2, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a22(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a22(l1, l2, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a22(l2, l2, b_set)
   end if
   tmp = contract_b_tensor_2(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b22(b_set)
   end if
end function contract2_2
!****f* src/interface/tens_comb/contract2_2s1
! NAME
!
! Function contract2_2s1
!
! USAGE
!
!  amp = contract2_2s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 2-point rank 2 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(complex(ki))
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract2_2s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   complex(ki), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1, l2
   complex(ki) :: tmp
   integer, dimension(2) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 2)
   l1 = unpinched(1)
   l2 = unpinched(2)
   tmp = coeffs
   amp = tmp * b22(b_set)
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract2_2s1
!****f* src/interface/tens_comb/contract2_3
! NAME
!
! Function contract2_3
!
! USAGE
!
!  amp = contract2_3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 2-point rank 3 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract2_3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   integer :: l1, l2
   complex(ki) :: tmp
   integer, dimension(2) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 2)
   l1 = unpinched(1)
   l2 = unpinched(2)
   tmp = coeffs%c0
   amp = tmp * a20(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a21(l2, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a22(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a22(l1, l2, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a22(l2, l2, b_set)
   end if
   tmp = contract_b_tensor_3(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b22(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a23(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a23(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a23(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a23(l2, l2, l2, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b23(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b23(l2, b_set)
   end if
end function contract2_3
!****f* src/interface/tens_comb/contract2_3s1
! NAME
!
! Function contract2_3s1
!
! USAGE
!
!  amp = contract2_3s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 2-point rank 3 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract2_3s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2
   complex(ki) :: tmp
   integer, dimension(2) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 2)
   l1 = unpinched(1)
   l2 = unpinched(2)
   tmp = coeffs%c0
   amp = tmp * b22(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b23(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b23(l2, b_set)
   end if
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract2_3s1
!****f* src/interface/tens_comb/contract3_1
! NAME
!
! Function contract3_1
!
! USAGE
!
!  amp = contract3_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * a30(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l3, b_set)
   end if
end function contract3_1
!****f* src/interface/tens_comb/contract3_2
! NAME
!
! Function contract3_2
!
! USAGE
!
!  amp = contract3_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * a30(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l3, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l3, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l2, l3, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l3, l3, b_set)
   end if
   tmp = contract_b_tensor_2(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b32(b_set)
   end if
end function contract3_2
!****f* src/interface/tens_comb/contract3_2s1
! NAME
!
! Function contract3_2s1
!
! USAGE
!
!  amp = contract3_2s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 2 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(complex(ki))
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_2s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   complex(ki), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs
   amp = tmp * b32(b_set)
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract3_2s1
!****f* src/interface/tens_comb/contract3_3
! NAME
!
! Function contract3_3
!
! USAGE
!
!  amp = contract3_3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 3 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * a30(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l3, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l3, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l2, l3, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l3, l3, b_set)
   end if
   tmp = contract_b_tensor_3(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b32(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a33(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l3, l3, l3, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l3, b_set)
   end if
end function contract3_3
!****f* src/interface/tens_comb/contract3_3s1
! NAME
!
! Function contract3_3s1
!
! USAGE
!
!  amp = contract3_3s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 3 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_3s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * b32(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l3, b_set)
   end if
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract3_3s1
!****f* src/interface/tens_comb/contract3_4
! NAME
!
! Function contract3_4
!
! USAGE
!
!  amp = contract3_4(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 4 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_4(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * a30(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a31(l3, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l1, l3, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a32(l2, l3, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a32(l3, l3, b_set)
   end if
   tmp = contract_b_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b32(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a33(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a33(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a33(l3, l3, l3, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a34(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a34(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a34(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a34(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a34(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a34(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a34(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a34(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a34(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a34(l3, l3, l3, l3, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l1, l3, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l2, l3, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l3, l3, b_set)
   end if
   tmp = contract_c_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c34(b_set)
   end if
end function contract3_4
!****f* src/interface/tens_comb/contract3_4s1
! NAME
!
! Function contract3_4s1
!
! USAGE
!
!  amp = contract3_4s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 4 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_4s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * b32(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b33(l3, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l1, l3, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b34(l2, l3, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b34(l3, l3, b_set)
   end if
   tmp = contract_b_tensor_2(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c34(b_set)
   end if
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract3_4s1
!****f* src/interface/tens_comb/contract3_4s2
! NAME
!
! Function contract3_4s2
!
! USAGE
!
!  amp = contract3_4s2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 3-point rank 4 tensor integral
!  with (mu^2)^2 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract3_4s2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1, l2, l3
   complex(ki) :: tmp
   integer, dimension(3) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 3)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   tmp = coeffs%c0
   amp = tmp * c34(b_set)
   ! multiply by -4*(epsilon-epsilon^2)
   amp%c = -4.0_ki*(amp%b-amp%a)
   amp%b = -4.0_ki*amp%a
   amp%a = 0.0_ki
end function contract3_4s2
!****f* src/interface/tens_comb/contract4_1
! NAME
!
! Function contract4_1
!
! USAGE
!
!  amp = contract4_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * a40(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l4, b_set)
   end if
end function contract4_1
!****f* src/interface/tens_comb/contract4_2
! NAME
!
! Function contract4_2
!
! USAGE
!
!  amp = contract4_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * a40(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l4, l4, b_set)
   end if
   tmp = contract_b_tensor_2(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b42(b_set)
   end if
end function contract4_2
!****f* src/interface/tens_comb/contract4_2s1
! NAME
!
! Function contract4_2s1
!
! USAGE
!
!  amp = contract4_2s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 2 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(complex(ki))
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_2s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   complex(ki), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   amp = 0.0_ki
end function contract4_2s1
!****f* src/interface/tens_comb/contract4_3
! NAME
!
! Function contract4_3
!
! USAGE
!
!  amp = contract4_3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 3 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * a40(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l4, l4, b_set)
   end if
   tmp = contract_b_tensor_3(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b42(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l4, l4, l4, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l4, b_set)
   end if
end function contract4_3
!****f* src/interface/tens_comb/contract4_3s1
! NAME
!
! Function contract4_3s1
!
! USAGE
!
!  amp = contract4_3s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 3 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_3s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   amp = 0.0_ki
end function contract4_3s1
!****f* src/interface/tens_comb/contract4_4
! NAME
!
! Function contract4_4
!
! USAGE
!
!  amp = contract4_4(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 4 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_4(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * a40(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l4, l4, b_set)
   end if
   tmp = contract_b_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b42(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l4, l4, l4, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a44(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l4, l4, l4, l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l4, l4, b_set)
   end if
   tmp = contract_c_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c44(b_set)
   end if
end function contract4_4
!****f* src/interface/tens_comb/contract4_4s1
! NAME
!
! Function contract4_4s1
!
! USAGE
!
!  amp = contract4_4s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 4 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_4s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = contract_b_tensor_2(coeffs)
   amp = tmp * c44(b_set)
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract4_4s1
!****f* src/interface/tens_comb/contract4_4s2
! NAME
!
! Function contract4_4s2
!
! USAGE
!
!  amp = contract4_4s2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 4 tensor integral
!  with (mu^2)^2 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_4s2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * c44(b_set)
   ! multiply by -4*(epsilon-epsilon^2)
   amp%c = -4.0_ki*(amp%b-amp%a)
   amp%b = -4.0_ki*amp%a
   amp%a = 0.0_ki
end function contract4_4s2
!****f* src/interface/tens_comb/contract4_5
! NAME
!
! Function contract4_5
!
! USAGE
!
!  amp = contract4_5(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 5 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_5(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   real(ki), dimension(5,0:3) :: mom5
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * a40(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a41(l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a42(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a42(l4, l4, b_set)
   end if
   tmp = contract_b_tensor_5(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b42(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a43(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a43(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a43(l4, l4, l4, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b43(l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a44(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a44(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a44(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a44(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a44(l4, l4, l4, l4, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l1, l4, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l2, l4, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b44(l3, l4, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b44(l4, l4, b_set)
   end if
   tmp = contract_c_tensor_5(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c44(b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a45(l1, l1, l1, l1, l1, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l1, l1, l1, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l1, l1, l1, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l1, l1, l1, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l1, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l1, l1, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l1, l1, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l1, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l1, l1, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l1, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a45(l1, l1, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l1, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l1, l1, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a45(l1, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a45(l1, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a45(l1, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l1, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l1, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l1, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a45(l2, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l2, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l2, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l2, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l2, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l2, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l2, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l2, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l2, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l2, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l2, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l2, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a45(l2, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a45(l2, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l2, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a45(l3, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l3, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l3, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a45(l3, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a45(l3, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a45(l4, l4, l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b45(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b45(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b45(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b45(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b45(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b45(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b45(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b45(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b45(l4, l4, l4, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l4, b_set)
   end if
end function contract4_5
!****f* src/interface/tens_comb/contract4_5s1
! NAME
!
! Function contract4_5s1
!
! USAGE
!
!  amp = contract4_5s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 5 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_5s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = contract_b_tensor_3(coeffs)
   amp = tmp * c44(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l4, b_set)
   end if
   ! multiply by 2*epsilon
   amp%c = 2.0_ki*amp%b
   amp%b = 2.0_ki*amp%a
   amp%a = 0.0_ki
end function contract4_5s1
!****f* src/interface/tens_comb/contract4_5s2
! NAME
!
! Function contract4_5s2
!
! USAGE
!
!  amp = contract4_5s2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 4-point rank 5 tensor integral
!  with (mu^2)^2 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract4_5s2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3, l4
   complex(ki) :: tmp
   integer, dimension(4) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 4)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   tmp = coeffs%c0
   amp = tmp * c44(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c45(l4, b_set)
   end if
   ! multiply by -4*(epsilon-epsilon^2)
   amp%c = -4.0_ki*(amp%b-amp%a)
   amp%b = -4.0_ki*amp%a
   amp%a = 0.0_ki
end function contract4_5s2
!****f* src/interface/tens_comb/contract5_1
! NAME
!
! Function contract5_1
!
! USAGE
!
!  amp = contract5_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_1(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
end function contract5_1
!****f* src/interface/tens_comb/contract5_2
! NAME
!
! Function contract5_2
!
! USAGE
!
!  amp = contract5_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_a_tensor_2(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l5, l5, b_set)
   end if
   tmp = contract_b_tensor_2(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b52(b_set)
   end if
end function contract5_2
!****f* src/interface/tens_comb/contract5_3
! NAME
!
! Function contract5_3
!
! USAGE
!
!  amp = contract5_3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 3 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l5, l5, b_set)
   end if
   tmp = contract_b_tensor_3(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b52(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_a_tensor_3(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_b_tensor_3(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l5, b_set)
   end if
end function contract5_3
!****f* src/interface/tens_comb/contract5_4
! NAME
!
! Function contract5_4
!
! USAGE
!
!  amp = contract5_4(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 4 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_4(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l5, l5, b_set)
   end if
   tmp = contract_b_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b52(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l2, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l4, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l4, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_4(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l5, l5, l5, l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_b_tensor_4(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l5, l5, b_set)
   end if
   tmp = contract_c_tensor_4(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c54(b_set)
   end if
end function contract5_4
!****f* src/interface/tens_comb/contract5_5
! NAME
!
! Function contract5_5
!
! USAGE
!
!  amp = contract5_5(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 5 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_5(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   real(ki), dimension(5,0:3) :: mom5
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l5, l5, b_set)
   end if
   tmp = contract_b_tensor_5(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b52(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l2, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l4, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l4, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l5, l5, l5, l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l5, l5, b_set)
   end if
   tmp = contract_c_tensor_5(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c54(b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l1, l1, l1, l1, l1, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a55(l1, l2, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l2, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l2, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l3, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l4, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l4, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l4, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l4, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l4, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_5(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l5, l5, l5, l5, l5, b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_b_tensor_5(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_c_tensor_5(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l5, b_set)
   end if
end function contract5_5
!****f* src/interface/tens_comb/contract5_6
! NAME
!
! Function contract5_6
!
! USAGE
!
!  amp = contract5_6(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 6 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_6(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   real(ki), dimension(1,0:3) :: mom1
   real(ki), dimension(2,0:3) :: mom2
   real(ki), dimension(3,0:3) :: mom3
   real(ki), dimension(4,0:3) :: mom4
   real(ki), dimension(5,0:3) :: mom5
   real(ki), dimension(6,0:3) :: mom6
   integer :: l1, l2, l3, l4, l5
   complex(ki) :: tmp
   integer, dimension(5) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 5)
   l1 = unpinched(1)
   l2 = unpinched(2)
   l3 = unpinched(3)
   l4 = unpinched(4)
   l5 = unpinched(5)
   tmp = coeffs%c0
   amp = tmp * a50(b_set)
   mom1 = momenta((/l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a51(l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * a52(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a52(l5, l5, b_set)
   end if
   tmp = contract_b_tensor_6(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b52(b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a53(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * a53(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a53(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_b_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b53(l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l1, l1, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l1, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l1, l1, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l1, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l1, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l1, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l2, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * a54(l2, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l2, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l2, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l3, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * a54(l3, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l3, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l4, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a54(l4, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * a54(l4, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a54(l5, l5, l5, l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * b54(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b54(l5, l5, b_set)
   end if
   tmp = contract_c_tensor_6(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c54(b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l1, l1, l1, l1, l1, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l1, l1, l1, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l1, l1, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l1, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l1, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l1, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l1, l1, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l2, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a55(l1, l2, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l2, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l2, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l1, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l1, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l1, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l1,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l1, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l2, l2, l2, l2, l2, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l2, l2, l2, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l2, l2, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l2, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l2, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l2, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l2, l2, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a55(l2, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l2, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l2, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l2,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l2, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l3, l3, l3, l3, l3, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l3, l3, l3, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l3, l3, l3, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l3, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l3, l3, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l3, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l3, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l3, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l3, l3, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a55(l3, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a55(l3, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l3,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l3, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l4, l4, l4, l4, l4, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l4, l4, l4, l4, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l4, l4, l4, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 10.0_ki * tmp * a55(l4, l4, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + 5.0_ki * tmp * a55(l4, l5, l5, l5, l5, b_set)
   end if
   mom5 = momenta((/l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom5)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a55(l5, l5, l5, l5, l5, b_set)
   end if
   mom3 = momenta((/l1,l1,l1/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l1, l1, l1, b_set)
   end if
   mom3 = momenta((/l1,l1,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l2, b_set)
   end if
   mom3 = momenta((/l1,l1,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l3, b_set)
   end if
   mom3 = momenta((/l1,l1,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l4, b_set)
   end if
   mom3 = momenta((/l1,l1,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l1, l5, b_set)
   end if
   mom3 = momenta((/l1,l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l2, l2, b_set)
   end if
   mom3 = momenta((/l1,l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l3, b_set)
   end if
   mom3 = momenta((/l1,l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l4, b_set)
   end if
   mom3 = momenta((/l1,l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l2, l5, b_set)
   end if
   mom3 = momenta((/l1,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l3, l3, b_set)
   end if
   mom3 = momenta((/l1,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l3, l4, b_set)
   end if
   mom3 = momenta((/l1,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l3, l5, b_set)
   end if
   mom3 = momenta((/l1,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l4, l4, b_set)
   end if
   mom3 = momenta((/l1,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l1, l4, l5, b_set)
   end if
   mom3 = momenta((/l1,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l1, l5, l5, b_set)
   end if
   mom3 = momenta((/l2,l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l2, l2, l2, b_set)
   end if
   mom3 = momenta((/l2,l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l3, b_set)
   end if
   mom3 = momenta((/l2,l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l4, b_set)
   end if
   mom3 = momenta((/l2,l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l2, l5, b_set)
   end if
   mom3 = momenta((/l2,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l3, l3, b_set)
   end if
   mom3 = momenta((/l2,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l3, l4, b_set)
   end if
   mom3 = momenta((/l2,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l3, l5, b_set)
   end if
   mom3 = momenta((/l2,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l4, l4, b_set)
   end if
   mom3 = momenta((/l2,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l2, l4, l5, b_set)
   end if
   mom3 = momenta((/l2,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l2, l5, l5, b_set)
   end if
   mom3 = momenta((/l3,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l3, l3, l3, b_set)
   end if
   mom3 = momenta((/l3,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l3, l4, b_set)
   end if
   mom3 = momenta((/l3,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l3, l5, b_set)
   end if
   mom3 = momenta((/l3,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l4, l4, b_set)
   end if
   mom3 = momenta((/l3,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b55(l3, l4, l5, b_set)
   end if
   mom3 = momenta((/l3,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l3, l5, l5, b_set)
   end if
   mom3 = momenta((/l4,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l4, l4, l4, b_set)
   end if
   mom3 = momenta((/l4,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l4, l4, l5, b_set)
   end if
   mom3 = momenta((/l4,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + 3.0_ki * tmp * b55(l4, l5, l5, b_set)
   end if
   mom3 = momenta((/l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom3)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b55(l5, l5, l5, b_set)
   end if
   mom1 = momenta((/l1/),:)
   tmp = contract_c_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l1, b_set)
   end if
   mom1 = momenta((/l2/),:)
   tmp = contract_c_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l2, b_set)
   end if
   mom1 = momenta((/l3/),:)
   tmp = contract_c_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l3, b_set)
   end if
   mom1 = momenta((/l4/),:)
   tmp = contract_c_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l4, b_set)
   end if
   mom1 = momenta((/l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom1)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c55(l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l1,l1/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a56(l1, l1, l1, l1, l1, l1, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l1,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l1, l1, l1, l1, l2, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l1,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l1, l1, l1, l1, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l1,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l1, l1, l1, l1, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l1,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l1, l1, l1, l1, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l1, l1, l2, l2, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l2, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l2, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l2, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l1, l1, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l1, l1, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l1, l1, l1, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l1,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l1, l1, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l1, l1, l1, l2, l2, l2, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l2, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l2, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l2, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l1, l1, l2, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l1, l1, l2, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l1, l1, l2, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l2, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l1, l1, l1, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l1, l1, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l1, l1, l1, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l1, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l1,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l1, l1, l1, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l2, l2, l2, l2, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l2, l2, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l2, l2, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l2, l2, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l2, l2, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l2, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l2, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l2, l2, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l2, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l2, l2, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 360.0_ki * tmp * a56(l1, l1, l2, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l2, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l2, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l1, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l1,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l1, l1, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l1, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l1,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l1, l1, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l2, l2, l2, l2, l2, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l2, l2, l2, l3, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l2, l2, l2, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l2, l2, l2, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l2, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l2, l2, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l2, l2, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l2, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l2, l2, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l2, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 360.0_ki * tmp * a56(l1, l2, l2, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l2, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l2, l2, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 360.0_ki * tmp * a56(l1, l2, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 360.0_ki * tmp * a56(l1, l2, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 360.0_ki * tmp * a56(l1, l2, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l2,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l2, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l2, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l2,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l2, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l3, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l3, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l3, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l3, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l3, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l3, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l3, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l3, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l3, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l3, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l3, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l3,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l3, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l1, l3, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l1, l3, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l3,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l3, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l4, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l1,l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l4, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l1,l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l4, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l1, l4, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l1, l4, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l1,l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l1, l5, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l2,l2/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a56(l2, l2, l2, l2, l2, l2, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l2,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l2, l2, l2, l2, l3, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l2,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l2, l2, l2, l2, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l2,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l2, l2, l2, l2, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l2, l2, l3, l3, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l2, l2, l2, l3, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l2, l2, l2, l3, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l2, l2, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l2, l2, l2, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l2,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l2, l2, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l2, l2, l2, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l2, l2, l2, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l2, l2, l2, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l2, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l2,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l2, l2, l2, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l2, l2, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l2, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l2, l2, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l2, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l2, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l2,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l2, l2, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l2, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l2,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l2, l2, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l3, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l3, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l3, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l3, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l2, l3, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l3, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l3, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l3, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l3, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l3, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l3, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l3,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l2, l3, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 180.0_ki * tmp * a56(l2, l3, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 120.0_ki * tmp * a56(l2, l3, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l3,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l3, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l4, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l2,l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l4, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l2,l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l4, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l2, l4, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l2, l4, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l2,l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l2, l5, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l3,l3/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a56(l3, l3, l3, l3, l3, l3, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l3,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l3, l3, l3, l3, l3, l4, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l3,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l3, l3, l3, l3, l3, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l3, l3, l3, l3, l4, l4, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l3, l3, l3, l3, l4, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l3,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l3, l3, l3, l3, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l3, l3, l3, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l3, l3, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l3, l3, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l3,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l3, l3, l3, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l3, l3, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l3,l3,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l3, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 90.0_ki * tmp * a56(l3, l3, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l3, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l3,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l3, l3, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l3, l4, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l3,l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l3, l4, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l3,l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l4, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 60.0_ki * tmp * a56(l3, l4, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 30.0_ki * tmp * a56(l3, l4, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l3,l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l3, l5, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l4,l4,l4,l4,l4,l4/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a56(l4, l4, l4, l4, l4, l4, b_set)
   end if
   mom6 = momenta((/l4,l4,l4,l4,l4,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l4, l4, l4, l4, l4, l5, b_set)
   end if
   mom6 = momenta((/l4,l4,l4,l4,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l4, l4, l4, l4, l5, l5, b_set)
   end if
   mom6 = momenta((/l4,l4,l4,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 20.0_ki * tmp * a56(l4, l4, l4, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l4,l4,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 15.0_ki * tmp * a56(l4, l4, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l4,l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * a56(l4, l5, l5, l5, l5, l5, b_set)
   end if
   mom6 = momenta((/l5,l5,l5,l5,l5,l5/),:)
   tmp = contract_a_tensor_6(coeffs, mom6)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * a56(l5, l5, l5, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l1/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b56(l1, l1, l1, l1, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l1, l1, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l1, l1, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l1, l1, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l1,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l1, l1, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l1, l1, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l1, l1, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l1, l1, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l1,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l1, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l1,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l1, l1, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * b56(l1, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * b56(l1, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l2,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * b56(l1, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l2,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l3,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * b56(l1, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l3,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l1,l4,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l1,l4,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l1, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l1,l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l1, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l2/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b56(l2, l2, l2, l2, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l2, l2, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l2, l2, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l2,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l2, l2, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l2, l2, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l2, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l2, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l2, l2, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l2,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l2, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l2,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l2, l2, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l3,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 24.0_ki * tmp * b56(l2, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l3,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l2,l4,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l2,l4,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l2, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l2,l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l2, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l3/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b56(l3, l3, l3, l3, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l3, l3, l3, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l3,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l3, l3, l3, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l3, l3, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l3,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l3, l3, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l3,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l3, l3, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l3, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l3,l4,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l3, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l3,l4,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 12.0_ki * tmp * b56(l3, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l3,l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l3, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l4/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b56(l4, l4, l4, l4, b_set)
   end if
   mom4 = momenta((/l4,l4,l4,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l4, l4, l4, l5, b_set)
   end if
   mom4 = momenta((/l4,l4,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 6.0_ki * tmp * b56(l4, l4, l5, l5, b_set)
   end if
   mom4 = momenta((/l4,l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + 4.0_ki * tmp * b56(l4, l5, l5, l5, b_set)
   end if
   mom4 = momenta((/l5,l5,l5,l5/),:)
   tmp = contract_b_tensor_6(coeffs, mom4)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * b56(l5, l5, l5, l5, b_set)
   end if
   mom2 = momenta((/l1,l1/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c56(l1, l1, b_set)
   end if
   mom2 = momenta((/l1,l2/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l1, l2, b_set)
   end if
   mom2 = momenta((/l1,l3/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l1, l3, b_set)
   end if
   mom2 = momenta((/l1,l4/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l1, l4, b_set)
   end if
   mom2 = momenta((/l1,l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l1, l5, b_set)
   end if
   mom2 = momenta((/l2,l2/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c56(l2, l2, b_set)
   end if
   mom2 = momenta((/l2,l3/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l2, l3, b_set)
   end if
   mom2 = momenta((/l2,l4/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l2, l4, b_set)
   end if
   mom2 = momenta((/l2,l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l2, l5, b_set)
   end if
   mom2 = momenta((/l3,l3/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c56(l3, l3, b_set)
   end if
   mom2 = momenta((/l3,l4/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l3, l4, b_set)
   end if
   mom2 = momenta((/l3,l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l3, l5, b_set)
   end if
   mom2 = momenta((/l4,l4/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c56(l4, l4, b_set)
   end if
   mom2 = momenta((/l4,l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + 2.0_ki * tmp * c56(l4, l5, b_set)
   end if
   mom2 = momenta((/l5,l5/),:)
   tmp = contract_c_tensor_6(coeffs, mom2)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * c56(l5, l5, b_set)
   end if
   tmp = contract_d_tensor_6(coeffs)
   if (test_if_nonzero(tmp)) then
    amp = amp + tmp * d56(b_set)
   end if
end function contract5_6
!****f* src/interface/tens_comb/contract5_6s1
! NAME
!
! Function contract5_6s1
!
! USAGE
!
!  amp = contract5_6s1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 6 tensor integral
!  with (mu^2)^1 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_6s1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   amp%c = 1./64._ki * ( coeffs%c1(1,4) + coeffs%c1(2,4) + &
             & coeffs%c1(3,4) + coeffs%c1(4,4) - coeffs%c2(1,5) - coeffs%c2(2,5) - coeffs%c2(3,5) &
             &  + coeffs%c2(4,5) + coeffs%c2(5,5) + coeffs%c2(6,5) )
   amp%b = 0.0_ki
   amp%a = 0.0_ki
end function contract5_6s1
!****f* src/interface/tens_comb/contract5_6s2
! NAME
!
! Function contract5_6s2
!
! USAGE
!
!  amp = contract5_6s2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 6 tensor integral
!  with (mu^2)^2 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_6s2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   amp%c = - 1./48._ki * ( coeffs%c1(1,2) - coeffs%c1(2,2) - &
                         & coeffs%c1(3,2) - coeffs%c1(4,2) )
   amp%b = 0.0_ki
   amp%a = 0.0_ki
end function contract5_6s2
!****f* src/interface/tens_comb/contract5_6s3
! NAME
!
! Function contract5_6s3
!
! USAGE
!
!  amp = contract5_6s3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 5-point rank 6 tensor integral
!  with (mu^2)^3 in the numerator with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract5_6s3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_simple
   amp%c =  coeffs%c0 * 1._ki/12._ki
   amp%b = 0.0_ki
   amp%a = 0.0_ki
end function contract5_6s3
!****f* src/interface/tens_comb/contract6_1
! NAME
!
! Function contract6_1
!
! USAGE
!
!  amp = contract6_1(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 1 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_1(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   complex(ki) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime = cprime + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime = cprime + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime = cprime + C(3) * coeffs%c1(4, 1)
      amp = amp - cprime * a50(new_set)
   end do
end function contract6_1
!****f* src/interface/tens_comb/contract6_2
! NAME
!
! Function contract6_2
!
! USAGE
!
!  amp = contract6_2(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 2 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_2(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   type(coeff_type_1) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime%c0 = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime%c0 = cprime%c0 + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime%c0 = cprime%c0 + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime%c0 = cprime%c0 + C(3) * coeffs%c1(4, 1)
      ! [0] <-- [0, 0]
      cprime%c1(1,1) = C(0) * coeffs%c1(1, 2)
      ! [1] <-- [1, 1]
      cprime%c1(2,1) = C(1) * coeffs%c1(2, 2)
      ! [2] <-- [2, 2]
      cprime%c1(3,1) = C(2) * coeffs%c1(3, 2)
      ! [3] <-- [3, 3]
      cprime%c1(4,1) = C(3) * coeffs%c1(4, 2)
      ! [1] <-- [0, 1]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(0) * coeffs%c2(1, 1)
      ! [0] <-- [0, 1]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(1) * coeffs%c2(1, 1)
      ! [2] <-- [0, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(0) * coeffs%c2(2, 1)
      ! [0] <-- [0, 2]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(2) * coeffs%c2(2, 1)
      ! [3] <-- [0, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(0) * coeffs%c2(3, 1)
      ! [0] <-- [0, 3]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(3) * coeffs%c2(3, 1)
      ! [2] <-- [1, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(1) * coeffs%c2(4, 1)
      ! [1] <-- [1, 2]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(2) * coeffs%c2(4, 1)
      ! [3] <-- [1, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(1) * coeffs%c2(5, 1)
      ! [1] <-- [1, 3]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(3) * coeffs%c2(5, 1)
      ! [3] <-- [2, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(2) * coeffs%c2(6, 1)
      ! [2] <-- [2, 3]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(3) * coeffs%c2(6, 1)
      amp = amp - contract5_1(cprime, momenta, new_set)
   end do
end function contract6_2
!****f* src/interface/tens_comb/contract6_3
! NAME
!
! Function contract6_3
!
! USAGE
!
!  amp = contract6_3(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 3 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_3(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   type(coeff_type_2) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime%c0 = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime%c0 = cprime%c0 + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime%c0 = cprime%c0 + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime%c0 = cprime%c0 + C(3) * coeffs%c1(4, 1)
      ! [0] <-- [0, 0]
      cprime%c1(1,1) = C(0) * coeffs%c1(1, 2)
      ! [1] <-- [1, 1]
      cprime%c1(2,1) = C(1) * coeffs%c1(2, 2)
      ! [2] <-- [2, 2]
      cprime%c1(3,1) = C(2) * coeffs%c1(3, 2)
      ! [3] <-- [3, 3]
      cprime%c1(4,1) = C(3) * coeffs%c1(4, 2)
      ! [1] <-- [0, 1]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(0) * coeffs%c2(1, 1)
      ! [0] <-- [0, 1]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(1) * coeffs%c2(1, 1)
      ! [2] <-- [0, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(0) * coeffs%c2(2, 1)
      ! [0] <-- [0, 2]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(2) * coeffs%c2(2, 1)
      ! [3] <-- [0, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(0) * coeffs%c2(3, 1)
      ! [0] <-- [0, 3]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(3) * coeffs%c2(3, 1)
      ! [2] <-- [1, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(1) * coeffs%c2(4, 1)
      ! [1] <-- [1, 2]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(2) * coeffs%c2(4, 1)
      ! [3] <-- [1, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(1) * coeffs%c2(5, 1)
      ! [1] <-- [1, 3]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(3) * coeffs%c2(5, 1)
      ! [3] <-- [2, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(2) * coeffs%c2(6, 1)
      ! [2] <-- [2, 3]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(3) * coeffs%c2(6, 1)
      ! [1, 1] <-- [0, 1, 1]
      cprime%c1(2,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(1, 2)
      ! [0, 1] <-- [0, 1, 1]
      cprime%c2(1,1) = 2.0_ki/3.0_ki * C(1) * coeffs%c2(1, 2)
      ! [2, 2] <-- [0, 2, 2]
      cprime%c1(3,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(2, 2)
      ! [0, 2] <-- [0, 2, 2]
      cprime%c2(2,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(2, 2)
      ! [3, 3] <-- [0, 3, 3]
      cprime%c1(4,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(3, 2)
      ! [0, 3] <-- [0, 3, 3]
      cprime%c2(3,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(3, 2)
      ! [2, 2] <-- [1, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(4, 2)
      ! [1, 2] <-- [1, 2, 2]
      cprime%c2(4,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(4, 2)
      ! [3, 3] <-- [1, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(5, 2)
      ! [1, 3] <-- [1, 3, 3]
      cprime%c2(5,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(5, 2)
      ! [3, 3] <-- [2, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(6, 2)
      ! [2, 3] <-- [2, 3, 3]
      cprime%c2(6,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(6, 2)
      ! [1, 2] <-- [0, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 1)
      ! [0, 2] <-- [0, 1, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 1)
      ! [0, 1] <-- [0, 1, 2]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 1)
      ! [1, 3] <-- [0, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 1)
      ! [0, 3] <-- [0, 1, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 1)
      ! [0, 1] <-- [0, 1, 3]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 1)
      ! [2, 3] <-- [0, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 1)
      ! [0, 3] <-- [0, 2, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 1)
      ! [0, 2] <-- [0, 2, 3]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 1)
      ! [2, 3] <-- [1, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 1)
      ! [1, 3] <-- [1, 2, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 1)
      ! [1, 2] <-- [1, 2, 3]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 1)
      ! [0, 0] <-- [0, 0, 0]
      cprime%c1(1,2) = C(0) * coeffs%c1(1, 3)
      ! [1, 1] <-- [1, 1, 1]
      cprime%c1(2,2) = cprime%c1(2,2) + C(1) * coeffs%c1(2, 3)
      ! [2, 2] <-- [2, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + C(2) * coeffs%c1(3, 3)
      ! [3, 3] <-- [3, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + C(3) * coeffs%c1(4, 3)
      ! [0, 1] <-- [0, 0, 1]
      cprime%c2(1,1) = cprime%c2(1,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(1, 3)
      ! [0, 0] <-- [0, 0, 1]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(1, 3)
      ! [0, 2] <-- [0, 0, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(2, 3)
      ! [0, 0] <-- [0, 0, 2]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(2, 3)
      ! [0, 3] <-- [0, 0, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(3, 3)
      ! [0, 0] <-- [0, 0, 3]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(3, 3)
      ! [1, 2] <-- [1, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(4, 3)
      ! [1, 1] <-- [1, 1, 2]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(4, 3)
      ! [1, 3] <-- [1, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(5, 3)
      ! [1, 1] <-- [1, 1, 3]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(5, 3)
      ! [2, 3] <-- [2, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(6, 3)
      ! [2, 2] <-- [2, 2, 3]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(6, 3)
      amp = amp - contract5_2(cprime, momenta, new_set)
   end do
end function contract6_3
!****f* src/interface/tens_comb/contract6_4
! NAME
!
! Function contract6_4
!
! USAGE
!
!  amp = contract6_4(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 4 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_4(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   type(coeff_type_3) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime%c0 = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime%c0 = cprime%c0 + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime%c0 = cprime%c0 + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime%c0 = cprime%c0 + C(3) * coeffs%c1(4, 1)
      ! [0] <-- [0, 0]
      cprime%c1(1,1) = C(0) * coeffs%c1(1, 2)
      ! [1] <-- [1, 1]
      cprime%c1(2,1) = C(1) * coeffs%c1(2, 2)
      ! [2] <-- [2, 2]
      cprime%c1(3,1) = C(2) * coeffs%c1(3, 2)
      ! [3] <-- [3, 3]
      cprime%c1(4,1) = C(3) * coeffs%c1(4, 2)
      ! [1] <-- [0, 1]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(0) * coeffs%c2(1, 1)
      ! [0] <-- [0, 1]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(1) * coeffs%c2(1, 1)
      ! [2] <-- [0, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(0) * coeffs%c2(2, 1)
      ! [0] <-- [0, 2]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(2) * coeffs%c2(2, 1)
      ! [3] <-- [0, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(0) * coeffs%c2(3, 1)
      ! [0] <-- [0, 3]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(3) * coeffs%c2(3, 1)
      ! [2] <-- [1, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(1) * coeffs%c2(4, 1)
      ! [1] <-- [1, 2]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(2) * coeffs%c2(4, 1)
      ! [3] <-- [1, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(1) * coeffs%c2(5, 1)
      ! [1] <-- [1, 3]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(3) * coeffs%c2(5, 1)
      ! [3] <-- [2, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(2) * coeffs%c2(6, 1)
      ! [2] <-- [2, 3]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(3) * coeffs%c2(6, 1)
      ! [1, 1] <-- [0, 1, 1]
      cprime%c1(2,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(1, 2)
      ! [0, 1] <-- [0, 1, 1]
      cprime%c2(1,1) = 2.0_ki/3.0_ki * C(1) * coeffs%c2(1, 2)
      ! [2, 2] <-- [0, 2, 2]
      cprime%c1(3,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(2, 2)
      ! [0, 2] <-- [0, 2, 2]
      cprime%c2(2,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(2, 2)
      ! [3, 3] <-- [0, 3, 3]
      cprime%c1(4,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(3, 2)
      ! [0, 3] <-- [0, 3, 3]
      cprime%c2(3,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(3, 2)
      ! [2, 2] <-- [1, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(4, 2)
      ! [1, 2] <-- [1, 2, 2]
      cprime%c2(4,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(4, 2)
      ! [3, 3] <-- [1, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(5, 2)
      ! [1, 3] <-- [1, 3, 3]
      cprime%c2(5,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(5, 2)
      ! [3, 3] <-- [2, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(6, 2)
      ! [2, 3] <-- [2, 3, 3]
      cprime%c2(6,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(6, 2)
      ! [1, 2] <-- [0, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 1)
      ! [0, 2] <-- [0, 1, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 1)
      ! [0, 1] <-- [0, 1, 2]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 1)
      ! [1, 3] <-- [0, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 1)
      ! [0, 3] <-- [0, 1, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 1)
      ! [0, 1] <-- [0, 1, 3]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 1)
      ! [2, 3] <-- [0, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 1)
      ! [0, 3] <-- [0, 2, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 1)
      ! [0, 2] <-- [0, 2, 3]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 1)
      ! [2, 3] <-- [1, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 1)
      ! [1, 3] <-- [1, 2, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 1)
      ! [1, 2] <-- [1, 2, 3]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 1)
      ! [0, 0] <-- [0, 0, 0]
      cprime%c1(1,2) = C(0) * coeffs%c1(1, 3)
      ! [1, 1] <-- [1, 1, 1]
      cprime%c1(2,2) = cprime%c1(2,2) + C(1) * coeffs%c1(2, 3)
      ! [2, 2] <-- [2, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + C(2) * coeffs%c1(3, 3)
      ! [3, 3] <-- [3, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + C(3) * coeffs%c1(4, 3)
      ! [0, 1] <-- [0, 0, 1]
      cprime%c2(1,1) = cprime%c2(1,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(1, 4)
      ! [0, 0] <-- [0, 0, 1]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(1, 4)
      ! [0, 2] <-- [0, 0, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(2, 4)
      ! [0, 0] <-- [0, 0, 2]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(2, 4)
      ! [0, 3] <-- [0, 0, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(3, 4)
      ! [0, 0] <-- [0, 0, 3]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(3, 4)
      ! [1, 2] <-- [1, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(4, 4)
      ! [1, 1] <-- [1, 1, 2]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(4, 4)
      ! [1, 3] <-- [1, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(5, 4)
      ! [1, 1] <-- [1, 1, 3]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(5, 4)
      ! [2, 3] <-- [2, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(6, 4)
      ! [2, 2] <-- [2, 2, 3]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(6, 4)
      ! [1, 1, 2] <-- [0, 1, 1, 2]
      cprime%c2(4,3) = 0.25_ki * C(0) * coeffs%c3(1, 3)
      ! [0, 1, 2] <-- [0, 1, 1, 2]
      cprime%c3(1,1) = 0.5_ki * C(1) * coeffs%c3(1, 3)
      ! [0, 1, 1] <-- [0, 1, 1, 2]
      cprime%c2(1,2) = 0.25_ki * C(2) * coeffs%c3(1, 3)
      ! [1, 1, 3] <-- [0, 1, 1, 3]
      cprime%c2(5,3) = 0.25_ki * C(0) * coeffs%c3(2, 3)
      ! [0, 1, 3] <-- [0, 1, 1, 3]
      cprime%c3(2,1) = 0.5_ki * C(1) * coeffs%c3(2, 3)
      ! [0, 1, 1] <-- [0, 1, 1, 3]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.25_ki * C(3) * coeffs%c3(2, 3)
      ! [2, 2, 3] <-- [0, 2, 2, 3]
      cprime%c2(6,3) = 0.25_ki * C(0) * coeffs%c3(3, 3)
      ! [0, 2, 3] <-- [0, 2, 2, 3]
      cprime%c3(3,1) = 0.5_ki * C(2) * coeffs%c3(3, 3)
      ! [0, 2, 2] <-- [0, 2, 2, 3]
      cprime%c2(2,2) = 0.25_ki * C(3) * coeffs%c3(3, 3)
      ! [2, 2, 3] <-- [1, 2, 2, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 0.25_ki * C(1) * coeffs%c3(4, 3)
      ! [1, 2, 3] <-- [1, 2, 2, 3]
      cprime%c3(4,1) = 0.5_ki * C(2) * coeffs%c3(4, 3)
      ! [1, 2, 2] <-- [1, 2, 2, 3]
      cprime%c2(4,2) = 0.25_ki * C(3) * coeffs%c3(4, 3)
      ! [1, 1, 1] <-- [0, 1, 1, 1]
      cprime%c1(2,3) = 0.25_ki * C(0) * coeffs%c2(1, 3)
      ! [0, 1, 1] <-- [0, 1, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(1, 3)
      ! [2, 2, 2] <-- [0, 2, 2, 2]
      cprime%c1(3,3) = 0.25_ki * C(0) * coeffs%c2(2, 3)
      ! [0, 2, 2] <-- [0, 2, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(2, 3)
      ! [3, 3, 3] <-- [0, 3, 3, 3]
      cprime%c1(4,3) = 0.25_ki * C(0) * coeffs%c2(3, 3)
      ! [0, 3, 3] <-- [0, 3, 3, 3]
      cprime%c2(3,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(3, 3)
      ! [2, 2, 2] <-- [1, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(1) * coeffs%c2(4, 3)
      ! [1, 2, 2] <-- [1, 2, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(4, 3)
      ! [3, 3, 3] <-- [1, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(1) * coeffs%c2(5, 3)
      ! [1, 3, 3] <-- [1, 3, 3, 3]
      cprime%c2(5,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(5, 3)
      ! [3, 3, 3] <-- [2, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(2) * coeffs%c2(6, 3)
      ! [2, 3, 3] <-- [2, 3, 3, 3]
      cprime%c2(6,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(6, 3)
      ! [0, 1, 2] <-- [0, 0, 1, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(0) * coeffs%c3(1, 4)
      ! [0, 0, 2] <-- [0, 0, 1, 2]
      cprime%c2(2,3) = 0.25_ki * C(1) * coeffs%c3(1, 4)
      ! [0, 0, 1] <-- [0, 0, 1, 2]
      cprime%c2(1,3) = 0.25_ki * C(2) * coeffs%c3(1, 4)
      ! [0, 1, 3] <-- [0, 0, 1, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(0) * coeffs%c3(2, 4)
      ! [0, 0, 3] <-- [0, 0, 1, 3]
      cprime%c2(3,3) = 0.25_ki * C(1) * coeffs%c3(2, 4)
      ! [0, 0, 1] <-- [0, 0, 1, 3]
      cprime%c2(1,3) = cprime%c2(1,3) + 0.25_ki * C(3) * coeffs%c3(2, 4)
      ! [0, 2, 3] <-- [0, 0, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(0) * coeffs%c3(3, 4)
      ! [0, 0, 3] <-- [0, 0, 2, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 0.25_ki * C(2) * coeffs%c3(3, 4)
      ! [0, 0, 2] <-- [0, 0, 2, 3]
      cprime%c2(2,3) = cprime%c2(2,3) + 0.25_ki * C(3) * coeffs%c3(3, 4)
      ! [1, 2, 3] <-- [1, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(1) * coeffs%c3(4, 4)
      ! [1, 1, 3] <-- [1, 1, 2, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 0.25_ki * C(2) * coeffs%c3(4, 4)
      ! [1, 1, 2] <-- [1, 1, 2, 3]
      cprime%c2(4,3) = cprime%c2(4,3) + 0.25_ki * C(3) * coeffs%c3(4, 4)
      ! [0, 0, 1] <-- [0, 0, 0, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(1, 6)
      ! [0, 0, 0] <-- [0, 0, 0, 1]
      cprime%c1(1,3) = 0.25_ki * C(1) * coeffs%c2(1, 6)
      ! [0, 0, 2] <-- [0, 0, 0, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(2, 6)
      ! [0, 0, 0] <-- [0, 0, 0, 2]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(2) * coeffs%c2(2, 6)
      ! [0, 0, 3] <-- [0, 0, 0, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(3, 6)
      ! [0, 0, 0] <-- [0, 0, 0, 3]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(3) * coeffs%c2(3, 6)
      ! [1, 1, 2] <-- [1, 1, 1, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(4, 6)
      ! [1, 1, 1] <-- [1, 1, 1, 2]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(2) * coeffs%c2(4, 6)
      ! [1, 1, 3] <-- [1, 1, 1, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(5, 6)
      ! [1, 1, 1] <-- [1, 1, 1, 3]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(3) * coeffs%c2(5, 6)
      ! [2, 2, 3] <-- [2, 2, 2, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(6, 6)
      ! [2, 2, 2] <-- [2, 2, 2, 3]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(3) * coeffs%c2(6, 6)
      ! [1, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.25_ki * C(0) * coeffs%c4(1, 1)
      ! [0, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.25_ki * C(1) * coeffs%c4(1, 1)
      ! [0, 1, 3] <-- [0, 1, 2, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.25_ki * C(2) * coeffs%c4(1, 1)
      ! [0, 1, 2] <-- [0, 1, 2, 3]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.25_ki * C(3) * coeffs%c4(1, 1)
      ! [1, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.25_ki * C(0) * coeffs%c3(1, 2)
      ! [0, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.25_ki * C(1) * coeffs%c3(1, 2)
      ! [0, 1, 2] <-- [0, 1, 2, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(2) * coeffs%c3(1, 2)
      ! [1, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(0) * coeffs%c3(2, 2)
      ! [0, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(1) * coeffs%c3(2, 2)
      ! [0, 1, 3] <-- [0, 1, 3, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(3) * coeffs%c3(2, 2)
      ! [2, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(0) * coeffs%c3(3, 2)
      ! [0, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(2) * coeffs%c3(3, 2)
      ! [0, 2, 3] <-- [0, 2, 3, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(3) * coeffs%c3(3, 2)
      ! [2, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(1) * coeffs%c3(4, 2)
      ! [1, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(2) * coeffs%c3(4, 2)
      ! [1, 2, 3] <-- [1, 2, 3, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(3) * coeffs%c3(4, 2)
      ! [0, 1, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.5_ki * C(0) * coeffs%c2(1, 5)
      ! [0, 0, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 0.5_ki * C(1) * coeffs%c2(1, 5)
      ! [0, 2, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.5_ki * C(0) * coeffs%c2(2, 5)
      ! [0, 0, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 0.5_ki * C(2) * coeffs%c2(2, 5)
      ! [0, 3, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.5_ki * C(0) * coeffs%c2(3, 5)
      ! [0, 0, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 0.5_ki * C(3) * coeffs%c2(3, 5)
      ! [1, 2, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.5_ki * C(1) * coeffs%c2(4, 5)
      ! [1, 1, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 0.5_ki * C(2) * coeffs%c2(4, 5)
      ! [1, 3, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.5_ki * C(1) * coeffs%c2(5, 5)
      ! [1, 1, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 0.5_ki * C(3) * coeffs%c2(5, 5)
      ! [2, 3, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.5_ki * C(2) * coeffs%c2(6, 5)
      ! [2, 2, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 0.5_ki * C(3) * coeffs%c2(6, 5)
      ! [0, 0, 0] <-- [0, 0, 0, 0]
      cprime%c1(1,3) = cprime%c1(1,3) + C(0) * coeffs%c1(1, 4)
      ! [1, 1, 1] <-- [1, 1, 1, 1]
      cprime%c1(2,3) = cprime%c1(2,3) + C(1) * coeffs%c1(2, 4)
      ! [2, 2, 2] <-- [2, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + C(2) * coeffs%c1(3, 4)
      ! [3, 3, 3] <-- [3, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + C(3) * coeffs%c1(4, 4)
      amp = amp - contract5_3(cprime, momenta, new_set)
   end do
end function contract6_4
!****f* src/interface/tens_comb/contract6_5
! NAME
!
! Function contract6_5
!
! USAGE
!
!  amp = contract6_5(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 5 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_5(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   type(coeff_type_4) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime%c0 = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime%c0 = cprime%c0 + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime%c0 = cprime%c0 + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime%c0 = cprime%c0 + C(3) * coeffs%c1(4, 1)
      ! [0] <-- [0, 0]
      cprime%c1(1,1) = C(0) * coeffs%c1(1, 2)
      ! [1] <-- [1, 1]
      cprime%c1(2,1) = C(1) * coeffs%c1(2, 2)
      ! [2] <-- [2, 2]
      cprime%c1(3,1) = C(2) * coeffs%c1(3, 2)
      ! [3] <-- [3, 3]
      cprime%c1(4,1) = C(3) * coeffs%c1(4, 2)
      ! [1] <-- [0, 1]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(0) * coeffs%c2(1, 1)
      ! [0] <-- [0, 1]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(1) * coeffs%c2(1, 1)
      ! [2] <-- [0, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(0) * coeffs%c2(2, 1)
      ! [0] <-- [0, 2]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(2) * coeffs%c2(2, 1)
      ! [3] <-- [0, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(0) * coeffs%c2(3, 1)
      ! [0] <-- [0, 3]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(3) * coeffs%c2(3, 1)
      ! [2] <-- [1, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(1) * coeffs%c2(4, 1)
      ! [1] <-- [1, 2]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(2) * coeffs%c2(4, 1)
      ! [3] <-- [1, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(1) * coeffs%c2(5, 1)
      ! [1] <-- [1, 3]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(3) * coeffs%c2(5, 1)
      ! [3] <-- [2, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(2) * coeffs%c2(6, 1)
      ! [2] <-- [2, 3]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(3) * coeffs%c2(6, 1)
      ! [1, 1] <-- [0, 1, 1]
      cprime%c1(2,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(1, 2)
      ! [0, 1] <-- [0, 1, 1]
      cprime%c2(1,1) = 2.0_ki/3.0_ki * C(1) * coeffs%c2(1, 2)
      ! [2, 2] <-- [0, 2, 2]
      cprime%c1(3,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(2, 2)
      ! [0, 2] <-- [0, 2, 2]
      cprime%c2(2,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(2, 2)
      ! [3, 3] <-- [0, 3, 3]
      cprime%c1(4,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(3, 2)
      ! [0, 3] <-- [0, 3, 3]
      cprime%c2(3,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(3, 2)
      ! [2, 2] <-- [1, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(4, 2)
      ! [1, 2] <-- [1, 2, 2]
      cprime%c2(4,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(4, 2)
      ! [3, 3] <-- [1, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(5, 2)
      ! [1, 3] <-- [1, 3, 3]
      cprime%c2(5,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(5, 2)
      ! [3, 3] <-- [2, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(6, 2)
      ! [2, 3] <-- [2, 3, 3]
      cprime%c2(6,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(6, 2)
      ! [1, 2] <-- [0, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 1)
      ! [0, 2] <-- [0, 1, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 1)
      ! [0, 1] <-- [0, 1, 2]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 1)
      ! [1, 3] <-- [0, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 1)
      ! [0, 3] <-- [0, 1, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 1)
      ! [0, 1] <-- [0, 1, 3]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 1)
      ! [2, 3] <-- [0, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 1)
      ! [0, 3] <-- [0, 2, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 1)
      ! [0, 2] <-- [0, 2, 3]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 1)
      ! [2, 3] <-- [1, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 1)
      ! [1, 3] <-- [1, 2, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 1)
      ! [1, 2] <-- [1, 2, 3]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 1)
      ! [0, 0] <-- [0, 0, 0]
      cprime%c1(1,2) = C(0) * coeffs%c1(1, 3)
      ! [1, 1] <-- [1, 1, 1]
      cprime%c1(2,2) = cprime%c1(2,2) + C(1) * coeffs%c1(2, 3)
      ! [2, 2] <-- [2, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + C(2) * coeffs%c1(3, 3)
      ! [3, 3] <-- [3, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + C(3) * coeffs%c1(4, 3)
      ! [0, 1] <-- [0, 0, 1]
      cprime%c2(1,1) = cprime%c2(1,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(1, 5)
      ! [0, 0] <-- [0, 0, 1]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(1, 5)
      ! [0, 2] <-- [0, 0, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(2, 5)
      ! [0, 0] <-- [0, 0, 2]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(2, 5)
      ! [0, 3] <-- [0, 0, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(3, 5)
      ! [0, 0] <-- [0, 0, 3]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(3, 5)
      ! [1, 2] <-- [1, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(4, 5)
      ! [1, 1] <-- [1, 1, 2]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(4, 5)
      ! [1, 3] <-- [1, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(5, 5)
      ! [1, 1] <-- [1, 1, 3]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(5, 5)
      ! [2, 3] <-- [2, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(6, 5)
      ! [2, 2] <-- [2, 2, 3]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(6, 5)
      ! [1, 1, 2] <-- [0, 1, 1, 2]
      cprime%c2(4,4) = 0.25_ki * C(0) * coeffs%c3(1, 4)
      ! [0, 1, 2] <-- [0, 1, 1, 2]
      cprime%c3(1,1) = 0.5_ki * C(1) * coeffs%c3(1, 4)
      ! [0, 1, 1] <-- [0, 1, 1, 2]
      cprime%c2(1,2) = 0.25_ki * C(2) * coeffs%c3(1, 4)
      ! [1, 1, 3] <-- [0, 1, 1, 3]
      cprime%c2(5,4) = 0.25_ki * C(0) * coeffs%c3(2, 4)
      ! [0, 1, 3] <-- [0, 1, 1, 3]
      cprime%c3(2,1) = 0.5_ki * C(1) * coeffs%c3(2, 4)
      ! [0, 1, 1] <-- [0, 1, 1, 3]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.25_ki * C(3) * coeffs%c3(2, 4)
      ! [2, 2, 3] <-- [0, 2, 2, 3]
      cprime%c2(6,4) = 0.25_ki * C(0) * coeffs%c3(3, 4)
      ! [0, 2, 3] <-- [0, 2, 2, 3]
      cprime%c3(3,1) = 0.5_ki * C(2) * coeffs%c3(3, 4)
      ! [0, 2, 2] <-- [0, 2, 2, 3]
      cprime%c2(2,2) = 0.25_ki * C(3) * coeffs%c3(3, 4)
      ! [2, 2, 3] <-- [1, 2, 2, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 0.25_ki * C(1) * coeffs%c3(4, 4)
      ! [1, 2, 3] <-- [1, 2, 2, 3]
      cprime%c3(4,1) = 0.5_ki * C(2) * coeffs%c3(4, 4)
      ! [1, 2, 2] <-- [1, 2, 2, 3]
      cprime%c2(4,2) = 0.25_ki * C(3) * coeffs%c3(4, 4)
      ! [1, 1, 1] <-- [0, 1, 1, 1]
      cprime%c1(2,3) = 0.25_ki * C(0) * coeffs%c2(1, 3)
      ! [0, 1, 1] <-- [0, 1, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(1, 3)
      ! [2, 2, 2] <-- [0, 2, 2, 2]
      cprime%c1(3,3) = 0.25_ki * C(0) * coeffs%c2(2, 3)
      ! [0, 2, 2] <-- [0, 2, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(2, 3)
      ! [3, 3, 3] <-- [0, 3, 3, 3]
      cprime%c1(4,3) = 0.25_ki * C(0) * coeffs%c2(3, 3)
      ! [0, 3, 3] <-- [0, 3, 3, 3]
      cprime%c2(3,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(3, 3)
      ! [2, 2, 2] <-- [1, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(1) * coeffs%c2(4, 3)
      ! [1, 2, 2] <-- [1, 2, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(4, 3)
      ! [3, 3, 3] <-- [1, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(1) * coeffs%c2(5, 3)
      ! [1, 3, 3] <-- [1, 3, 3, 3]
      cprime%c2(5,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(5, 3)
      ! [3, 3, 3] <-- [2, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(2) * coeffs%c2(6, 3)
      ! [2, 3, 3] <-- [2, 3, 3, 3]
      cprime%c2(6,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(6, 3)
      ! [0, 1, 2] <-- [0, 0, 1, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(0) * coeffs%c3(1, 7)
      ! [0, 0, 2] <-- [0, 0, 1, 2]
      cprime%c2(2,4) = 0.25_ki * C(1) * coeffs%c3(1, 7)
      ! [0, 0, 1] <-- [0, 0, 1, 2]
      cprime%c2(1,4) = 0.25_ki * C(2) * coeffs%c3(1, 7)
      ! [0, 1, 3] <-- [0, 0, 1, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(0) * coeffs%c3(2, 7)
      ! [0, 0, 3] <-- [0, 0, 1, 3]
      cprime%c2(3,4) = 0.25_ki * C(1) * coeffs%c3(2, 7)
      ! [0, 0, 1] <-- [0, 0, 1, 3]
      cprime%c2(1,4) = cprime%c2(1,4) + 0.25_ki * C(3) * coeffs%c3(2, 7)
      ! [0, 2, 3] <-- [0, 0, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(0) * coeffs%c3(3, 7)
      ! [0, 0, 3] <-- [0, 0, 2, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 0.25_ki * C(2) * coeffs%c3(3, 7)
      ! [0, 0, 2] <-- [0, 0, 2, 3]
      cprime%c2(2,4) = cprime%c2(2,4) + 0.25_ki * C(3) * coeffs%c3(3, 7)
      ! [1, 2, 3] <-- [1, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(1) * coeffs%c3(4, 7)
      ! [1, 1, 3] <-- [1, 1, 2, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 0.25_ki * C(2) * coeffs%c3(4, 7)
      ! [1, 1, 2] <-- [1, 1, 2, 3]
      cprime%c2(4,4) = cprime%c2(4,4) + 0.25_ki * C(3) * coeffs%c3(4, 7)
      ! [0, 0, 1] <-- [0, 0, 0, 1]
      cprime%c2(1,4) = cprime%c2(1,4) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(1, 8)
      ! [0, 0, 0] <-- [0, 0, 0, 1]
      cprime%c1(1,3) = 0.25_ki * C(1) * coeffs%c2(1, 8)
      ! [0, 0, 2] <-- [0, 0, 0, 2]
      cprime%c2(2,4) = cprime%c2(2,4) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(2, 8)
      ! [0, 0, 0] <-- [0, 0, 0, 2]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(2) * coeffs%c2(2, 8)
      ! [0, 0, 3] <-- [0, 0, 0, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(3, 8)
      ! [0, 0, 0] <-- [0, 0, 0, 3]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(3) * coeffs%c2(3, 8)
      ! [1, 1, 2] <-- [1, 1, 1, 2]
      cprime%c2(4,4) = cprime%c2(4,4) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(4, 8)
      ! [1, 1, 1] <-- [1, 1, 1, 2]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(2) * coeffs%c2(4, 8)
      ! [1, 1, 3] <-- [1, 1, 1, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(5, 8)
      ! [1, 1, 1] <-- [1, 1, 1, 3]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(3) * coeffs%c2(5, 8)
      ! [2, 2, 3] <-- [2, 2, 2, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(6, 8)
      ! [2, 2, 2] <-- [2, 2, 2, 3]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(3) * coeffs%c2(6, 8)
      ! [1, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.25_ki * C(0) * coeffs%c4(1, 1)
      ! [0, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.25_ki * C(1) * coeffs%c4(1, 1)
      ! [0, 1, 3] <-- [0, 1, 2, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.25_ki * C(2) * coeffs%c4(1, 1)
      ! [0, 1, 2] <-- [0, 1, 2, 3]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.25_ki * C(3) * coeffs%c4(1, 1)
      ! [1, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.25_ki * C(0) * coeffs%c3(1, 2)
      ! [0, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.25_ki * C(1) * coeffs%c3(1, 2)
      ! [0, 1, 2] <-- [0, 1, 2, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(2) * coeffs%c3(1, 2)
      ! [1, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(0) * coeffs%c3(2, 2)
      ! [0, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(1) * coeffs%c3(2, 2)
      ! [0, 1, 3] <-- [0, 1, 3, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(3) * coeffs%c3(2, 2)
      ! [2, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(0) * coeffs%c3(3, 2)
      ! [0, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(2) * coeffs%c3(3, 2)
      ! [0, 2, 3] <-- [0, 2, 3, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(3) * coeffs%c3(3, 2)
      ! [2, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(1) * coeffs%c3(4, 2)
      ! [1, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(2) * coeffs%c3(4, 2)
      ! [1, 2, 3] <-- [1, 2, 3, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(3) * coeffs%c3(4, 2)
      ! [0, 1, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.5_ki * C(0) * coeffs%c2(1, 6)
      ! [0, 0, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,4) = cprime%c2(1,4) + 0.5_ki * C(1) * coeffs%c2(1, 6)
      ! [0, 2, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.5_ki * C(0) * coeffs%c2(2, 6)
      ! [0, 0, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,4) = cprime%c2(2,4) + 0.5_ki * C(2) * coeffs%c2(2, 6)
      ! [0, 3, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.5_ki * C(0) * coeffs%c2(3, 6)
      ! [0, 0, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 0.5_ki * C(3) * coeffs%c2(3, 6)
      ! [1, 2, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.5_ki * C(1) * coeffs%c2(4, 6)
      ! [1, 1, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,4) = cprime%c2(4,4) + 0.5_ki * C(2) * coeffs%c2(4, 6)
      ! [1, 3, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.5_ki * C(1) * coeffs%c2(5, 6)
      ! [1, 1, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 0.5_ki * C(3) * coeffs%c2(5, 6)
      ! [2, 3, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.5_ki * C(2) * coeffs%c2(6, 6)
      ! [2, 2, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 0.5_ki * C(3) * coeffs%c2(6, 6)
      ! [0, 0, 0] <-- [0, 0, 0, 0]
      cprime%c1(1,3) = cprime%c1(1,3) + C(0) * coeffs%c1(1, 4)
      ! [1, 1, 1] <-- [1, 1, 1, 1]
      cprime%c1(2,3) = cprime%c1(2,3) + C(1) * coeffs%c1(2, 4)
      ! [2, 2, 2] <-- [2, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + C(2) * coeffs%c1(3, 4)
      ! [3, 3, 3] <-- [3, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + C(3) * coeffs%c1(4, 4)
      ! [0, 0, 1, 1] <-- [0, 0, 0, 1, 1]
      cprime%c2(1,5) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(1, 9)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 1]
      cprime%c2(1,6) = 2.0_ki/5.0_ki * C(1) * coeffs%c2(1, 9)
      ! [0, 0, 2, 2] <-- [0, 0, 0, 2, 2]
      cprime%c2(2,5) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(2, 9)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 2, 2]
      cprime%c2(2,6) = 2.0_ki/5.0_ki * C(2) * coeffs%c2(2, 9)
      ! [0, 0, 3, 3] <-- [0, 0, 0, 3, 3]
      cprime%c2(3,5) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(3, 9)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 3, 3]
      cprime%c2(3,6) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(3, 9)
      ! [1, 1, 2, 2] <-- [1, 1, 1, 2, 2]
      cprime%c2(4,5) = 3.0_ki/5.0_ki * C(1) * coeffs%c2(4, 9)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 2, 2]
      cprime%c2(4,6) = 2.0_ki/5.0_ki * C(2) * coeffs%c2(4, 9)
      ! [1, 1, 3, 3] <-- [1, 1, 1, 3, 3]
      cprime%c2(5,5) = 3.0_ki/5.0_ki * C(1) * coeffs%c2(5, 9)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 3, 3]
      cprime%c2(5,6) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(5, 9)
      ! [2, 2, 3, 3] <-- [2, 2, 2, 3, 3]
      cprime%c2(6,5) = 3.0_ki/5.0_ki * C(2) * coeffs%c2(6, 9)
      ! [2, 2, 2, 3] <-- [2, 2, 2, 3, 3]
      cprime%c2(6,6) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(6, 9)
      ! [0, 0, 1, 2] <-- [0, 0, 0, 1, 2]
      cprime%c3(1,4) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(1, 10)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 1, 2]
      cprime%c2(2,6) = cprime%c2(2,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 10)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 2]
      cprime%c2(1,6) = cprime%c2(1,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 10)
      ! [0, 0, 1, 3] <-- [0, 0, 0, 1, 3]
      cprime%c3(2,4) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(2, 10)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 1, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 10)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 3]
      cprime%c2(1,6) = cprime%c2(1,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 10)
      ! [0, 0, 2, 3] <-- [0, 0, 0, 2, 3]
      cprime%c3(3,4) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(3, 10)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 2, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 10)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 2, 3]
      cprime%c2(2,6) = cprime%c2(2,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 10)
      ! [1, 1, 2, 3] <-- [1, 1, 1, 2, 3]
      cprime%c3(4,4) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(4, 10)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 2, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 10)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 2, 3]
      cprime%c2(4,6) = cprime%c2(4,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 10)
      ! [1, 2, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(4,2) = 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 2)
      ! [0, 2, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(3,2) = 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 2)
      ! [0, 1, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(2,2) = 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 2)
      ! [0, 1, 2, 3] <-- [0, 1, 2, 3, 3]
      cprime%c4(1,1) = 2.0_ki/5.0_ki * C(3) * coeffs%c4(1, 2)
      ! [1, 2, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c2(4,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 3)
      ! [0, 2, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c2(2,3) = 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 3)
      ! [0, 1, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c3(1,2) = 3.0_ki/5.0_ki * C(2) * coeffs%c3(1, 3)
      ! [1, 3, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c2(5,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 3)
      ! [0, 3, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c2(3,3) = 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 3)
      ! [0, 1, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(2, 3)
      ! [2, 3, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c2(6,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 3)
      ! [0, 3, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 3)
      ! [0, 2, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(3, 3)
      ! [2, 3, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 3)
      ! [1, 3, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 3)
      ! [1, 2, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(4, 3)
      ! [0, 1, 2, 3] <-- [0, 0, 1, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(0) * coeffs%c4(1, 5)
      ! [0, 0, 2, 3] <-- [0, 0, 1, 2, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 5)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 2, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 5)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 2, 3]
      cprime%c3(1,4) = cprime%c3(1,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 5)
      ! [1, 2, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c3(4,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 3)
      ! [0, 2, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c3(3,3) = 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 3)
      ! [0, 1, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(2) * coeffs%c4(1, 3)
      ! [0, 1, 2, 2] <-- [0, 1, 2, 2, 3]
      cprime%c3(1,2) = cprime%c3(1,2) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 3)
      ! [1, 1, 1, 2] <-- [0, 1, 1, 1, 2]
      cprime%c2(4,6) = cprime%c2(4,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 6)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 1, 2]
      cprime%c3(1,3) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(1, 6)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 2]
      cprime%c2(1,3) = 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 6)
      ! [1, 1, 1, 3] <-- [0, 1, 1, 1, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 6)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 1, 3]
      cprime%c3(2,3) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(2, 6)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 3]
      cprime%c2(1,3) = cprime%c2(1,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 6)
      ! [2, 2, 2, 3] <-- [0, 2, 2, 2, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 6)
      ! [0, 2, 2, 3] <-- [0, 2, 2, 2, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 3.0_ki/5.0_ki * C(2) * coeffs%c3(3, 6)
      ! [0, 2, 2, 2] <-- [0, 2, 2, 2, 3]
      cprime%c2(2,3) = cprime%c2(2,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 6)
      ! [2, 2, 2, 3] <-- [1, 2, 2, 2, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 6)
      ! [1, 2, 2, 3] <-- [1, 2, 2, 2, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 3.0_ki/5.0_ki * C(2) * coeffs%c3(4, 6)
      ! [1, 2, 2, 2] <-- [1, 2, 2, 2, 3]
      cprime%c2(4,3) = cprime%c2(4,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 6)
      ! [1, 1, 1, 1] <-- [0, 1, 1, 1, 1]
      cprime%c1(2,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(1, 4)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(1, 4)
      ! [2, 2, 2, 2] <-- [0, 2, 2, 2, 2]
      cprime%c1(3,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(2, 4)
      ! [0, 2, 2, 2] <-- [0, 2, 2, 2, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(2, 4)
      ! [3, 3, 3, 3] <-- [0, 3, 3, 3, 3]
      cprime%c1(4,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(3, 4)
      ! [0, 3, 3, 3] <-- [0, 3, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(3, 4)
      ! [2, 2, 2, 2] <-- [1, 2, 2, 2, 2]
      cprime%c1(3,4) = cprime%c1(3,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(4, 4)
      ! [1, 2, 2, 2] <-- [1, 2, 2, 2, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(4, 4)
      ! [3, 3, 3, 3] <-- [1, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(5, 4)
      ! [1, 3, 3, 3] <-- [1, 3, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(5, 4)
      ! [3, 3, 3, 3] <-- [2, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(6, 4)
      ! [2, 3, 3, 3] <-- [2, 3, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(6, 4)
      ! [1, 1, 2, 2] <-- [0, 1, 1, 2, 2]
      cprime%c2(4,5) = cprime%c2(4,5) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 5)
      ! [0, 1, 2, 2] <-- [0, 1, 1, 2, 2]
      cprime%c3(1,2) = cprime%c3(1,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(1, 5)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 2, 2]
      cprime%c3(1,3) = cprime%c3(1,3) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(1, 5)
      ! [1, 1, 3, 3] <-- [0, 1, 1, 3, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 5)
      ! [0, 1, 3, 3] <-- [0, 1, 1, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(2, 5)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 3, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(2, 5)
      ! [2, 2, 3, 3] <-- [0, 2, 2, 3, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 5)
      ! [0, 2, 3, 3] <-- [0, 2, 2, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(3, 5)
      ! [0, 2, 2, 3] <-- [0, 2, 2, 3, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(3, 5)
      ! [2, 2, 3, 3] <-- [1, 2, 2, 3, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 5)
      ! [1, 2, 3, 3] <-- [1, 2, 2, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(4, 5)
      ! [1, 2, 2, 3] <-- [1, 2, 2, 3, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(4, 5)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 0]
      cprime%c1(1,4) = C(0) * coeffs%c1(1, 5)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 1]
      cprime%c1(2,4) = cprime%c1(2,4) + C(1) * coeffs%c1(2, 5)
      ! [2, 2, 2, 2] <-- [2, 2, 2, 2, 2]
      cprime%c1(3,4) = cprime%c1(3,4) + C(2) * coeffs%c1(3, 5)
      ! [3, 3, 3, 3] <-- [3, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + C(3) * coeffs%c1(4, 5)
      ! [0, 1, 1, 1] <-- [0, 0, 1, 1, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(1, 7)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 1]
      cprime%c2(1,5) = cprime%c2(1,5) + 3.0_ki/5.0_ki * C(1) * coeffs%c2(1, 7)
      ! [0, 2, 2, 2] <-- [0, 0, 2, 2, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(2, 7)
      ! [0, 0, 2, 2] <-- [0, 0, 2, 2, 2]
      cprime%c2(2,5) = cprime%c2(2,5) + 3.0_ki/5.0_ki * C(2) * coeffs%c2(2, 7)
      ! [0, 3, 3, 3] <-- [0, 0, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(3, 7)
      ! [0, 0, 3, 3] <-- [0, 0, 3, 3, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(3, 7)
      ! [1, 2, 2, 2] <-- [1, 1, 2, 2, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 2.0_ki/5.0_ki * C(1) * coeffs%c2(4, 7)
      ! [1, 1, 2, 2] <-- [1, 1, 2, 2, 2]
      cprime%c2(4,5) = cprime%c2(4,5) + 3.0_ki/5.0_ki * C(2) * coeffs%c2(4, 7)
      ! [1, 3, 3, 3] <-- [1, 1, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 2.0_ki/5.0_ki * C(1) * coeffs%c2(5, 7)
      ! [1, 1, 3, 3] <-- [1, 1, 3, 3, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(5, 7)
      ! [2, 3, 3, 3] <-- [2, 2, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 2.0_ki/5.0_ki * C(2) * coeffs%c2(6, 7)
      ! [2, 2, 3, 3] <-- [2, 2, 3, 3, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(6, 7)
      ! [0, 1, 2, 2] <-- [0, 0, 1, 2, 2]
      cprime%c3(1,2) = cprime%c3(1,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(1, 8)
      ! [0, 0, 2, 2] <-- [0, 0, 1, 2, 2]
      cprime%c2(2,5) = cprime%c2(2,5) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 8)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 2, 2]
      cprime%c3(1,4) = cprime%c3(1,4) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(1, 8)
      ! [0, 1, 3, 3] <-- [0, 0, 1, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(2, 8)
      ! [0, 0, 3, 3] <-- [0, 0, 1, 3, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 8)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 3, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(2, 8)
      ! [0, 2, 3, 3] <-- [0, 0, 2, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(3, 8)
      ! [0, 0, 3, 3] <-- [0, 0, 2, 3, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 8)
      ! [0, 0, 2, 3] <-- [0, 0, 2, 3, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(3, 8)
      ! [1, 2, 3, 3] <-- [1, 1, 2, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(4, 8)
      ! [1, 1, 3, 3] <-- [1, 1, 2, 3, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 8)
      ! [1, 1, 2, 3] <-- [1, 1, 2, 3, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(4, 8)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 0, 1]
      cprime%c2(1,6) = cprime%c2(1,6) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(1, 10)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 1]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(1, 10)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 0, 2]
      cprime%c2(2,6) = cprime%c2(2,6) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(2, 10)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 2]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(2, 10)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 0, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(3, 10)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 3]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(3, 10)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 1, 2]
      cprime%c2(4,6) = cprime%c2(4,6) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(4, 10)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 2]
      cprime%c1(2,4) = cprime%c1(2,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(4, 10)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 1, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(5, 10)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 3]
      cprime%c1(2,4) = cprime%c1(2,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(5, 10)
      ! [2, 2, 2, 3] <-- [2, 2, 2, 2, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(6, 10)
      ! [2, 2, 2, 2] <-- [2, 2, 2, 2, 3]
      cprime%c1(3,4) = cprime%c1(3,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(6, 10)
      ! [0, 1, 1, 2] <-- [0, 0, 1, 1, 2]
      cprime%c3(1,3) = cprime%c3(1,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(1, 9)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 1, 2]
      cprime%c3(1,4) = cprime%c3(1,4) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(1, 9)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 2]
      cprime%c2(1,5) = cprime%c2(1,5) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 9)
      ! [0, 1, 1, 3] <-- [0, 0, 1, 1, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(2, 9)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 1, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(2, 9)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 3]
      cprime%c2(1,5) = cprime%c2(1,5) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 9)
      ! [0, 2, 2, 3] <-- [0, 0, 2, 2, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(3, 9)
      ! [0, 0, 2, 3] <-- [0, 0, 2, 2, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(3, 9)
      ! [0, 0, 2, 2] <-- [0, 0, 2, 2, 3]
      cprime%c2(2,5) = cprime%c2(2,5) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 9)
      ! [1, 2, 2, 3] <-- [1, 1, 2, 2, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(4, 9)
      ! [1, 1, 2, 3] <-- [1, 1, 2, 2, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(4, 9)
      ! [1, 1, 2, 2] <-- [1, 1, 2, 2, 3]
      cprime%c2(4,5) = cprime%c2(4,5) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 9)
      ! [1, 1, 2, 3] <-- [0, 1, 1, 2, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 4)
      ! [0, 1, 2, 3] <-- [0, 1, 1, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(1) * coeffs%c4(1, 4)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 2, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 4)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 2, 3]
      cprime%c3(1,3) = cprime%c3(1,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 4)
      amp = amp - contract5_4(cprime, momenta, new_set)
   end do
end function contract6_5
!****f* src/interface/tens_comb/contract6_6
! NAME
!
! Function contract6_6
!
! USAGE
!
!  amp = contract6_6(coeffs, momenta, b_set)
!
! DESCRIPTION
!
!  Contracts the 6-point rank 6 tensor integral
!  with its coefficients.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!  * b_set   -- the set of pinched propagators as integer number (bit-set)
!
! RETURN VALUE
!
!  The result of contracting the tensor integral with its coefficient.
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
function     contract6_6(coeffs, momenta, b_set) result(amp)
   ! generated by: write_function_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in) :: momenta
   integer, intent(in) :: b_set
   type(form_factor) :: amp
   ! generated by: write_contract_split
   complex(ki), dimension(0:3) :: C
   type(coeff_type_5) :: cprime
   integer :: i, pnch, new_set
   integer, dimension(1) :: pnch_set
   integer, dimension(6) :: unpinched
   unpinched = unpackb(pminus(b_ref, b_set), 6)
   amp = coeffs%c0 * a60(b_set)
   do pnch=1,6
      ! Eq. (54) in hep-ph/0504267
      C(:) = 0.0_ki
      do i=1,6
         C(:) = C(:) + inv_s(unpinched(pnch),unpinched(i),b_set) * &
              & momenta(unpinched(i),:)
      end do
      ! Eq. (63) in hep-ph/0504267
      pnch_set(1) = pnch
      new_set = punion(packb(pnch_set),b_set)
      ! [] <-- [0]
      cprime%c0 = C(0) * coeffs%c1(1, 1)
      ! [] <-- [1]
      cprime%c0 = cprime%c0 + C(1) * coeffs%c1(2, 1)
      ! [] <-- [2]
      cprime%c0 = cprime%c0 + C(2) * coeffs%c1(3, 1)
      ! [] <-- [3]
      cprime%c0 = cprime%c0 + C(3) * coeffs%c1(4, 1)
      ! [0] <-- [0, 0]
      cprime%c1(1,1) = C(0) * coeffs%c1(1, 2)
      ! [1] <-- [1, 1]
      cprime%c1(2,1) = C(1) * coeffs%c1(2, 2)
      ! [2] <-- [2, 2]
      cprime%c1(3,1) = C(2) * coeffs%c1(3, 2)
      ! [3] <-- [3, 3]
      cprime%c1(4,1) = C(3) * coeffs%c1(4, 2)
      ! [1] <-- [0, 1]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(0) * coeffs%c2(1, 1)
      ! [0] <-- [0, 1]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(1) * coeffs%c2(1, 1)
      ! [2] <-- [0, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(0) * coeffs%c2(2, 1)
      ! [0] <-- [0, 2]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(2) * coeffs%c2(2, 1)
      ! [3] <-- [0, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(0) * coeffs%c2(3, 1)
      ! [0] <-- [0, 3]
      cprime%c1(1,1) = cprime%c1(1,1) + 0.5_ki * C(3) * coeffs%c2(3, 1)
      ! [2] <-- [1, 2]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(1) * coeffs%c2(4, 1)
      ! [1] <-- [1, 2]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(2) * coeffs%c2(4, 1)
      ! [3] <-- [1, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(1) * coeffs%c2(5, 1)
      ! [1] <-- [1, 3]
      cprime%c1(2,1) = cprime%c1(2,1) + 0.5_ki * C(3) * coeffs%c2(5, 1)
      ! [3] <-- [2, 3]
      cprime%c1(4,1) = cprime%c1(4,1) + 0.5_ki * C(2) * coeffs%c2(6, 1)
      ! [2] <-- [2, 3]
      cprime%c1(3,1) = cprime%c1(3,1) + 0.5_ki * C(3) * coeffs%c2(6, 1)
      ! [1, 1] <-- [0, 1, 1]
      cprime%c1(2,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(1, 2)
      ! [0, 1] <-- [0, 1, 1]
      cprime%c2(1,1) = 2.0_ki/3.0_ki * C(1) * coeffs%c2(1, 2)
      ! [2, 2] <-- [0, 2, 2]
      cprime%c1(3,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(2, 2)
      ! [0, 2] <-- [0, 2, 2]
      cprime%c2(2,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(2, 2)
      ! [3, 3] <-- [0, 3, 3]
      cprime%c1(4,2) = 1.0_ki/3.0_ki * C(0) * coeffs%c2(3, 2)
      ! [0, 3] <-- [0, 3, 3]
      cprime%c2(3,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(3, 2)
      ! [2, 2] <-- [1, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(4, 2)
      ! [1, 2] <-- [1, 2, 2]
      cprime%c2(4,1) = 2.0_ki/3.0_ki * C(2) * coeffs%c2(4, 2)
      ! [3, 3] <-- [1, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(5, 2)
      ! [1, 3] <-- [1, 3, 3]
      cprime%c2(5,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(5, 2)
      ! [3, 3] <-- [2, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(6, 2)
      ! [2, 3] <-- [2, 3, 3]
      cprime%c2(6,1) = 2.0_ki/3.0_ki * C(3) * coeffs%c2(6, 2)
      ! [1, 2] <-- [0, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 1)
      ! [0, 2] <-- [0, 1, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 1)
      ! [0, 1] <-- [0, 1, 2]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 1)
      ! [1, 3] <-- [0, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 1)
      ! [0, 3] <-- [0, 1, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 1)
      ! [0, 1] <-- [0, 1, 3]
      cprime%c2(1,1) = cprime%c2(1,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 1)
      ! [2, 3] <-- [0, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 1)
      ! [0, 3] <-- [0, 2, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 1)
      ! [0, 2] <-- [0, 2, 3]
      cprime%c2(2,1) = cprime%c2(2,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 1)
      ! [2, 3] <-- [1, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 1)
      ! [1, 3] <-- [1, 2, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 1)
      ! [1, 2] <-- [1, 2, 3]
      cprime%c2(4,1) = cprime%c2(4,1) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 1)
      ! [0, 0] <-- [0, 0, 0]
      cprime%c1(1,2) = C(0) * coeffs%c1(1, 3)
      ! [1, 1] <-- [1, 1, 1]
      cprime%c1(2,2) = cprime%c1(2,2) + C(1) * coeffs%c1(2, 3)
      ! [2, 2] <-- [2, 2, 2]
      cprime%c1(3,2) = cprime%c1(3,2) + C(2) * coeffs%c1(3, 3)
      ! [3, 3] <-- [3, 3, 3]
      cprime%c1(4,2) = cprime%c1(4,2) + C(3) * coeffs%c1(4, 3)
      ! [0, 1] <-- [0, 0, 1]
      cprime%c2(1,1) = cprime%c2(1,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(1, 6)
      ! [0, 0] <-- [0, 0, 1]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(1, 6)
      ! [0, 2] <-- [0, 0, 2]
      cprime%c2(2,1) = cprime%c2(2,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(2, 6)
      ! [0, 0] <-- [0, 0, 2]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(2, 6)
      ! [0, 3] <-- [0, 0, 3]
      cprime%c2(3,1) = cprime%c2(3,1) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(3, 6)
      ! [0, 0] <-- [0, 0, 3]
      cprime%c1(1,2) = cprime%c1(1,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(3, 6)
      ! [1, 2] <-- [1, 1, 2]
      cprime%c2(4,1) = cprime%c2(4,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(4, 6)
      ! [1, 1] <-- [1, 1, 2]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(4, 6)
      ! [1, 3] <-- [1, 1, 3]
      cprime%c2(5,1) = cprime%c2(5,1) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(5, 6)
      ! [1, 1] <-- [1, 1, 3]
      cprime%c1(2,2) = cprime%c1(2,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(5, 6)
      ! [2, 3] <-- [2, 2, 3]
      cprime%c2(6,1) = cprime%c2(6,1) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(6, 6)
      ! [2, 2] <-- [2, 2, 3]
      cprime%c1(3,2) = cprime%c1(3,2) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(6, 6)
      ! [1, 1, 2] <-- [0, 1, 1, 2]
      cprime%c2(4,5) = 0.25_ki * C(0) * coeffs%c3(1, 5)
      ! [0, 1, 2] <-- [0, 1, 1, 2]
      cprime%c3(1,1) = 0.5_ki * C(1) * coeffs%c3(1, 5)
      ! [0, 1, 1] <-- [0, 1, 1, 2]
      cprime%c2(1,2) = 0.25_ki * C(2) * coeffs%c3(1, 5)
      ! [1, 1, 3] <-- [0, 1, 1, 3]
      cprime%c2(5,5) = 0.25_ki * C(0) * coeffs%c3(2, 5)
      ! [0, 1, 3] <-- [0, 1, 1, 3]
      cprime%c3(2,1) = 0.5_ki * C(1) * coeffs%c3(2, 5)
      ! [0, 1, 1] <-- [0, 1, 1, 3]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.25_ki * C(3) * coeffs%c3(2, 5)
      ! [2, 2, 3] <-- [0, 2, 2, 3]
      cprime%c2(6,5) = 0.25_ki * C(0) * coeffs%c3(3, 5)
      ! [0, 2, 3] <-- [0, 2, 2, 3]
      cprime%c3(3,1) = 0.5_ki * C(2) * coeffs%c3(3, 5)
      ! [0, 2, 2] <-- [0, 2, 2, 3]
      cprime%c2(2,2) = 0.25_ki * C(3) * coeffs%c3(3, 5)
      ! [2, 2, 3] <-- [1, 2, 2, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 0.25_ki * C(1) * coeffs%c3(4, 5)
      ! [1, 2, 3] <-- [1, 2, 2, 3]
      cprime%c3(4,1) = 0.5_ki * C(2) * coeffs%c3(4, 5)
      ! [1, 2, 2] <-- [1, 2, 2, 3]
      cprime%c2(4,2) = 0.25_ki * C(3) * coeffs%c3(4, 5)
      ! [1, 1, 1] <-- [0, 1, 1, 1]
      cprime%c1(2,3) = 0.25_ki * C(0) * coeffs%c2(1, 3)
      ! [0, 1, 1] <-- [0, 1, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(1, 3)
      ! [2, 2, 2] <-- [0, 2, 2, 2]
      cprime%c1(3,3) = 0.25_ki * C(0) * coeffs%c2(2, 3)
      ! [0, 2, 2] <-- [0, 2, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(2, 3)
      ! [3, 3, 3] <-- [0, 3, 3, 3]
      cprime%c1(4,3) = 0.25_ki * C(0) * coeffs%c2(3, 3)
      ! [0, 3, 3] <-- [0, 3, 3, 3]
      cprime%c2(3,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(3, 3)
      ! [2, 2, 2] <-- [1, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(1) * coeffs%c2(4, 3)
      ! [1, 2, 2] <-- [1, 2, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(4, 3)
      ! [3, 3, 3] <-- [1, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(1) * coeffs%c2(5, 3)
      ! [1, 3, 3] <-- [1, 3, 3, 3]
      cprime%c2(5,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(5, 3)
      ! [3, 3, 3] <-- [2, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + 0.25_ki * C(2) * coeffs%c2(6, 3)
      ! [2, 3, 3] <-- [2, 3, 3, 3]
      cprime%c2(6,2) = 3.0_ki/4.0_ki * C(3) * coeffs%c2(6, 3)
      ! [0, 1, 2] <-- [0, 0, 1, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(0) * coeffs%c3(1, 11)
      ! [0, 0, 2] <-- [0, 0, 1, 2]
      cprime%c2(2,5) = 0.25_ki * C(1) * coeffs%c3(1, 11)
      ! [0, 0, 1] <-- [0, 0, 1, 2]
      cprime%c2(1,5) = 0.25_ki * C(2) * coeffs%c3(1, 11)
      ! [0, 1, 3] <-- [0, 0, 1, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(0) * coeffs%c3(2, 11)
      ! [0, 0, 3] <-- [0, 0, 1, 3]
      cprime%c2(3,5) = 0.25_ki * C(1) * coeffs%c3(2, 11)
      ! [0, 0, 1] <-- [0, 0, 1, 3]
      cprime%c2(1,5) = cprime%c2(1,5) + 0.25_ki * C(3) * coeffs%c3(2, 11)
      ! [0, 2, 3] <-- [0, 0, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(0) * coeffs%c3(3, 11)
      ! [0, 0, 3] <-- [0, 0, 2, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 0.25_ki * C(2) * coeffs%c3(3, 11)
      ! [0, 0, 2] <-- [0, 0, 2, 3]
      cprime%c2(2,5) = cprime%c2(2,5) + 0.25_ki * C(3) * coeffs%c3(3, 11)
      ! [1, 2, 3] <-- [1, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(1) * coeffs%c3(4, 11)
      ! [1, 1, 3] <-- [1, 1, 2, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 0.25_ki * C(2) * coeffs%c3(4, 11)
      ! [1, 1, 2] <-- [1, 1, 2, 3]
      cprime%c2(4,5) = cprime%c2(4,5) + 0.25_ki * C(3) * coeffs%c3(4, 11)
      ! [0, 0, 1] <-- [0, 0, 0, 1]
      cprime%c2(1,5) = cprime%c2(1,5) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(1, 10)
      ! [0, 0, 0] <-- [0, 0, 0, 1]
      cprime%c1(1,3) = 0.25_ki * C(1) * coeffs%c2(1, 10)
      ! [0, 0, 2] <-- [0, 0, 0, 2]
      cprime%c2(2,5) = cprime%c2(2,5) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(2, 10)
      ! [0, 0, 0] <-- [0, 0, 0, 2]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(2) * coeffs%c2(2, 10)
      ! [0, 0, 3] <-- [0, 0, 0, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 3.0_ki/4.0_ki * C(0) * coeffs%c2(3, 10)
      ! [0, 0, 0] <-- [0, 0, 0, 3]
      cprime%c1(1,3) = cprime%c1(1,3) + 0.25_ki * C(3) * coeffs%c2(3, 10)
      ! [1, 1, 2] <-- [1, 1, 1, 2]
      cprime%c2(4,5) = cprime%c2(4,5) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(4, 10)
      ! [1, 1, 1] <-- [1, 1, 1, 2]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(2) * coeffs%c2(4, 10)
      ! [1, 1, 3] <-- [1, 1, 1, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 3.0_ki/4.0_ki * C(1) * coeffs%c2(5, 10)
      ! [1, 1, 1] <-- [1, 1, 1, 3]
      cprime%c1(2,3) = cprime%c1(2,3) + 0.25_ki * C(3) * coeffs%c2(5, 10)
      ! [2, 2, 3] <-- [2, 2, 2, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 3.0_ki/4.0_ki * C(2) * coeffs%c2(6, 10)
      ! [2, 2, 2] <-- [2, 2, 2, 3]
      cprime%c1(3,3) = cprime%c1(3,3) + 0.25_ki * C(3) * coeffs%c2(6, 10)
      ! [1, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.25_ki * C(0) * coeffs%c4(1, 1)
      ! [0, 2, 3] <-- [0, 1, 2, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.25_ki * C(1) * coeffs%c4(1, 1)
      ! [0, 1, 3] <-- [0, 1, 2, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.25_ki * C(2) * coeffs%c4(1, 1)
      ! [0, 1, 2] <-- [0, 1, 2, 3]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.25_ki * C(3) * coeffs%c4(1, 1)
      ! [1, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.25_ki * C(0) * coeffs%c3(1, 2)
      ! [0, 2, 2] <-- [0, 1, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.25_ki * C(1) * coeffs%c3(1, 2)
      ! [0, 1, 2] <-- [0, 1, 2, 2]
      cprime%c3(1,1) = cprime%c3(1,1) + 0.5_ki * C(2) * coeffs%c3(1, 2)
      ! [1, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(0) * coeffs%c3(2, 2)
      ! [0, 3, 3] <-- [0, 1, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(1) * coeffs%c3(2, 2)
      ! [0, 1, 3] <-- [0, 1, 3, 3]
      cprime%c3(2,1) = cprime%c3(2,1) + 0.5_ki * C(3) * coeffs%c3(2, 2)
      ! [2, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(0) * coeffs%c3(3, 2)
      ! [0, 3, 3] <-- [0, 2, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.25_ki * C(2) * coeffs%c3(3, 2)
      ! [0, 2, 3] <-- [0, 2, 3, 3]
      cprime%c3(3,1) = cprime%c3(3,1) + 0.5_ki * C(3) * coeffs%c3(3, 2)
      ! [2, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.25_ki * C(1) * coeffs%c3(4, 2)
      ! [1, 3, 3] <-- [1, 2, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.25_ki * C(2) * coeffs%c3(4, 2)
      ! [1, 2, 3] <-- [1, 2, 3, 3]
      cprime%c3(4,1) = cprime%c3(4,1) + 0.5_ki * C(3) * coeffs%c3(4, 2)
      ! [0, 1, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,2) = cprime%c2(1,2) + 0.5_ki * C(0) * coeffs%c2(1, 7)
      ! [0, 0, 1] <-- [0, 0, 1, 1]
      cprime%c2(1,5) = cprime%c2(1,5) + 0.5_ki * C(1) * coeffs%c2(1, 7)
      ! [0, 2, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,2) = cprime%c2(2,2) + 0.5_ki * C(0) * coeffs%c2(2, 7)
      ! [0, 0, 2] <-- [0, 0, 2, 2]
      cprime%c2(2,5) = cprime%c2(2,5) + 0.5_ki * C(2) * coeffs%c2(2, 7)
      ! [0, 3, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,2) = cprime%c2(3,2) + 0.5_ki * C(0) * coeffs%c2(3, 7)
      ! [0, 0, 3] <-- [0, 0, 3, 3]
      cprime%c2(3,5) = cprime%c2(3,5) + 0.5_ki * C(3) * coeffs%c2(3, 7)
      ! [1, 2, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,2) = cprime%c2(4,2) + 0.5_ki * C(1) * coeffs%c2(4, 7)
      ! [1, 1, 2] <-- [1, 1, 2, 2]
      cprime%c2(4,5) = cprime%c2(4,5) + 0.5_ki * C(2) * coeffs%c2(4, 7)
      ! [1, 3, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,2) = cprime%c2(5,2) + 0.5_ki * C(1) * coeffs%c2(5, 7)
      ! [1, 1, 3] <-- [1, 1, 3, 3]
      cprime%c2(5,5) = cprime%c2(5,5) + 0.5_ki * C(3) * coeffs%c2(5, 7)
      ! [2, 3, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,2) = cprime%c2(6,2) + 0.5_ki * C(2) * coeffs%c2(6, 7)
      ! [2, 2, 3] <-- [2, 2, 3, 3]
      cprime%c2(6,5) = cprime%c2(6,5) + 0.5_ki * C(3) * coeffs%c2(6, 7)
      ! [0, 0, 0] <-- [0, 0, 0, 0]
      cprime%c1(1,3) = cprime%c1(1,3) + C(0) * coeffs%c1(1, 4)
      ! [1, 1, 1] <-- [1, 1, 1, 1]
      cprime%c1(2,3) = cprime%c1(2,3) + C(1) * coeffs%c1(2, 4)
      ! [2, 2, 2] <-- [2, 2, 2, 2]
      cprime%c1(3,3) = cprime%c1(3,3) + C(2) * coeffs%c1(3, 4)
      ! [3, 3, 3] <-- [3, 3, 3, 3]
      cprime%c1(4,3) = cprime%c1(4,3) + C(3) * coeffs%c1(4, 4)
      ! [0, 0, 1, 1] <-- [0, 0, 0, 1, 1]
      cprime%c2(1,6) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(1, 11)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 1]
      cprime%c2(1,8) = 2.0_ki/5.0_ki * C(1) * coeffs%c2(1, 11)
      ! [0, 0, 2, 2] <-- [0, 0, 0, 2, 2]
      cprime%c2(2,6) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(2, 11)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 2, 2]
      cprime%c2(2,8) = 2.0_ki/5.0_ki * C(2) * coeffs%c2(2, 11)
      ! [0, 0, 3, 3] <-- [0, 0, 0, 3, 3]
      cprime%c2(3,6) = 3.0_ki/5.0_ki * C(0) * coeffs%c2(3, 11)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 3, 3]
      cprime%c2(3,8) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(3, 11)
      ! [1, 1, 2, 2] <-- [1, 1, 1, 2, 2]
      cprime%c2(4,6) = 3.0_ki/5.0_ki * C(1) * coeffs%c2(4, 11)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 2, 2]
      cprime%c2(4,8) = 2.0_ki/5.0_ki * C(2) * coeffs%c2(4, 11)
      ! [1, 1, 3, 3] <-- [1, 1, 1, 3, 3]
      cprime%c2(5,6) = 3.0_ki/5.0_ki * C(1) * coeffs%c2(5, 11)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 3, 3]
      cprime%c2(5,8) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(5, 11)
      ! [2, 2, 3, 3] <-- [2, 2, 2, 3, 3]
      cprime%c2(6,6) = 3.0_ki/5.0_ki * C(2) * coeffs%c2(6, 11)
      ! [2, 2, 2, 3] <-- [2, 2, 2, 3, 3]
      cprime%c2(6,8) = 2.0_ki/5.0_ki * C(3) * coeffs%c2(6, 11)
      ! [0, 0, 1, 2] <-- [0, 0, 0, 1, 2]
      cprime%c3(1,7) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(1, 17)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 1, 2]
      cprime%c2(2,8) = cprime%c2(2,8) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 17)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 2]
      cprime%c2(1,8) = cprime%c2(1,8) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 17)
      ! [0, 0, 1, 3] <-- [0, 0, 0, 1, 3]
      cprime%c3(2,7) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(2, 17)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 1, 3]
      cprime%c2(3,8) = cprime%c2(3,8) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 17)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 1, 3]
      cprime%c2(1,8) = cprime%c2(1,8) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 17)
      ! [0, 0, 2, 3] <-- [0, 0, 0, 2, 3]
      cprime%c3(3,7) = 3.0_ki/5.0_ki * C(0) * coeffs%c3(3, 17)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 2, 3]
      cprime%c2(3,8) = cprime%c2(3,8) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 17)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 2, 3]
      cprime%c2(2,8) = cprime%c2(2,8) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 17)
      ! [1, 1, 2, 3] <-- [1, 1, 1, 2, 3]
      cprime%c3(4,7) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(4, 17)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 2, 3]
      cprime%c2(5,8) = cprime%c2(5,8) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 17)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 2, 3]
      cprime%c2(4,8) = cprime%c2(4,8) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 17)
      ! [1, 2, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(4,2) = 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 2)
      ! [0, 2, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(3,2) = 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 2)
      ! [0, 1, 3, 3] <-- [0, 1, 2, 3, 3]
      cprime%c3(2,2) = 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 2)
      ! [0, 1, 2, 3] <-- [0, 1, 2, 3, 3]
      cprime%c4(1,1) = 2.0_ki/5.0_ki * C(3) * coeffs%c4(1, 2)
      ! [1, 2, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c2(4,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 3)
      ! [0, 2, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c2(2,3) = 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 3)
      ! [0, 1, 2, 2] <-- [0, 1, 2, 2, 2]
      cprime%c3(1,2) = 3.0_ki/5.0_ki * C(2) * coeffs%c3(1, 3)
      ! [1, 3, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c2(5,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 3)
      ! [0, 3, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c2(3,3) = 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 3)
      ! [0, 1, 3, 3] <-- [0, 1, 3, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(2, 3)
      ! [2, 3, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c2(6,3) = 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 3)
      ! [0, 3, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 3)
      ! [0, 2, 3, 3] <-- [0, 2, 3, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(3, 3)
      ! [2, 3, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 3)
      ! [1, 3, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 3)
      ! [1, 2, 3, 3] <-- [1, 2, 3, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 3.0_ki/5.0_ki * C(3) * coeffs%c3(4, 3)
      ! [0, 1, 2, 3] <-- [0, 0, 1, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(0) * coeffs%c4(1, 11)
      ! [0, 0, 2, 3] <-- [0, 0, 1, 2, 3]
      cprime%c3(3,7) = cprime%c3(3,7) + 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 11)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 2, 3]
      cprime%c3(2,7) = cprime%c3(2,7) + 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 11)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 2, 3]
      cprime%c3(1,7) = cprime%c3(1,7) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 11)
      ! [1, 2, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c3(4,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 4)
      ! [0, 2, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c3(3,4) = 1.0_ki/5.0_ki * C(1) * coeffs%c4(1, 4)
      ! [0, 1, 2, 3] <-- [0, 1, 2, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(2) * coeffs%c4(1, 4)
      ! [0, 1, 2, 2] <-- [0, 1, 2, 2, 3]
      cprime%c3(1,2) = cprime%c3(1,2) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 4)
      ! [1, 1, 1, 2] <-- [0, 1, 1, 1, 2]
      cprime%c2(4,8) = cprime%c2(4,8) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 8)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 1, 2]
      cprime%c3(1,4) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(1, 8)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 2]
      cprime%c2(1,3) = 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 8)
      ! [1, 1, 1, 3] <-- [0, 1, 1, 1, 3]
      cprime%c2(5,8) = cprime%c2(5,8) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 8)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 1, 3]
      cprime%c3(2,4) = 3.0_ki/5.0_ki * C(1) * coeffs%c3(2, 8)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 3]
      cprime%c2(1,3) = cprime%c2(1,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 8)
      ! [2, 2, 2, 3] <-- [0, 2, 2, 2, 3]
      cprime%c2(6,8) = cprime%c2(6,8) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 8)
      ! [0, 2, 2, 3] <-- [0, 2, 2, 2, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 3.0_ki/5.0_ki * C(2) * coeffs%c3(3, 8)
      ! [0, 2, 2, 2] <-- [0, 2, 2, 2, 3]
      cprime%c2(2,3) = cprime%c2(2,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 8)
      ! [2, 2, 2, 3] <-- [1, 2, 2, 2, 3]
      cprime%c2(6,8) = cprime%c2(6,8) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 8)
      ! [1, 2, 2, 3] <-- [1, 2, 2, 2, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 3.0_ki/5.0_ki * C(2) * coeffs%c3(4, 8)
      ! [1, 2, 2, 2] <-- [1, 2, 2, 2, 3]
      cprime%c2(4,3) = cprime%c2(4,3) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 8)
      ! [1, 1, 1, 1] <-- [0, 1, 1, 1, 1]
      cprime%c1(2,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(1, 4)
      ! [0, 1, 1, 1] <-- [0, 1, 1, 1, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(1, 4)
      ! [2, 2, 2, 2] <-- [0, 2, 2, 2, 2]
      cprime%c1(3,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(2, 4)
      ! [0, 2, 2, 2] <-- [0, 2, 2, 2, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(2, 4)
      ! [3, 3, 3, 3] <-- [0, 3, 3, 3, 3]
      cprime%c1(4,4) = 1.0_ki/5.0_ki * C(0) * coeffs%c2(3, 4)
      ! [0, 3, 3, 3] <-- [0, 3, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(3, 4)
      ! [2, 2, 2, 2] <-- [1, 2, 2, 2, 2]
      cprime%c1(3,4) = cprime%c1(3,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(4, 4)
      ! [1, 2, 2, 2] <-- [1, 2, 2, 2, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(4, 4)
      ! [3, 3, 3, 3] <-- [1, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(5, 4)
      ! [1, 3, 3, 3] <-- [1, 3, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(5, 4)
      ! [3, 3, 3, 3] <-- [2, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(6, 4)
      ! [2, 3, 3, 3] <-- [2, 3, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 4.0_ki/5.0_ki * C(3) * coeffs%c2(6, 4)
      ! [1, 1, 2, 2] <-- [0, 1, 1, 2, 2]
      cprime%c2(4,6) = cprime%c2(4,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(1, 6)
      ! [0, 1, 2, 2] <-- [0, 1, 1, 2, 2]
      cprime%c3(1,2) = cprime%c3(1,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(1, 6)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 2, 2]
      cprime%c3(1,4) = cprime%c3(1,4) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(1, 6)
      ! [1, 1, 3, 3] <-- [0, 1, 1, 3, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(2, 6)
      ! [0, 1, 3, 3] <-- [0, 1, 1, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(2, 6)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 3, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(2, 6)
      ! [2, 2, 3, 3] <-- [0, 2, 2, 3, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 1.0_ki/5.0_ki * C(0) * coeffs%c3(3, 6)
      ! [0, 2, 3, 3] <-- [0, 2, 2, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(3, 6)
      ! [0, 2, 2, 3] <-- [0, 2, 2, 3, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(3, 6)
      ! [2, 2, 3, 3] <-- [1, 2, 2, 3, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(4, 6)
      ! [1, 2, 3, 3] <-- [1, 2, 2, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(4, 6)
      ! [1, 2, 2, 3] <-- [1, 2, 2, 3, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(4, 6)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 0]
      cprime%c1(1,4) = C(0) * coeffs%c1(1, 5)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 1]
      cprime%c1(2,4) = cprime%c1(2,4) + C(1) * coeffs%c1(2, 5)
      ! [2, 2, 2, 2] <-- [2, 2, 2, 2, 2]
      cprime%c1(3,4) = cprime%c1(3,4) + C(2) * coeffs%c1(3, 5)
      ! [3, 3, 3, 3] <-- [3, 3, 3, 3, 3]
      cprime%c1(4,4) = cprime%c1(4,4) + C(3) * coeffs%c1(4, 5)
      ! [0, 1, 1, 1] <-- [0, 0, 1, 1, 1]
      cprime%c2(1,3) = cprime%c2(1,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(1, 8)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 1]
      cprime%c2(1,6) = cprime%c2(1,6) + 3.0_ki/5.0_ki * C(1) * coeffs%c2(1, 8)
      ! [0, 2, 2, 2] <-- [0, 0, 2, 2, 2]
      cprime%c2(2,3) = cprime%c2(2,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(2, 8)
      ! [0, 0, 2, 2] <-- [0, 0, 2, 2, 2]
      cprime%c2(2,6) = cprime%c2(2,6) + 3.0_ki/5.0_ki * C(2) * coeffs%c2(2, 8)
      ! [0, 3, 3, 3] <-- [0, 0, 3, 3, 3]
      cprime%c2(3,3) = cprime%c2(3,3) + 2.0_ki/5.0_ki * C(0) * coeffs%c2(3, 8)
      ! [0, 0, 3, 3] <-- [0, 0, 3, 3, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(3, 8)
      ! [1, 2, 2, 2] <-- [1, 1, 2, 2, 2]
      cprime%c2(4,3) = cprime%c2(4,3) + 2.0_ki/5.0_ki * C(1) * coeffs%c2(4, 8)
      ! [1, 1, 2, 2] <-- [1, 1, 2, 2, 2]
      cprime%c2(4,6) = cprime%c2(4,6) + 3.0_ki/5.0_ki * C(2) * coeffs%c2(4, 8)
      ! [1, 3, 3, 3] <-- [1, 1, 3, 3, 3]
      cprime%c2(5,3) = cprime%c2(5,3) + 2.0_ki/5.0_ki * C(1) * coeffs%c2(5, 8)
      ! [1, 1, 3, 3] <-- [1, 1, 3, 3, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(5, 8)
      ! [2, 3, 3, 3] <-- [2, 2, 3, 3, 3]
      cprime%c2(6,3) = cprime%c2(6,3) + 2.0_ki/5.0_ki * C(2) * coeffs%c2(6, 8)
      ! [2, 2, 3, 3] <-- [2, 2, 3, 3, 3]
      cprime%c2(6,6) = cprime%c2(6,6) + 3.0_ki/5.0_ki * C(3) * coeffs%c2(6, 8)
      ! [0, 1, 2, 2] <-- [0, 0, 1, 2, 2]
      cprime%c3(1,2) = cprime%c3(1,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(1, 12)
      ! [0, 0, 2, 2] <-- [0, 0, 1, 2, 2]
      cprime%c2(2,6) = cprime%c2(2,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(1, 12)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 2, 2]
      cprime%c3(1,7) = cprime%c3(1,7) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(1, 12)
      ! [0, 1, 3, 3] <-- [0, 0, 1, 3, 3]
      cprime%c3(2,2) = cprime%c3(2,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(2, 12)
      ! [0, 0, 3, 3] <-- [0, 0, 1, 3, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 1.0_ki/5.0_ki * C(1) * coeffs%c3(2, 12)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 3, 3]
      cprime%c3(2,7) = cprime%c3(2,7) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(2, 12)
      ! [0, 2, 3, 3] <-- [0, 0, 2, 3, 3]
      cprime%c3(3,2) = cprime%c3(3,2) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(3, 12)
      ! [0, 0, 3, 3] <-- [0, 0, 2, 3, 3]
      cprime%c2(3,6) = cprime%c2(3,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(3, 12)
      ! [0, 0, 2, 3] <-- [0, 0, 2, 3, 3]
      cprime%c3(3,7) = cprime%c3(3,7) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(3, 12)
      ! [1, 2, 3, 3] <-- [1, 1, 2, 3, 3]
      cprime%c3(4,2) = cprime%c3(4,2) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(4, 12)
      ! [1, 1, 3, 3] <-- [1, 1, 2, 3, 3]
      cprime%c2(5,6) = cprime%c2(5,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(4, 12)
      ! [1, 1, 2, 3] <-- [1, 1, 2, 3, 3]
      cprime%c3(4,7) = cprime%c3(4,7) + 2.0_ki/5.0_ki * C(3) * coeffs%c3(4, 12)
      ! [0, 0, 0, 1] <-- [0, 0, 0, 0, 1]
      cprime%c2(1,8) = cprime%c2(1,8) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(1, 13)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 1]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(1) * coeffs%c2(1, 13)
      ! [0, 0, 0, 2] <-- [0, 0, 0, 0, 2]
      cprime%c2(2,8) = cprime%c2(2,8) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(2, 13)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 2]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(2, 13)
      ! [0, 0, 0, 3] <-- [0, 0, 0, 0, 3]
      cprime%c2(3,8) = cprime%c2(3,8) + 4.0_ki/5.0_ki * C(0) * coeffs%c2(3, 13)
      ! [0, 0, 0, 0] <-- [0, 0, 0, 0, 3]
      cprime%c1(1,4) = cprime%c1(1,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(3, 13)
      ! [1, 1, 1, 2] <-- [1, 1, 1, 1, 2]
      cprime%c2(4,8) = cprime%c2(4,8) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(4, 13)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 2]
      cprime%c1(2,4) = cprime%c1(2,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c2(4, 13)
      ! [1, 1, 1, 3] <-- [1, 1, 1, 1, 3]
      cprime%c2(5,8) = cprime%c2(5,8) + 4.0_ki/5.0_ki * C(1) * coeffs%c2(5, 13)
      ! [1, 1, 1, 1] <-- [1, 1, 1, 1, 3]
      cprime%c1(2,4) = cprime%c1(2,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(5, 13)
      ! [2, 2, 2, 3] <-- [2, 2, 2, 2, 3]
      cprime%c2(6,8) = cprime%c2(6,8) + 4.0_ki/5.0_ki * C(2) * coeffs%c2(6, 13)
      ! [2, 2, 2, 2] <-- [2, 2, 2, 2, 3]
      cprime%c1(3,4) = cprime%c1(3,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c2(6, 13)
      ! [0, 1, 1, 2] <-- [0, 0, 1, 1, 2]
      cprime%c3(1,4) = cprime%c3(1,4) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(1, 14)
      ! [0, 0, 1, 2] <-- [0, 0, 1, 1, 2]
      cprime%c3(1,7) = cprime%c3(1,7) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(1, 14)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 2]
      cprime%c2(1,6) = cprime%c2(1,6) + 1.0_ki/5.0_ki * C(2) * coeffs%c3(1, 14)
      ! [0, 1, 1, 3] <-- [0, 0, 1, 1, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(2, 14)
      ! [0, 0, 1, 3] <-- [0, 0, 1, 1, 3]
      cprime%c3(2,7) = cprime%c3(2,7) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(2, 14)
      ! [0, 0, 1, 1] <-- [0, 0, 1, 1, 3]
      cprime%c2(1,6) = cprime%c2(1,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(2, 14)
      ! [0, 2, 2, 3] <-- [0, 0, 2, 2, 3]
      cprime%c3(3,4) = cprime%c3(3,4) + 2.0_ki/5.0_ki * C(0) * coeffs%c3(3, 14)
      ! [0, 0, 2, 3] <-- [0, 0, 2, 2, 3]
      cprime%c3(3,7) = cprime%c3(3,7) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(3, 14)
      ! [0, 0, 2, 2] <-- [0, 0, 2, 2, 3]
      cprime%c2(2,6) = cprime%c2(2,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(3, 14)
      ! [1, 2, 2, 3] <-- [1, 1, 2, 2, 3]
      cprime%c3(4,4) = cprime%c3(4,4) + 2.0_ki/5.0_ki * C(1) * coeffs%c3(4, 14)
      ! [1, 1, 2, 3] <-- [1, 1, 2, 2, 3]
      cprime%c3(4,7) = cprime%c3(4,7) + 2.0_ki/5.0_ki * C(2) * coeffs%c3(4, 14)
      ! [1, 1, 2, 2] <-- [1, 1, 2, 2, 3]
      cprime%c2(4,6) = cprime%c2(4,6) + 1.0_ki/5.0_ki * C(3) * coeffs%c3(4, 14)
      ! [1, 1, 2, 3] <-- [0, 1, 1, 2, 3]
      cprime%c3(4,7) = cprime%c3(4,7) + 1.0_ki/5.0_ki * C(0) * coeffs%c4(1, 7)
      ! [0, 1, 2, 3] <-- [0, 1, 1, 2, 3]
      cprime%c4(1,1) = cprime%c4(1,1) + 2.0_ki/5.0_ki * C(1) * coeffs%c4(1, 7)
      ! [0, 1, 1, 3] <-- [0, 1, 1, 2, 3]
      cprime%c3(2,4) = cprime%c3(2,4) + 1.0_ki/5.0_ki * C(2) * coeffs%c4(1, 7)
      ! [0, 1, 1, 2] <-- [0, 1, 1, 2, 3]
      cprime%c3(1,4) = cprime%c3(1,4) + 1.0_ki/5.0_ki * C(3) * coeffs%c4(1, 7)
      ! [0, 1, 1, 2, 3] <-- [0, 0, 1, 1, 2, 3]
      cprime%c4(1,4) = 1.0_ki/3.0_ki * C(0) * coeffs%c4(1, 14)
      ! [0, 0, 1, 2, 3] <-- [0, 0, 1, 1, 2, 3]
      cprime%c4(1,5) = 1.0_ki/3.0_ki * C(1) * coeffs%c4(1, 14)
      ! [0, 0, 1, 1, 3] <-- [0, 0, 1, 1, 2, 3]
      cprime%c3(2,9) = 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 14)
      ! [0, 0, 1, 1, 2] <-- [0, 0, 1, 1, 2, 3]
      cprime%c3(1,9) = 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 14)
      ! [1, 1, 1, 2, 2] <-- [0, 1, 1, 1, 2, 2]
      cprime%c2(4,9) = 1.0_ki/6.0_ki * C(0) * coeffs%c3(1, 9)
      ! [0, 1, 1, 2, 2] <-- [0, 1, 1, 1, 2, 2]
      cprime%c3(1,5) = 0.5_ki * C(1) * coeffs%c3(1, 9)
      ! [0, 1, 1, 1, 2] <-- [0, 1, 1, 1, 2, 2]
      cprime%c3(1,6) = 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 9)
      ! [1, 1, 1, 3, 3] <-- [0, 1, 1, 1, 3, 3]
      cprime%c2(5,9) = 1.0_ki/6.0_ki * C(0) * coeffs%c3(2, 9)
      ! [0, 1, 1, 3, 3] <-- [0, 1, 1, 1, 3, 3]
      cprime%c3(2,5) = 0.5_ki * C(1) * coeffs%c3(2, 9)
      ! [0, 1, 1, 1, 3] <-- [0, 1, 1, 1, 3, 3]
      cprime%c3(2,6) = 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 9)
      ! [2, 2, 2, 3, 3] <-- [0, 2, 2, 2, 3, 3]
      cprime%c2(6,9) = 1.0_ki/6.0_ki * C(0) * coeffs%c3(3, 9)
      ! [0, 2, 2, 3, 3] <-- [0, 2, 2, 2, 3, 3]
      cprime%c3(3,5) = 0.5_ki * C(2) * coeffs%c3(3, 9)
      ! [0, 2, 2, 2, 3] <-- [0, 2, 2, 2, 3, 3]
      cprime%c3(3,6) = 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 9)
      ! [2, 2, 2, 3, 3] <-- [1, 2, 2, 2, 3, 3]
      cprime%c2(6,9) = cprime%c2(6,9) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(4, 9)
      ! [1, 2, 2, 3, 3] <-- [1, 2, 2, 2, 3, 3]
      cprime%c3(4,5) = 0.5_ki * C(2) * coeffs%c3(4, 9)
      ! [1, 2, 2, 2, 3] <-- [1, 2, 2, 2, 3, 3]
      cprime%c3(4,6) = 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 9)
      ! [1, 2, 2, 2, 3] <-- [0, 1, 2, 2, 2, 3]
      cprime%c3(4,6) = cprime%c3(4,6) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 6)
      ! [0, 2, 2, 2, 3] <-- [0, 1, 2, 2, 2, 3]
      cprime%c3(3,6) = cprime%c3(3,6) + 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 6)
      ! [0, 1, 2, 2, 3] <-- [0, 1, 2, 2, 2, 3]
      cprime%c4(1,3) = 0.5_ki * C(2) * coeffs%c4(1, 6)
      ! [0, 1, 2, 2, 2] <-- [0, 1, 2, 2, 2, 3]
      cprime%c3(1,3) = 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 6)
      ! [0, 0, 0, 0, 1] <-- [0, 0, 0, 0, 0, 1]
      cprime%c2(1,10) = 5.0_ki/6.0_ki * C(0) * coeffs%c2(1, 15)
      ! [0, 0, 0, 0, 0] <-- [0, 0, 0, 0, 0, 1]
      cprime%c1(1,5) = 1.0_ki/6.0_ki * C(1) * coeffs%c2(1, 15)
      ! [0, 0, 0, 0, 2] <-- [0, 0, 0, 0, 0, 2]
      cprime%c2(2,10) = 5.0_ki/6.0_ki * C(0) * coeffs%c2(2, 15)
      ! [0, 0, 0, 0, 0] <-- [0, 0, 0, 0, 0, 2]
      cprime%c1(1,5) = cprime%c1(1,5) + 1.0_ki/6.0_ki * C(2) * coeffs%c2(2, 15)
      ! [0, 0, 0, 0, 3] <-- [0, 0, 0, 0, 0, 3]
      cprime%c2(3,10) = 5.0_ki/6.0_ki * C(0) * coeffs%c2(3, 15)
      ! [0, 0, 0, 0, 0] <-- [0, 0, 0, 0, 0, 3]
      cprime%c1(1,5) = cprime%c1(1,5) + 1.0_ki/6.0_ki * C(3) * coeffs%c2(3, 15)
      ! [1, 1, 1, 1, 2] <-- [1, 1, 1, 1, 1, 2]
      cprime%c2(4,10) = 5.0_ki/6.0_ki * C(1) * coeffs%c2(4, 15)
      ! [1, 1, 1, 1, 1] <-- [1, 1, 1, 1, 1, 2]
      cprime%c1(2,5) = 1.0_ki/6.0_ki * C(2) * coeffs%c2(4, 15)
      ! [1, 1, 1, 1, 3] <-- [1, 1, 1, 1, 1, 3]
      cprime%c2(5,10) = 5.0_ki/6.0_ki * C(1) * coeffs%c2(5, 15)
      ! [1, 1, 1, 1, 1] <-- [1, 1, 1, 1, 1, 3]
      cprime%c1(2,5) = cprime%c1(2,5) + 1.0_ki/6.0_ki * C(3) * coeffs%c2(5, 15)
      ! [2, 2, 2, 2, 3] <-- [2, 2, 2, 2, 2, 3]
      cprime%c2(6,10) = 5.0_ki/6.0_ki * C(2) * coeffs%c2(6, 15)
      ! [2, 2, 2, 2, 2] <-- [2, 2, 2, 2, 2, 3]
      cprime%c1(3,5) = 1.0_ki/6.0_ki * C(3) * coeffs%c2(6, 15)
      ! [0, 1, 1, 1, 2] <-- [0, 0, 1, 1, 1, 2]
      cprime%c3(1,6) = cprime%c3(1,6) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 16)
      ! [0, 0, 1, 1, 2] <-- [0, 0, 1, 1, 1, 2]
      cprime%c3(1,9) = cprime%c3(1,9) + 0.5_ki * C(1) * coeffs%c3(1, 16)
      ! [0, 0, 1, 1, 1] <-- [0, 0, 1, 1, 1, 2]
      cprime%c2(1,7) = 1.0_ki/6.0_ki * C(2) * coeffs%c3(1, 16)
      ! [0, 1, 1, 1, 3] <-- [0, 0, 1, 1, 1, 3]
      cprime%c3(2,6) = cprime%c3(2,6) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 16)
      ! [0, 0, 1, 1, 3] <-- [0, 0, 1, 1, 1, 3]
      cprime%c3(2,9) = cprime%c3(2,9) + 0.5_ki * C(1) * coeffs%c3(2, 16)
      ! [0, 0, 1, 1, 1] <-- [0, 0, 1, 1, 1, 3]
      cprime%c2(1,7) = cprime%c2(1,7) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(2, 16)
      ! [0, 2, 2, 2, 3] <-- [0, 0, 2, 2, 2, 3]
      cprime%c3(3,6) = cprime%c3(3,6) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 16)
      ! [0, 0, 2, 2, 3] <-- [0, 0, 2, 2, 2, 3]
      cprime%c3(3,9) = 0.5_ki * C(2) * coeffs%c3(3, 16)
      ! [0, 0, 2, 2, 2] <-- [0, 0, 2, 2, 2, 3]
      cprime%c2(2,7) = 1.0_ki/6.0_ki * C(3) * coeffs%c3(3, 16)
      ! [1, 2, 2, 2, 3] <-- [1, 1, 2, 2, 2, 3]
      cprime%c3(4,6) = cprime%c3(4,6) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 16)
      ! [1, 1, 2, 2, 3] <-- [1, 1, 2, 2, 2, 3]
      cprime%c3(4,9) = 0.5_ki * C(2) * coeffs%c3(4, 16)
      ! [1, 1, 2, 2, 2] <-- [1, 1, 2, 2, 2, 3]
      cprime%c2(4,7) = 1.0_ki/6.0_ki * C(3) * coeffs%c3(4, 16)
      ! [1, 2, 2, 3, 3] <-- [0, 1, 2, 2, 3, 3]
      cprime%c3(4,5) = cprime%c3(4,5) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 5)
      ! [0, 2, 2, 3, 3] <-- [0, 1, 2, 2, 3, 3]
      cprime%c3(3,5) = cprime%c3(3,5) + 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 5)
      ! [0, 1, 2, 3, 3] <-- [0, 1, 2, 2, 3, 3]
      cprime%c4(1,2) = 1.0_ki/3.0_ki * C(2) * coeffs%c4(1, 5)
      ! [0, 1, 2, 2, 3] <-- [0, 1, 2, 2, 3, 3]
      cprime%c4(1,3) = cprime%c4(1,3) + 1.0_ki/3.0_ki * C(3) * coeffs%c4(1, 5)
      ! [0, 1, 2, 2, 3] <-- [0, 0, 1, 2, 2, 3]
      cprime%c4(1,3) = cprime%c4(1,3) + 1.0_ki/3.0_ki * C(0) * coeffs%c4(1, 13)
      ! [0, 0, 2, 2, 3] <-- [0, 0, 1, 2, 2, 3]
      cprime%c3(3,9) = cprime%c3(3,9) + 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 13)
      ! [0, 0, 1, 2, 3] <-- [0, 0, 1, 2, 2, 3]
      cprime%c4(1,5) = cprime%c4(1,5) + 1.0_ki/3.0_ki * C(2) * coeffs%c4(1, 13)
      ! [0, 0, 1, 2, 2] <-- [0, 0, 1, 2, 2, 3]
      cprime%c3(1,8) = 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 13)
      ! [0, 0, 1, 1, 1] <-- [0, 0, 0, 1, 1, 1]
      cprime%c2(1,7) = cprime%c2(1,7) + 0.5_ki * C(0) * coeffs%c2(1, 12)
      ! [0, 0, 0, 1, 1] <-- [0, 0, 0, 1, 1, 1]
      cprime%c2(1,9) = 0.5_ki * C(1) * coeffs%c2(1, 12)
      ! [0, 0, 2, 2, 2] <-- [0, 0, 0, 2, 2, 2]
      cprime%c2(2,7) = cprime%c2(2,7) + 0.5_ki * C(0) * coeffs%c2(2, 12)
      ! [0, 0, 0, 2, 2] <-- [0, 0, 0, 2, 2, 2]
      cprime%c2(2,9) = 0.5_ki * C(2) * coeffs%c2(2, 12)
      ! [0, 0, 3, 3, 3] <-- [0, 0, 0, 3, 3, 3]
      cprime%c2(3,7) = 0.5_ki * C(0) * coeffs%c2(3, 12)
      ! [0, 0, 0, 3, 3] <-- [0, 0, 0, 3, 3, 3]
      cprime%c2(3,9) = 0.5_ki * C(3) * coeffs%c2(3, 12)
      ! [1, 1, 2, 2, 2] <-- [1, 1, 1, 2, 2, 2]
      cprime%c2(4,7) = cprime%c2(4,7) + 0.5_ki * C(1) * coeffs%c2(4, 12)
      ! [1, 1, 1, 2, 2] <-- [1, 1, 1, 2, 2, 2]
      cprime%c2(4,9) = cprime%c2(4,9) + 0.5_ki * C(2) * coeffs%c2(4, 12)
      ! [1, 1, 3, 3, 3] <-- [1, 1, 1, 3, 3, 3]
      cprime%c2(5,7) = 0.5_ki * C(1) * coeffs%c2(5, 12)
      ! [1, 1, 1, 3, 3] <-- [1, 1, 1, 3, 3, 3]
      cprime%c2(5,9) = cprime%c2(5,9) + 0.5_ki * C(3) * coeffs%c2(5, 12)
      ! [2, 2, 3, 3, 3] <-- [2, 2, 2, 3, 3, 3]
      cprime%c2(6,7) = 0.5_ki * C(2) * coeffs%c2(6, 12)
      ! [2, 2, 2, 3, 3] <-- [2, 2, 2, 3, 3, 3]
      cprime%c2(6,9) = cprime%c2(6,9) + 0.5_ki * C(3) * coeffs%c2(6, 12)
      ! [1, 1, 1, 1, 2] <-- [0, 1, 1, 1, 1, 2]
      cprime%c2(4,10) = cprime%c2(4,10) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(1, 10)
      ! [0, 1, 1, 1, 2] <-- [0, 1, 1, 1, 1, 2]
      cprime%c3(1,6) = cprime%c3(1,6) + 2.0_ki/3.0_ki * C(1) * coeffs%c3(1, 10)
      ! [0, 1, 1, 1, 1] <-- [0, 1, 1, 1, 1, 2]
      cprime%c2(1,4) = 1.0_ki/6.0_ki * C(2) * coeffs%c3(1, 10)
      ! [1, 1, 1, 1, 3] <-- [0, 1, 1, 1, 1, 3]
      cprime%c2(5,10) = cprime%c2(5,10) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(2, 10)
      ! [0, 1, 1, 1, 3] <-- [0, 1, 1, 1, 1, 3]
      cprime%c3(2,6) = cprime%c3(2,6) + 2.0_ki/3.0_ki * C(1) * coeffs%c3(2, 10)
      ! [0, 1, 1, 1, 1] <-- [0, 1, 1, 1, 1, 3]
      cprime%c2(1,4) = cprime%c2(1,4) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(2, 10)
      ! [2, 2, 2, 2, 3] <-- [0, 2, 2, 2, 2, 3]
      cprime%c2(6,10) = cprime%c2(6,10) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(3, 10)
      ! [0, 2, 2, 2, 3] <-- [0, 2, 2, 2, 2, 3]
      cprime%c3(3,6) = cprime%c3(3,6) + 2.0_ki/3.0_ki * C(2) * coeffs%c3(3, 10)
      ! [0, 2, 2, 2, 2] <-- [0, 2, 2, 2, 2, 3]
      cprime%c2(2,4) = 1.0_ki/6.0_ki * C(3) * coeffs%c3(3, 10)
      ! [2, 2, 2, 2, 3] <-- [1, 2, 2, 2, 2, 3]
      cprime%c2(6,10) = cprime%c2(6,10) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(4, 10)
      ! [1, 2, 2, 2, 3] <-- [1, 2, 2, 2, 2, 3]
      cprime%c3(4,6) = cprime%c3(4,6) + 2.0_ki/3.0_ki * C(2) * coeffs%c3(4, 10)
      ! [1, 2, 2, 2, 2] <-- [1, 2, 2, 2, 2, 3]
      cprime%c2(4,4) = 1.0_ki/6.0_ki * C(3) * coeffs%c3(4, 10)
      ! [1, 1, 2, 2, 3] <-- [0, 1, 1, 2, 2, 3]
      cprime%c3(4,9) = cprime%c3(4,9) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 9)
      ! [0, 1, 2, 2, 3] <-- [0, 1, 1, 2, 2, 3]
      cprime%c4(1,3) = cprime%c4(1,3) + 1.0_ki/3.0_ki * C(1) * coeffs%c4(1, 9)
      ! [0, 1, 1, 2, 3] <-- [0, 1, 1, 2, 2, 3]
      cprime%c4(1,4) = cprime%c4(1,4) + 1.0_ki/3.0_ki * C(2) * coeffs%c4(1, 9)
      ! [0, 1, 1, 2, 2] <-- [0, 1, 1, 2, 2, 3]
      cprime%c3(1,5) = cprime%c3(1,5) + 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 9)
      ! [0, 0, 1, 2, 3] <-- [0, 0, 0, 1, 2, 3]
      cprime%c4(1,5) = cprime%c4(1,5) + 0.5_ki * C(0) * coeffs%c4(1, 15)
      ! [0, 0, 0, 2, 3] <-- [0, 0, 0, 1, 2, 3]
      cprime%c3(3,10) = 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 15)
      ! [0, 0, 0, 1, 3] <-- [0, 0, 0, 1, 2, 3]
      cprime%c3(2,10) = 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 15)
      ! [0, 0, 0, 1, 2] <-- [0, 0, 0, 1, 2, 3]
      cprime%c3(1,10) = 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 15)
      ! [1, 1, 1, 1, 1] <-- [0, 1, 1, 1, 1, 1]
      cprime%c1(2,5) = cprime%c1(2,5) + 1.0_ki/6.0_ki * C(0) * coeffs%c2(1, 5)
      ! [0, 1, 1, 1, 1] <-- [0, 1, 1, 1, 1, 1]
      cprime%c2(1,4) = cprime%c2(1,4) + 5.0_ki/6.0_ki * C(1) * coeffs%c2(1, 5)
      ! [2, 2, 2, 2, 2] <-- [0, 2, 2, 2, 2, 2]
      cprime%c1(3,5) = cprime%c1(3,5) + 1.0_ki/6.0_ki * C(0) * coeffs%c2(2, 5)
      ! [0, 2, 2, 2, 2] <-- [0, 2, 2, 2, 2, 2]
      cprime%c2(2,4) = cprime%c2(2,4) + 5.0_ki/6.0_ki * C(2) * coeffs%c2(2, 5)
      ! [3, 3, 3, 3, 3] <-- [0, 3, 3, 3, 3, 3]
      cprime%c1(4,5) = 1.0_ki/6.0_ki * C(0) * coeffs%c2(3, 5)
      ! [0, 3, 3, 3, 3] <-- [0, 3, 3, 3, 3, 3]
      cprime%c2(3,4) = 5.0_ki/6.0_ki * C(3) * coeffs%c2(3, 5)
      ! [2, 2, 2, 2, 2] <-- [1, 2, 2, 2, 2, 2]
      cprime%c1(3,5) = cprime%c1(3,5) + 1.0_ki/6.0_ki * C(1) * coeffs%c2(4, 5)
      ! [1, 2, 2, 2, 2] <-- [1, 2, 2, 2, 2, 2]
      cprime%c2(4,4) = cprime%c2(4,4) + 5.0_ki/6.0_ki * C(2) * coeffs%c2(4, 5)
      ! [3, 3, 3, 3, 3] <-- [1, 3, 3, 3, 3, 3]
      cprime%c1(4,5) = cprime%c1(4,5) + 1.0_ki/6.0_ki * C(1) * coeffs%c2(5, 5)
      ! [1, 3, 3, 3, 3] <-- [1, 3, 3, 3, 3, 3]
      cprime%c2(5,4) = 5.0_ki/6.0_ki * C(3) * coeffs%c2(5, 5)
      ! [3, 3, 3, 3, 3] <-- [2, 3, 3, 3, 3, 3]
      cprime%c1(4,5) = cprime%c1(4,5) + 1.0_ki/6.0_ki * C(2) * coeffs%c2(6, 5)
      ! [2, 3, 3, 3, 3] <-- [2, 3, 3, 3, 3, 3]
      cprime%c2(6,4) = 5.0_ki/6.0_ki * C(3) * coeffs%c2(6, 5)
      ! [0, 1, 2, 3, 3] <-- [0, 0, 1, 2, 3, 3]
      cprime%c4(1,2) = cprime%c4(1,2) + 1.0_ki/3.0_ki * C(0) * coeffs%c4(1, 12)
      ! [0, 0, 2, 3, 3] <-- [0, 0, 1, 2, 3, 3]
      cprime%c3(3,8) = 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 12)
      ! [0, 0, 1, 3, 3] <-- [0, 0, 1, 2, 3, 3]
      cprime%c3(2,8) = 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 12)
      ! [0, 0, 1, 2, 3] <-- [0, 0, 1, 2, 3, 3]
      cprime%c4(1,5) = cprime%c4(1,5) + 1.0_ki/3.0_ki * C(3) * coeffs%c4(1, 12)
      ! [0, 0, 1, 2, 2] <-- [0, 0, 0, 1, 2, 2]
      cprime%c3(1,8) = cprime%c3(1,8) + 0.5_ki * C(0) * coeffs%c3(1, 18)
      ! [0, 0, 0, 2, 2] <-- [0, 0, 0, 1, 2, 2]
      cprime%c2(2,9) = cprime%c2(2,9) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(1, 18)
      ! [0, 0, 0, 1, 2] <-- [0, 0, 0, 1, 2, 2]
      cprime%c3(1,10) = cprime%c3(1,10) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 18)
      ! [0, 0, 1, 3, 3] <-- [0, 0, 0, 1, 3, 3]
      cprime%c3(2,8) = cprime%c3(2,8) + 0.5_ki * C(0) * coeffs%c3(2, 18)
      ! [0, 0, 0, 3, 3] <-- [0, 0, 0, 1, 3, 3]
      cprime%c2(3,9) = cprime%c2(3,9) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(2, 18)
      ! [0, 0, 0, 1, 3] <-- [0, 0, 0, 1, 3, 3]
      cprime%c3(2,10) = cprime%c3(2,10) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 18)
      ! [0, 0, 2, 3, 3] <-- [0, 0, 0, 2, 3, 3]
      cprime%c3(3,8) = cprime%c3(3,8) + 0.5_ki * C(0) * coeffs%c3(3, 18)
      ! [0, 0, 0, 3, 3] <-- [0, 0, 0, 2, 3, 3]
      cprime%c2(3,9) = cprime%c2(3,9) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(3, 18)
      ! [0, 0, 0, 2, 3] <-- [0, 0, 0, 2, 3, 3]
      cprime%c3(3,10) = cprime%c3(3,10) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 18)
      ! [1, 1, 2, 3, 3] <-- [1, 1, 1, 2, 3, 3]
      cprime%c3(4,8) = 0.5_ki * C(1) * coeffs%c3(4, 18)
      ! [1, 1, 1, 3, 3] <-- [1, 1, 1, 2, 3, 3]
      cprime%c2(5,9) = cprime%c2(5,9) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(4, 18)
      ! [1, 1, 1, 2, 3] <-- [1, 1, 1, 2, 3, 3]
      cprime%c3(4,10) = 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 18)
      ! [1, 1, 2, 3, 3] <-- [0, 1, 1, 2, 3, 3]
      cprime%c3(4,8) = cprime%c3(4,8) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 8)
      ! [0, 1, 2, 3, 3] <-- [0, 1, 1, 2, 3, 3]
      cprime%c4(1,2) = cprime%c4(1,2) + 1.0_ki/3.0_ki * C(1) * coeffs%c4(1, 8)
      ! [0, 1, 1, 3, 3] <-- [0, 1, 1, 2, 3, 3]
      cprime%c3(2,5) = cprime%c3(2,5) + 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 8)
      ! [0, 1, 1, 2, 3] <-- [0, 1, 1, 2, 3, 3]
      cprime%c4(1,4) = cprime%c4(1,4) + 1.0_ki/3.0_ki * C(3) * coeffs%c4(1, 8)
      ! [0, 0, 1, 1, 2] <-- [0, 0, 0, 1, 1, 2]
      cprime%c3(1,9) = cprime%c3(1,9) + 0.5_ki * C(0) * coeffs%c3(1, 19)
      ! [0, 0, 0, 1, 2] <-- [0, 0, 0, 1, 1, 2]
      cprime%c3(1,10) = cprime%c3(1,10) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 19)
      ! [0, 0, 0, 1, 1] <-- [0, 0, 0, 1, 1, 2]
      cprime%c2(1,9) = cprime%c2(1,9) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(1, 19)
      ! [0, 0, 1, 1, 3] <-- [0, 0, 0, 1, 1, 3]
      cprime%c3(2,9) = cprime%c3(2,9) + 0.5_ki * C(0) * coeffs%c3(2, 19)
      ! [0, 0, 0, 1, 3] <-- [0, 0, 0, 1, 1, 3]
      cprime%c3(2,10) = cprime%c3(2,10) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 19)
      ! [0, 0, 0, 1, 1] <-- [0, 0, 0, 1, 1, 3]
      cprime%c2(1,9) = cprime%c2(1,9) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(2, 19)
      ! [0, 0, 2, 2, 3] <-- [0, 0, 0, 2, 2, 3]
      cprime%c3(3,9) = cprime%c3(3,9) + 0.5_ki * C(0) * coeffs%c3(3, 19)
      ! [0, 0, 0, 2, 3] <-- [0, 0, 0, 2, 2, 3]
      cprime%c3(3,10) = cprime%c3(3,10) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 19)
      ! [0, 0, 0, 2, 2] <-- [0, 0, 0, 2, 2, 3]
      cprime%c2(2,9) = cprime%c2(2,9) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(3, 19)
      ! [1, 1, 2, 2, 3] <-- [1, 1, 1, 2, 2, 3]
      cprime%c3(4,9) = cprime%c3(4,9) + 0.5_ki * C(1) * coeffs%c3(4, 19)
      ! [1, 1, 1, 2, 3] <-- [1, 1, 1, 2, 2, 3]
      cprime%c3(4,10) = cprime%c3(4,10) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 19)
      ! [1, 1, 1, 2, 2] <-- [1, 1, 1, 2, 2, 3]
      cprime%c2(4,9) = cprime%c2(4,9) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(4, 19)
      ! [1, 1, 2, 2, 2] <-- [0, 1, 1, 2, 2, 2]
      cprime%c2(4,7) = cprime%c2(4,7) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(1, 7)
      ! [0, 1, 2, 2, 2] <-- [0, 1, 1, 2, 2, 2]
      cprime%c3(1,3) = cprime%c3(1,3) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 7)
      ! [0, 1, 1, 2, 2] <-- [0, 1, 1, 2, 2, 2]
      cprime%c3(1,5) = cprime%c3(1,5) + 0.5_ki * C(2) * coeffs%c3(1, 7)
      ! [1, 1, 3, 3, 3] <-- [0, 1, 1, 3, 3, 3]
      cprime%c2(5,7) = cprime%c2(5,7) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(2, 7)
      ! [0, 1, 3, 3, 3] <-- [0, 1, 1, 3, 3, 3]
      cprime%c3(2,3) = 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 7)
      ! [0, 1, 1, 3, 3] <-- [0, 1, 1, 3, 3, 3]
      cprime%c3(2,5) = cprime%c3(2,5) + 0.5_ki * C(3) * coeffs%c3(2, 7)
      ! [2, 2, 3, 3, 3] <-- [0, 2, 2, 3, 3, 3]
      cprime%c2(6,7) = cprime%c2(6,7) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(3, 7)
      ! [0, 2, 3, 3, 3] <-- [0, 2, 2, 3, 3, 3]
      cprime%c3(3,3) = 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 7)
      ! [0, 2, 2, 3, 3] <-- [0, 2, 2, 3, 3, 3]
      cprime%c3(3,5) = cprime%c3(3,5) + 0.5_ki * C(3) * coeffs%c3(3, 7)
      ! [2, 2, 3, 3, 3] <-- [1, 2, 2, 3, 3, 3]
      cprime%c2(6,7) = cprime%c2(6,7) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(4, 7)
      ! [1, 2, 3, 3, 3] <-- [1, 2, 2, 3, 3, 3]
      cprime%c3(4,3) = 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 7)
      ! [1, 2, 2, 3, 3] <-- [1, 2, 2, 3, 3, 3]
      cprime%c3(4,5) = cprime%c3(4,5) + 0.5_ki * C(3) * coeffs%c3(4, 7)
      ! [0, 0, 0, 1, 2] <-- [0, 0, 0, 0, 1, 2]
      cprime%c3(1,10) = cprime%c3(1,10) + 2.0_ki/3.0_ki * C(0) * coeffs%c3(1, 20)
      ! [0, 0, 0, 0, 2] <-- [0, 0, 0, 0, 1, 2]
      cprime%c2(2,10) = cprime%c2(2,10) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(1, 20)
      ! [0, 0, 0, 0, 1] <-- [0, 0, 0, 0, 1, 2]
      cprime%c2(1,10) = cprime%c2(1,10) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(1, 20)
      ! [0, 0, 0, 1, 3] <-- [0, 0, 0, 0, 1, 3]
      cprime%c3(2,10) = cprime%c3(2,10) + 2.0_ki/3.0_ki * C(0) * coeffs%c3(2, 20)
      ! [0, 0, 0, 0, 3] <-- [0, 0, 0, 0, 1, 3]
      cprime%c2(3,10) = cprime%c2(3,10) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(2, 20)
      ! [0, 0, 0, 0, 1] <-- [0, 0, 0, 0, 1, 3]
      cprime%c2(1,10) = cprime%c2(1,10) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(2, 20)
      ! [0, 0, 0, 2, 3] <-- [0, 0, 0, 0, 2, 3]
      cprime%c3(3,10) = cprime%c3(3,10) + 2.0_ki/3.0_ki * C(0) * coeffs%c3(3, 20)
      ! [0, 0, 0, 0, 3] <-- [0, 0, 0, 0, 2, 3]
      cprime%c2(3,10) = cprime%c2(3,10) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(3, 20)
      ! [0, 0, 0, 0, 2] <-- [0, 0, 0, 0, 2, 3]
      cprime%c2(2,10) = cprime%c2(2,10) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(3, 20)
      ! [1, 1, 1, 2, 3] <-- [1, 1, 1, 1, 2, 3]
      cprime%c3(4,10) = cprime%c3(4,10) + 2.0_ki/3.0_ki * C(1) * coeffs%c3(4, 20)
      ! [1, 1, 1, 1, 3] <-- [1, 1, 1, 1, 2, 3]
      cprime%c2(5,10) = cprime%c2(5,10) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(4, 20)
      ! [1, 1, 1, 1, 2] <-- [1, 1, 1, 1, 2, 3]
      cprime%c2(4,10) = cprime%c2(4,10) + 1.0_ki/6.0_ki * C(3) * coeffs%c3(4, 20)
      ! [1, 1, 1, 2, 3] <-- [0, 1, 1, 1, 2, 3]
      cprime%c3(4,10) = cprime%c3(4,10) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 10)
      ! [0, 1, 1, 2, 3] <-- [0, 1, 1, 1, 2, 3]
      cprime%c4(1,4) = cprime%c4(1,4) + 0.5_ki * C(1) * coeffs%c4(1, 10)
      ! [0, 1, 1, 1, 3] <-- [0, 1, 1, 1, 2, 3]
      cprime%c3(2,6) = cprime%c3(2,6) + 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 10)
      ! [0, 1, 1, 1, 2] <-- [0, 1, 1, 1, 2, 3]
      cprime%c3(1,6) = cprime%c3(1,6) + 1.0_ki/6.0_ki * C(3) * coeffs%c4(1, 10)
      ! [0, 0, 0, 1, 1] <-- [0, 0, 0, 0, 1, 1]
      cprime%c2(1,9) = cprime%c2(1,9) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(1, 14)
      ! [0, 0, 0, 0, 1] <-- [0, 0, 0, 0, 1, 1]
      cprime%c2(1,10) = cprime%c2(1,10) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(1, 14)
      ! [0, 0, 0, 2, 2] <-- [0, 0, 0, 0, 2, 2]
      cprime%c2(2,9) = cprime%c2(2,9) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(2, 14)
      ! [0, 0, 0, 0, 2] <-- [0, 0, 0, 0, 2, 2]
      cprime%c2(2,10) = cprime%c2(2,10) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(2, 14)
      ! [0, 0, 0, 3, 3] <-- [0, 0, 0, 0, 3, 3]
      cprime%c2(3,9) = cprime%c2(3,9) + 2.0_ki/3.0_ki * C(0) * coeffs%c2(3, 14)
      ! [0, 0, 0, 0, 3] <-- [0, 0, 0, 0, 3, 3]
      cprime%c2(3,10) = cprime%c2(3,10) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(3, 14)
      ! [1, 1, 1, 2, 2] <-- [1, 1, 1, 1, 2, 2]
      cprime%c2(4,9) = cprime%c2(4,9) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(4, 14)
      ! [1, 1, 1, 1, 2] <-- [1, 1, 1, 1, 2, 2]
      cprime%c2(4,10) = cprime%c2(4,10) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(4, 14)
      ! [1, 1, 1, 3, 3] <-- [1, 1, 1, 1, 3, 3]
      cprime%c2(5,9) = cprime%c2(5,9) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(5, 14)
      ! [1, 1, 1, 1, 3] <-- [1, 1, 1, 1, 3, 3]
      cprime%c2(5,10) = cprime%c2(5,10) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(5, 14)
      ! [2, 2, 2, 3, 3] <-- [2, 2, 2, 2, 3, 3]
      cprime%c2(6,9) = cprime%c2(6,9) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(6, 14)
      ! [2, 2, 2, 2, 3] <-- [2, 2, 2, 2, 3, 3]
      cprime%c2(6,10) = cprime%c2(6,10) + 1.0_ki/3.0_ki * C(3) * coeffs%c2(6, 14)
      ! [0, 1, 2, 2, 2] <-- [0, 0, 1, 2, 2, 2]
      cprime%c3(1,3) = cprime%c3(1,3) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 13)
      ! [0, 0, 2, 2, 2] <-- [0, 0, 1, 2, 2, 2]
      cprime%c2(2,7) = cprime%c2(2,7) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(1, 13)
      ! [0, 0, 1, 2, 2] <-- [0, 0, 1, 2, 2, 2]
      cprime%c3(1,8) = cprime%c3(1,8) + 0.5_ki * C(2) * coeffs%c3(1, 13)
      ! [0, 1, 3, 3, 3] <-- [0, 0, 1, 3, 3, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 13)
      ! [0, 0, 3, 3, 3] <-- [0, 0, 1, 3, 3, 3]
      cprime%c2(3,7) = cprime%c2(3,7) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(2, 13)
      ! [0, 0, 1, 3, 3] <-- [0, 0, 1, 3, 3, 3]
      cprime%c3(2,8) = cprime%c3(2,8) + 0.5_ki * C(3) * coeffs%c3(2, 13)
      ! [0, 2, 3, 3, 3] <-- [0, 0, 2, 3, 3, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 13)
      ! [0, 0, 3, 3, 3] <-- [0, 0, 2, 3, 3, 3]
      cprime%c2(3,7) = cprime%c2(3,7) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(3, 13)
      ! [0, 0, 2, 3, 3] <-- [0, 0, 2, 3, 3, 3]
      cprime%c3(3,8) = cprime%c3(3,8) + 0.5_ki * C(3) * coeffs%c3(3, 13)
      ! [1, 2, 3, 3, 3] <-- [1, 1, 2, 3, 3, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 13)
      ! [1, 1, 3, 3, 3] <-- [1, 1, 2, 3, 3, 3]
      cprime%c2(5,7) = cprime%c2(5,7) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(4, 13)
      ! [1, 1, 2, 3, 3] <-- [1, 1, 2, 3, 3, 3]
      cprime%c3(4,8) = cprime%c3(4,8) + 0.5_ki * C(3) * coeffs%c3(4, 13)
      ! [1, 2, 3, 3, 3] <-- [0, 1, 2, 3, 3, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 1.0_ki/6.0_ki * C(0) * coeffs%c4(1, 3)
      ! [0, 2, 3, 3, 3] <-- [0, 1, 2, 3, 3, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 1.0_ki/6.0_ki * C(1) * coeffs%c4(1, 3)
      ! [0, 1, 3, 3, 3] <-- [0, 1, 2, 3, 3, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 1.0_ki/6.0_ki * C(2) * coeffs%c4(1, 3)
      ! [0, 1, 2, 3, 3] <-- [0, 1, 2, 3, 3, 3]
      cprime%c4(1,2) = cprime%c4(1,2) + 0.5_ki * C(3) * coeffs%c4(1, 3)
      ! [0, 1, 1, 2, 2] <-- [0, 0, 1, 1, 2, 2]
      cprime%c3(1,5) = cprime%c3(1,5) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(1, 15)
      ! [0, 0, 1, 2, 2] <-- [0, 0, 1, 1, 2, 2]
      cprime%c3(1,8) = cprime%c3(1,8) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(1, 15)
      ! [0, 0, 1, 1, 2] <-- [0, 0, 1, 1, 2, 2]
      cprime%c3(1,9) = cprime%c3(1,9) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(1, 15)
      ! [0, 1, 1, 3, 3] <-- [0, 0, 1, 1, 3, 3]
      cprime%c3(2,5) = cprime%c3(2,5) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(2, 15)
      ! [0, 0, 1, 3, 3] <-- [0, 0, 1, 1, 3, 3]
      cprime%c3(2,8) = cprime%c3(2,8) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(2, 15)
      ! [0, 0, 1, 1, 3] <-- [0, 0, 1, 1, 3, 3]
      cprime%c3(2,9) = cprime%c3(2,9) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(2, 15)
      ! [0, 2, 2, 3, 3] <-- [0, 0, 2, 2, 3, 3]
      cprime%c3(3,5) = cprime%c3(3,5) + 1.0_ki/3.0_ki * C(0) * coeffs%c3(3, 15)
      ! [0, 0, 2, 3, 3] <-- [0, 0, 2, 2, 3, 3]
      cprime%c3(3,8) = cprime%c3(3,8) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(3, 15)
      ! [0, 0, 2, 2, 3] <-- [0, 0, 2, 2, 3, 3]
      cprime%c3(3,9) = cprime%c3(3,9) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(3, 15)
      ! [1, 2, 2, 3, 3] <-- [1, 1, 2, 2, 3, 3]
      cprime%c3(4,5) = cprime%c3(4,5) + 1.0_ki/3.0_ki * C(1) * coeffs%c3(4, 15)
      ! [1, 1, 2, 3, 3] <-- [1, 1, 2, 2, 3, 3]
      cprime%c3(4,8) = cprime%c3(4,8) + 1.0_ki/3.0_ki * C(2) * coeffs%c3(4, 15)
      ! [1, 1, 2, 2, 3] <-- [1, 1, 2, 2, 3, 3]
      cprime%c3(4,9) = cprime%c3(4,9) + 1.0_ki/3.0_ki * C(3) * coeffs%c3(4, 15)
      ! [0, 1, 1, 1, 1] <-- [0, 0, 1, 1, 1, 1]
      cprime%c2(1,4) = cprime%c2(1,4) + 1.0_ki/3.0_ki * C(0) * coeffs%c2(1, 9)
      ! [0, 0, 1, 1, 1] <-- [0, 0, 1, 1, 1, 1]
      cprime%c2(1,7) = cprime%c2(1,7) + 2.0_ki/3.0_ki * C(1) * coeffs%c2(1, 9)
      ! [0, 2, 2, 2, 2] <-- [0, 0, 2, 2, 2, 2]
      cprime%c2(2,4) = cprime%c2(2,4) + 1.0_ki/3.0_ki * C(0) * coeffs%c2(2, 9)
      ! [0, 0, 2, 2, 2] <-- [0, 0, 2, 2, 2, 2]
      cprime%c2(2,7) = cprime%c2(2,7) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(2, 9)
      ! [0, 3, 3, 3, 3] <-- [0, 0, 3, 3, 3, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 1.0_ki/3.0_ki * C(0) * coeffs%c2(3, 9)
      ! [0, 0, 3, 3, 3] <-- [0, 0, 3, 3, 3, 3]
      cprime%c2(3,7) = cprime%c2(3,7) + 2.0_ki/3.0_ki * C(3) * coeffs%c2(3, 9)
      ! [1, 2, 2, 2, 2] <-- [1, 1, 2, 2, 2, 2]
      cprime%c2(4,4) = cprime%c2(4,4) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(4, 9)
      ! [1, 1, 2, 2, 2] <-- [1, 1, 2, 2, 2, 2]
      cprime%c2(4,7) = cprime%c2(4,7) + 2.0_ki/3.0_ki * C(2) * coeffs%c2(4, 9)
      ! [1, 3, 3, 3, 3] <-- [1, 1, 3, 3, 3, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 1.0_ki/3.0_ki * C(1) * coeffs%c2(5, 9)
      ! [1, 1, 3, 3, 3] <-- [1, 1, 3, 3, 3, 3]
      cprime%c2(5,7) = cprime%c2(5,7) + 2.0_ki/3.0_ki * C(3) * coeffs%c2(5, 9)
      ! [2, 3, 3, 3, 3] <-- [2, 2, 3, 3, 3, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 1.0_ki/3.0_ki * C(2) * coeffs%c2(6, 9)
      ! [2, 2, 3, 3, 3] <-- [2, 2, 3, 3, 3, 3]
      cprime%c2(6,7) = cprime%c2(6,7) + 2.0_ki/3.0_ki * C(3) * coeffs%c2(6, 9)
      ! [0, 0, 0, 0, 0] <-- [0, 0, 0, 0, 0, 0]
      cprime%c1(1,5) = cprime%c1(1,5) + C(0) * coeffs%c1(1, 6)
      ! [1, 1, 1, 1, 1] <-- [1, 1, 1, 1, 1, 1]
      cprime%c1(2,5) = cprime%c1(2,5) + C(1) * coeffs%c1(2, 6)
      ! [2, 2, 2, 2, 2] <-- [2, 2, 2, 2, 2, 2]
      cprime%c1(3,5) = cprime%c1(3,5) + C(2) * coeffs%c1(3, 6)
      ! [3, 3, 3, 3, 3] <-- [3, 3, 3, 3, 3, 3]
      cprime%c1(4,5) = cprime%c1(4,5) + C(3) * coeffs%c1(4, 6)
      ! [1, 2, 2, 2, 2] <-- [0, 1, 2, 2, 2, 2]
      cprime%c2(4,4) = cprime%c2(4,4) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(1, 4)
      ! [0, 2, 2, 2, 2] <-- [0, 1, 2, 2, 2, 2]
      cprime%c2(2,4) = cprime%c2(2,4) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(1, 4)
      ! [0, 1, 2, 2, 2] <-- [0, 1, 2, 2, 2, 2]
      cprime%c3(1,3) = cprime%c3(1,3) + 2.0_ki/3.0_ki * C(2) * coeffs%c3(1, 4)
      ! [1, 3, 3, 3, 3] <-- [0, 1, 3, 3, 3, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(2, 4)
      ! [0, 3, 3, 3, 3] <-- [0, 1, 3, 3, 3, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(2, 4)
      ! [0, 1, 3, 3, 3] <-- [0, 1, 3, 3, 3, 3]
      cprime%c3(2,3) = cprime%c3(2,3) + 2.0_ki/3.0_ki * C(3) * coeffs%c3(2, 4)
      ! [2, 3, 3, 3, 3] <-- [0, 2, 3, 3, 3, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 1.0_ki/6.0_ki * C(0) * coeffs%c3(3, 4)
      ! [0, 3, 3, 3, 3] <-- [0, 2, 3, 3, 3, 3]
      cprime%c2(3,4) = cprime%c2(3,4) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(3, 4)
      ! [0, 2, 3, 3, 3] <-- [0, 2, 3, 3, 3, 3]
      cprime%c3(3,3) = cprime%c3(3,3) + 2.0_ki/3.0_ki * C(3) * coeffs%c3(3, 4)
      ! [2, 3, 3, 3, 3] <-- [1, 2, 3, 3, 3, 3]
      cprime%c2(6,4) = cprime%c2(6,4) + 1.0_ki/6.0_ki * C(1) * coeffs%c3(4, 4)
      ! [1, 3, 3, 3, 3] <-- [1, 2, 3, 3, 3, 3]
      cprime%c2(5,4) = cprime%c2(5,4) + 1.0_ki/6.0_ki * C(2) * coeffs%c3(4, 4)
      ! [1, 2, 3, 3, 3] <-- [1, 2, 3, 3, 3, 3]
      cprime%c3(4,3) = cprime%c3(4,3) + 2.0_ki/3.0_ki * C(3) * coeffs%c3(4, 4)
      amp = amp - contract5_5(cprime, momenta, new_set)
   end do
end function contract6_6
!****f* src/interface/tens_comb/contract_a_tensor_1
! NAME
!
! Function contract_a_tensor_1
!
! USAGE
!
!  ans = contract_a_tensor_1(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 1 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_1)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_1(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_1), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_1
!****f* src/interface/tens_comb/contract_a_tensor_2
! NAME
!
! Function contract_a_tensor_2
!
! USAGE
!
!  ans = contract_a_tensor_2(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 2 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_2(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case(2)
      acc = momenta(1,0)*momenta(2,0)
      res = acc * coeffs%c1(1,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,2)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,2)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,1)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,1)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,1)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_2
!****f* src/interface/tens_comb/contract_b_tensor_2
! NAME
!
! Function contract_b_tensor_2
!
! USAGE
!
!  ans = contract_b_tensor_2(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an B-type tensor
!  of rank 2 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_2)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_b_tensor_2(coeffs, momenta) result(res)
   ! generated by: write_function_b_tensor_contract
   implicit none
   type(coeff_type_2), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = coeffs%c1(1,2)
      res = res - coeffs%c1(2,2)
      res = res - coeffs%c1(3,2)
      res = res - coeffs%c1(4,2)
   case default
      res = 0.0_ki
   end select
end  function contract_b_tensor_2
!****f* src/interface/tens_comb/contract_a_tensor_3
! NAME
!
! Function contract_a_tensor_3
!
! USAGE
!
!  ans = contract_a_tensor_3(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 3 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_3(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case(2)
      acc = momenta(1,0)*momenta(2,0)
      res = acc * coeffs%c1(1,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,2)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,2)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,1)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,1)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,1)
   case(3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,2)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,1)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,3)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,3)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,3)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,3)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_3
!****f* src/interface/tens_comb/contract_b_tensor_3
! NAME
!
! Function contract_b_tensor_3
!
! USAGE
!
!  ans = contract_b_tensor_3(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an B-type tensor
!  of rank 3 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_3)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_b_tensor_3(coeffs, momenta) result(res)
   ! generated by: write_function_b_tensor_contract
   implicit none
   type(coeff_type_3), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = coeffs%c1(1,2)
      res = res - coeffs%c1(2,2)
      res = res - coeffs%c1(3,2)
      res = res - coeffs%c1(4,2)
   case(1)
      acc = -momenta(1,0)
      res = acc * coeffs%c2(1,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(2,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(3,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(4,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(5,2)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(6,2)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c1(1,3)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c1(2,3)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c1(3,3)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,1)
      res = res + acc * coeffs%c2(1,3)
      acc = momenta(1,2)
      res = res + acc * coeffs%c2(2,3)
      acc = momenta(1,3)
      res = res + acc * coeffs%c2(3,3)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(4,3)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(5,3)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(6,3)
   case default
      res = 0.0_ki
   end select
end  function contract_b_tensor_3
!****f* src/interface/tens_comb/contract_a_tensor_4
! NAME
!
! Function contract_a_tensor_4
!
! USAGE
!
!  ans = contract_a_tensor_4(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 4 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_4(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg4
   real(ki) :: reg5
   real(ki) :: reg2
   real(ki) :: reg3
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case(2)
      acc = momenta(1,0)*momenta(2,0)
      res = acc * coeffs%c1(1,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,2)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,2)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,1)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,1)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,1)
   case(3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,2)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,1)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,3)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,3)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,3)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,4)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,4)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,4)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,4)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,4)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,4)
   case(4)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = acc * coeffs%c3(1,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = momenta(2,2)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,4)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,4)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,6)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,6)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,6)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,6)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,6)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,6)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,5)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,5)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,5)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,5)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,5)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,5)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c1(1,4)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c1(2,4)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c1(3,4)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c1(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_4
!****f* src/interface/tens_comb/contract_b_tensor_4
! NAME
!
! Function contract_b_tensor_4
!
! USAGE
!
!  ans = contract_b_tensor_4(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an B-type tensor
!  of rank 4 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_b_tensor_4(coeffs, momenta) result(res)
   ! generated by: write_function_b_tensor_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = coeffs%c1(1,2)
      res = res - coeffs%c1(2,2)
      res = res - coeffs%c1(3,2)
      res = res - coeffs%c1(4,2)
   case(1)
      acc = -momenta(1,0)
      res = acc * coeffs%c2(1,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(2,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(3,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(4,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(5,2)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(6,2)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c1(1,3)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c1(2,3)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c1(3,3)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,1)
      res = res + acc * coeffs%c2(1,4)
      acc = momenta(1,2)
      res = res + acc * coeffs%c2(2,4)
      acc = momenta(1,3)
      res = res + acc * coeffs%c2(3,4)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(4,4)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(5,4)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(6,4)
   case(2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = acc * coeffs%c3(1,3)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,4)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,4)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,6)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,6)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,6)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,6)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,6)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,6)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(1,5)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(2,5)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(3,5)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(4,5)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(5,5)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(6,5)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(1,5)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(2,5)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(3,5)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(4,5)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(5,5)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(6,5)
      acc = 6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c1(1,4)
      acc = -6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,4)
      acc = -6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,4)
      acc = -6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_b_tensor_4
!****f* src/interface/tens_comb/contract_c_tensor_4
! NAME
!
! Function contract_c_tensor_4
!
! USAGE
!
!  ans = contract_c_tensor_4(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an C-type tensor
!  of rank 4 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_4)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_c_tensor_4(coeffs, momenta) result(res)
   ! generated by: write_function_c_tensor_contract
   implicit none
   type(coeff_type_4), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = -coeffs%c2(1,5)
      res = res - coeffs%c2(2,5)
      res = res - coeffs%c2(3,5)
      res = res + coeffs%c2(4,5)
      res = res + coeffs%c2(5,5)
      res = res + coeffs%c2(6,5)
      acc = 3
      res = res + acc * coeffs%c1(1,4)
      acc = 3
      res = res + acc * coeffs%c1(2,4)
      acc = 3
      res = res + acc * coeffs%c1(3,4)
      acc = 3
      res = res + acc * coeffs%c1(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_c_tensor_4
!****f* src/interface/tens_comb/contract_a_tensor_5
! NAME
!
! Function contract_a_tensor_5
!
! USAGE
!
!  ans = contract_a_tensor_5(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 5 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_5(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg6
   real(ki) :: reg4
   real(ki) :: reg5
   real(ki) :: reg2
   real(ki) :: reg3
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case(2)
      acc = momenta(1,0)*momenta(2,0)
      res = acc * coeffs%c1(1,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,2)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,2)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,1)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,1)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,1)
   case(3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,2)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,1)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,3)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,3)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,3)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,5)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,5)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,5)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,5)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,5)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,5)
   case(4)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = acc * coeffs%c3(1,4)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,4)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = momenta(2,2)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,8)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,8)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,8)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,8)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,8)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,8)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,6)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,6)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,6)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,6)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,6)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,6)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c1(1,4)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c1(2,4)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c1(3,4)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c1(4,4)
   case(5)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = acc * coeffs%c2(1,9)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(2,9)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg3
      acc = acc * momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(3,9)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(4,9)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg3
      acc = acc * momenta(1,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(5,9)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg3
      acc = acc * momenta(1,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(6,9)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,10)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,10)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,10)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,10)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,0)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      reg5 = acc * momenta(4,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,6)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,6)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,6)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,6)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,0)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(1,4)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(2,4)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(3,4)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(4,4)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(5,4)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(6,4)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,5)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      acc = acc * momenta(5,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,5)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,5)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg2 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,5)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)
      res = res + acc * coeffs%c1(1,5)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,1)
      res = res + acc * coeffs%c1(2,5)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)*momenta(5,2)
      res = res + acc * coeffs%c1(3,5)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      res = res + acc * coeffs%c1(4,5)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(1,7)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(2,7)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(3,7)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(4,7)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(5,7)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(6,7)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,8)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(5,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,8)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,8)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,8)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,1)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(1,10)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(2,10)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(3,10)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(4,10)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(5,10)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(6,10)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,4)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_5
!****f* src/interface/tens_comb/contract_b_tensor_5
! NAME
!
! Function contract_b_tensor_5
!
! USAGE
!
!  ans = contract_b_tensor_5(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an B-type tensor
!  of rank 5 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_b_tensor_5(coeffs, momenta) result(res)
   ! generated by: write_function_b_tensor_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = coeffs%c1(1,2)
      res = res - coeffs%c1(2,2)
      res = res - coeffs%c1(3,2)
      res = res - coeffs%c1(4,2)
   case(1)
      acc = -momenta(1,0)
      res = acc * coeffs%c2(1,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(2,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(3,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(4,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(5,2)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(6,2)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c1(1,3)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c1(2,3)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c1(3,3)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,1)
      res = res + acc * coeffs%c2(1,5)
      acc = momenta(1,2)
      res = res + acc * coeffs%c2(2,5)
      acc = momenta(1,3)
      res = res + acc * coeffs%c2(3,5)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(4,5)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(5,5)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(6,5)
   case(2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = acc * coeffs%c3(1,4)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,4)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,8)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,8)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,8)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,8)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,8)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,8)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(1,6)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(2,6)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(3,6)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(4,6)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(5,6)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(6,6)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(1,6)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(2,6)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(3,6)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(4,6)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(5,6)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(6,6)
      acc = 6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c1(1,4)
      acc = -6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,4)
      acc = -6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,4)
      acc = -6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,4)
   case(3)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = 3*momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,9)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = 3*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,9)
      acc = 3*momenta(2,0)*momenta(3,3)
      acc = acc + 3*momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,9)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,9)
      acc = -3*momenta(2,1)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,9)
      acc = -3*momenta(2,2)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,9)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(1,9)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(2,9)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(3,9)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(4,9)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(5,9)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(6,9)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,10)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,10)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,10)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,10)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,3)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,6)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,6)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,6)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,6)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,4)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,4)
      acc = -6*momenta(2,0)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,4)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,4)
      acc = -6*momenta(2,1)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,4)
      acc = -6*momenta(2,2)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,4)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,5)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,5)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,5)
      acc = -momenta(2,1)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,5)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,5)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,5)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,5)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,5)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,5)
      acc = -10*momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,5)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,5)
      acc = -10*momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,5)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(1,7)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(2,7)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(3,7)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(4,7)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(5,7)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(6,7)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,7)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,7)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,7)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,7)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,7)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,7)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,8)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,8)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,8)
      acc = -momenta(2,2)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,8)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,8)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,8)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,8)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,8)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,10)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,10)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,10)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,10)
      acc = -6*momenta(1,1)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,10)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,10)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,4)
   case default
      res = 0.0_ki
   end select
end  function contract_b_tensor_5
!****f* src/interface/tens_comb/contract_c_tensor_5
! NAME
!
! Function contract_c_tensor_5
!
! USAGE
!
!  ans = contract_c_tensor_5(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an C-type tensor
!  of rank 5 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_5)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_c_tensor_5(coeffs, momenta) result(res)
   ! generated by: write_function_c_tensor_contract
   implicit none
   type(coeff_type_5), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = -coeffs%c2(1,6)
      res = res - coeffs%c2(2,6)
      res = res - coeffs%c2(3,6)
      res = res + coeffs%c2(4,6)
      res = res + coeffs%c2(5,6)
      res = res + coeffs%c2(6,6)
      acc = 3
      res = res + acc * coeffs%c1(1,4)
      acc = 3
      res = res + acc * coeffs%c1(2,4)
      acc = 3
      res = res + acc * coeffs%c1(3,4)
      acc = 3
      res = res + acc * coeffs%c1(4,4)
   case(1)
      acc = -3*momenta(1,0)
      res = acc * coeffs%c2(1,9)
      acc = -3*momenta(1,0)
      res = res + acc * coeffs%c2(2,9)
      acc = -3*momenta(1,0)
      res = res + acc * coeffs%c2(3,9)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(4,9)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(5,9)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(6,9)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(1,4)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(2,4)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(3,4)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(4,4)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(5,4)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(6,4)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(1,5)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(2,5)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(3,5)
      acc = momenta(1,1)
      res = res + acc * coeffs%c3(4,5)
      acc = 15*momenta(1,0)
      res = res + acc * coeffs%c1(1,5)
      acc = 15*momenta(1,1)
      res = res + acc * coeffs%c1(2,5)
      acc = 15*momenta(1,2)
      res = res + acc * coeffs%c1(3,5)
      acc = 15*momenta(1,3)
      res = res + acc * coeffs%c1(4,5)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c2(1,7)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c2(2,7)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c2(3,7)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(4,7)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(5,7)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(6,7)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c3(1,8)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c3(2,8)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c3(3,8)
      acc = momenta(1,2)
      res = res + acc * coeffs%c3(4,8)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(1,10)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(2,10)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(3,10)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(4,10)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(5,10)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(6,10)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c3(1,9)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c3(2,9)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c3(3,9)
      acc = momenta(1,3)
      res = res + acc * coeffs%c3(4,9)
   case default
      res = 0.0_ki
   end select
end  function contract_c_tensor_5
!****f* src/interface/tens_comb/contract_a_tensor_6
! NAME
!
! Function contract_a_tensor_6
!
! USAGE
!
!  ans = contract_a_tensor_6(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an A-type tensor
!  of rank 6 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_a_tensor_6(coeffs, momenta) result(res)
   ! generated by: write_function_a_tensor_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg8
   real(ki) :: reg9
   real(ki) :: reg6
   real(ki) :: reg7
   real(ki) :: reg4
   real(ki) :: reg5
   real(ki) :: reg2
   real(ki) :: reg3
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(1)
      acc = momenta(1,0)
      res = acc * coeffs%c1(1,1)
      acc = momenta(1,1)
      res = res + acc * coeffs%c1(2,1)
      acc = momenta(1,2)
      res = res + acc * coeffs%c1(3,1)
      acc = momenta(1,3)
      res = res + acc * coeffs%c1(4,1)
   case(2)
      acc = momenta(1,0)*momenta(2,0)
      res = acc * coeffs%c1(1,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,2)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,2)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,1)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,1)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,1)
   case(3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,2)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,1)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,3)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,3)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,3)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,6)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,6)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,6)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,6)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,6)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,6)
   case(4)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = acc * coeffs%c3(1,5)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,5)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,5)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,5)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = momenta(2,2)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,11)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,11)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,11)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,11)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,10)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,10)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,10)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,10)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,10)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,10)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,7)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,7)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,7)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,7)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,7)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,7)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c1(1,4)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c1(2,4)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c1(3,4)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c1(4,4)
   case(5)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = acc * coeffs%c2(1,11)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(2,11)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg3
      acc = acc * momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(3,11)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(4,11)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg3
      acc = acc * momenta(1,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(5,11)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg3
      acc = acc * momenta(1,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(6,11)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,17)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,17)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,17)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,17)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,0)
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      reg5 = acc * momenta(4,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,11)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,1)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,4)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(1,8)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(2,8)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(3,8)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c3(4,8)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,0)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(1,4)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(2,4)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(3,4)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(4,4)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(5,4)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(6,4)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,6)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      acc = acc * momenta(5,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,6)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg2 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,6)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg2 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,6)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)
      res = res + acc * coeffs%c1(1,5)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,1)
      res = res + acc * coeffs%c1(2,5)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)*momenta(5,2)
      res = res + acc * coeffs%c1(3,5)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)
      res = res + acc * coeffs%c1(4,5)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(5,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(1,8)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(2,8)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,0)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(3,8)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(4,8)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(5,8)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(1,2)
      acc = acc + reg0
      acc = acc / 10.0_ki
      res = res + acc * coeffs%c2(6,8)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,12)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(5,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,12)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,12)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,12)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,1)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(1,13)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(2,13)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(3,13)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(4,13)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,3)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(5,13)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)*momenta(5,2)
      acc = acc + reg0
      acc = acc / 5.0_ki
      res = res + acc * coeffs%c2(6,13)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,14)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,14)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,14)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,14)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      reg2 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c4(1,7)
   case(6)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(5,0)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(5,0)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg8 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg8 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      reg7 = acc * momenta(4,2)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      reg8 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      reg5 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      reg6 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg7
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg8 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg8
      acc = acc * momenta(5,1)
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      reg6 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg8 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      reg7 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(6,2)
      acc = acc + momenta(4,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg8 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg8
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg9 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg9
      acc = acc * momenta(6,1)
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = acc * coeffs%c4(1,14)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(5,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(1,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(5,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(1,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(1,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg6 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,2)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(6,2)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,1)
      reg7 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc * momenta(4,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 120.0_ki
      res = res + acc * coeffs%c4(1,6)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,1)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)*momenta(6,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,15)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)*momenta(6,2)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,15)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)*momenta(6,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,15)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,2)*momenta(6,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,15)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,3)*momenta(6,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,15)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg3 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)*momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)*momenta(5,2)*momenta(6,2)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,15)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,0)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,16)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(6,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,0)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,16)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc * momenta(6,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,16)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,16)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(6,1)
      acc = acc + momenta(1,1)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg6 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg8 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(6,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg7 = acc * momenta(6,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg8 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg8 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      reg6 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg7
      reg6 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg8 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg8
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg7 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg7
      reg6 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg8 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,3)
      acc = acc + reg8
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg8 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg8
      reg7 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg9 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg9
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg6 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg8 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg8 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      reg7 = acc * momenta(4,2)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      reg8 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,1)
      acc = acc + reg7
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,1)
      acc = acc + momenta(2,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg7
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg8
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg8 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,3)
      acc = acc + reg8
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg8
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg9 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg9
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc * momenta(4,3)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = res + acc * coeffs%c4(1,13)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(5,0)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg4
      acc = acc * momenta(5,1)
      acc = acc + reg2
      acc = acc * momenta(6,1)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(1,12)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg4
      acc = acc * momenta(5,2)
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(5,2)
      acc = acc + reg3
      acc = acc * momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg5
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(2,12)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg4
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)
      acc = acc + reg3
      acc = acc * momenta(1,0)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(3,3)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg4
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(6,3)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(3,12)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(5,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(4,12)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(1,1)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg5
      acc = acc * momenta(1,1)
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg4
      acc = acc * momenta(6,3)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(5,12)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(1,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(1,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 20.0_ki
      res = res + acc * coeffs%c2(6,12)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(5,0)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,10)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(5,0)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,10)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      acc = acc * momenta(6,2)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,10)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,10)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(5,0)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(5,0)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg8 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(6,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg7 = acc * momenta(6,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg8 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,2)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg8 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      reg6 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg7
      reg6 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg8 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg8
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc * momenta(4,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      reg6 = acc * momenta(6,1)
      acc = momenta(5,0)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg8 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,1)
      acc = acc + reg8
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,1)
      acc = acc + momenta(2,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg8 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg8
      reg7 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg9 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg9
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = res + acc * coeffs%c4(1,9)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg6 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      reg6 = acc * momenta(4,2)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      reg7 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg7 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg7 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg5 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(3,1)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,1)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,0)
      acc = acc * momenta(6,0)
      acc = acc + reg3
      reg2 = acc * momenta(1,0)
      acc = momenta(1,1)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,1)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,0)
      acc = acc * momenta(5,0)
      acc = acc * momenta(6,0)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 120.0_ki
      res = res + acc * coeffs%c4(1,15)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,0)*momenta(6,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,5)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg3 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)*momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)*momenta(5,2)*momenta(6,2)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,5)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,0)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,5)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg3 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)*momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)*momenta(5,2)*momenta(6,2)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,5)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,1)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,5)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,5)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(6,1)
      acc = acc + momenta(1,1)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg6 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg8 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(2,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg8 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      reg7 = acc * momenta(4,2)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      reg8 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      reg6 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg7 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg7
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg8 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg8
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc * momenta(6,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg7 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg7
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg7 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg8
      reg7 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg8 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg8
      reg7 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg9 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg9
      acc = acc * momenta(6,3)
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = res + acc * coeffs%c4(1,12)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(6,2)
      acc = acc + momenta(4,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc * momenta(1,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg6
      acc = acc * momenta(4,0)
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(5,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,18)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(5,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc * momenta(6,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,18)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,18)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(4,1)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,18)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,2)*momenta(3,3)
      acc = acc + momenta(1,3)*momenta(3,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg6 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg8 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg7 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg8 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      reg8 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(5,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(2,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      reg7 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg8 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg8
      acc = acc + reg7
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg7 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg7
      reg6 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg7 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg7
      reg6 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg8 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg8
      acc = acc * momenta(1,1)
      acc = acc + reg6
      acc = acc * momenta(5,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg6 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      reg6 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg7 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg8 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg8
      acc = acc + reg7
      reg6 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg8 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg8
      reg7 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg8 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg8
      reg7 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg9 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg9
      acc = acc * momenta(1,1)
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(6,0)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 180.0_ki
      res = res + acc * coeffs%c4(1,8)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(6,2)
      acc = acc + momenta(4,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc * momenta(1,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(6,1)
      acc = acc + reg3
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,19)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg5 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(5,1)
      acc = acc + reg3
      acc = acc * momenta(6,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,19)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg6
      acc = acc * momenta(4,0)
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg5
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,19)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(4,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(4,1)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg6
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg6 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg5
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,19)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(5,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,1)
      acc = acc + momenta(2,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(5,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg5 = acc * momenta(2,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,1)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,1)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(1,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(1,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(3,3)
      acc = acc + reg4
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg6
      acc = acc * momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,2)
      acc = acc + reg3
      acc = acc * momenta(1,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(6,2)
      acc = acc + momenta(4,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc * momenta(1,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(1,0)
      acc = acc * momenta(3,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,20)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,1)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc * momenta(3,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,20)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(3,2)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,0)
      acc = momenta(4,2)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc * momenta(3,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,20)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(4,1)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,20)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(6,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg7 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(1,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(3,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(4,3)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg7 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(4,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(5,3)
      acc = acc + momenta(1,3)*momenta(5,2)
      acc = acc * momenta(4,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(5,3)
      acc = acc + momenta(4,3)*momenta(5,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(5,0)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(5,0)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 120.0_ki
      res = res + acc * coeffs%c4(1,10)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg3
      acc = acc * momenta(5,1)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,1)
      acc = acc + reg2
      acc = acc * momenta(6,1)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(1,14)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg4
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg3
      acc = acc * momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(6,2)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(2,14)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg4
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg3
      acc = acc * momenta(5,3)
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg2
      acc = acc * momenta(6,3)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(3,14)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(6,2)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,2)
      acc = acc + reg2
      acc = acc * momenta(5,2)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(4,14)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg4
      acc = acc * momenta(1,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(6,3)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg2
      acc = acc * momenta(5,3)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(5,14)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,3)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)*momenta(5,2)*momenta(6,2)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(6,14)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg6
      acc = acc * momenta(5,2)
      acc = acc + reg4
      acc = acc * momenta(6,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,1)
      acc = acc + momenta(2,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg6
      acc = acc * momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg5
      acc = acc * momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(1,13)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg6
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc * momenta(6,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg5 = acc * momenta(2,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,0)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(1,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(2,13)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg3 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,2)
      acc = acc + momenta(2,2)*momenta(4,0)
      reg5 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      acc = acc * momenta(5,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,0)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg5
      reg4 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg6
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,0)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,0)
      acc = acc + reg3
      acc = acc * momenta(1,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(3,13)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg3 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(4,2)*momenta(6,3)
      acc = acc + momenta(4,3)*momenta(6,2)
      reg5 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc * momenta(5,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,1)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(5,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,1)*momenta(6,3)
      acc = acc + reg6
      reg5 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)*momenta(6,3)
      acc = acc + reg5
      acc = acc * momenta(3,1)
      acc = acc + reg3
      acc = acc * momenta(1,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 60.0_ki
      res = res + acc * coeffs%c3(4,13)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg4 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(3,0)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(5,3)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,3)
      acc = momenta(2,1)*momenta(5,2)
      acc = acc + momenta(2,2)*momenta(5,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,0)*momenta(6,1)
      acc = acc + momenta(2,1)*momenta(6,0)
      reg5 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg6 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(5,0)*momenta(6,1)
      acc = acc + momenta(5,1)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg7 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(3,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(6,1)
      acc = acc + momenta(3,1)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,2)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,2)
      acc = acc + momenta(5,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,1)
      acc = acc * momenta(1,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(6,1)
      acc = acc + momenta(1,1)*momenta(6,0)
      reg6 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg7 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg7
      acc = acc + reg6
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 120.0_ki
      res = res + acc * coeffs%c4(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg6
      acc = acc * momenta(5,1)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(3,0)*momenta(6,2)
      acc = acc + momenta(3,2)*momenta(6,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg7
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc * momenta(5,0)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,0)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg7
      reg6 = acc * momenta(3,0)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg7
      reg6 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg8 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg8
      acc = acc * momenta(5,2)
      acc = acc + reg6
      acc = acc * momenta(6,1)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 90.0_ki
      res = res + acc * coeffs%c3(1,15)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg5 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      acc = acc * momenta(5,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg5 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      reg4 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      reg5 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg7 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg7
      acc = acc * momenta(3,3)
      acc = acc + reg5
      acc = acc * momenta(5,1)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg6 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg6
      reg5 = acc * momenta(2,1)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      reg6 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg7 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      reg6 = acc * momenta(4,0)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg7 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg7
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg8 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg8
      acc = acc * momenta(3,1)
      acc = acc + reg6
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc * momenta(6,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 90.0_ki
      res = res + acc * coeffs%c3(2,15)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg3 = acc * momenta(5,0)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(2,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg5 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      reg4 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,3)
      acc = momenta(2,0)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,0)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg6
      acc = acc * momenta(5,3)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg5 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg5
      reg4 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg5 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc * momenta(5,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,0)
      acc = momenta(3,2)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,2)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg6 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg6
      reg5 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg7 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg7
      acc = acc * momenta(5,2)
      acc = acc + reg5
      acc = acc * momenta(6,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg5 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,0)
      acc = acc * momenta(3,3)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,0)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg7
      reg6 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg7
      reg6 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg8 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg8
      acc = acc * momenta(3,3)
      acc = acc + reg6
      acc = acc * momenta(1,0)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc * momenta(4,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 90.0_ki
      res = res + acc * coeffs%c3(3,15)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg3 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg5 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg5
      reg4 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,3)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg5 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(3,1)*momenta(6,3)
      acc = acc + reg5
      reg4 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(6,3)
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      acc = acc * momenta(5,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg4 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg5 = acc * momenta(3,3)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      reg4 = acc * momenta(2,2)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg5 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(6,1)
      acc = momenta(1,2)*momenta(6,3)
      acc = acc + momenta(1,3)*momenta(6,2)
      reg6 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc + reg6
      reg5 = acc * momenta(3,1)
      acc = momenta(3,2)*momenta(6,3)
      acc = acc + momenta(3,3)*momenta(6,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(2,1)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg6 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg6
      reg5 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg7 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg7
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc * momenta(5,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(4,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg5 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg6 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(3,2)
      acc = momenta(2,1)*momenta(6,2)
      acc = acc + momenta(2,2)*momenta(6,1)
      reg6 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg6
      reg5 = acc * momenta(5,2)
      acc = momenta(3,1)*momenta(5,3)
      acc = acc + momenta(3,3)*momenta(5,1)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg5
      acc = acc + reg4
      reg3 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg6 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg7 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg7
      acc = acc + reg6
      reg5 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg7 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg7
      reg6 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg6
      acc = acc + reg5
      reg4 = acc * momenta(1,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg7 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg7
      reg6 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg8 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg8
      acc = acc * momenta(3,3)
      acc = acc + reg6
      acc = acc * momenta(1,1)
      acc = acc + reg4
      acc = acc + reg3
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 90.0_ki
      res = res + acc * coeffs%c3(4,15)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg3 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      reg4 = acc * momenta(4,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg3
      acc = acc * momenta(6,0)
      acc = acc + reg1
      reg0 = acc * momenta(5,1)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg3
      reg2 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(6,0)
      acc = acc + reg2
      acc = acc * momenta(5,0)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(1,9)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(2,0)*momenta(5,2)
      acc = acc + reg4
      acc = acc * momenta(6,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,0)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,0)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,0)*momenta(5,2)*momenta(6,2)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(2,9)
      acc = momenta(2,0)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,0)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)
      acc = acc + reg3
      reg2 = acc * momenta(6,0)
      acc = momenta(2,0)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,0)
      reg4 = acc * momenta(5,0)
      acc = momenta(2,0)*momenta(4,0)*momenta(5,3)
      acc = acc + reg4
      acc = acc * momenta(6,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,0)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,0)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,0)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,0)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      acc = acc * momenta(1,0)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(3,9)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg3 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(5,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,2)
      acc = momenta(1,2)*momenta(2,1)*momenta(6,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(5,1)*momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)*momenta(5,2)*momenta(6,2)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(4,9)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(5,1)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(6,1)
      acc = momenta(2,1)*momenta(4,1)*momenta(6,3)
      acc = acc + reg4
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,1)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,1)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,1)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(5,9)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,2)
      acc = momenta(2,2)*momenta(5,3)*momenta(6,2)
      acc = acc + reg4
      acc = acc * momenta(4,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      acc = acc * momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,3)
      acc = momenta(2,3)*momenta(5,2)*momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(4,3)
      acc = momenta(2,3)*momenta(4,2)*momenta(5,3)*momenta(6,3)
      acc = acc + reg3
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      acc = acc + reg2
      acc = acc * momenta(1,2)
      acc = acc + reg0
      acc = acc / 15.0_ki
      res = res + acc * coeffs%c2(6,9)
      acc = momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)*momenta(5,0)*momenta(6,0)
      res = res + acc * coeffs%c1(1,6)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)*momenta(5,1)*momenta(6,1)
      res = res + acc * coeffs%c1(2,6)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)*momenta(5,2)*momenta(6,2)
      res = res + acc * coeffs%c1(3,6)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)*momenta(5,3)*momenta(6,3)
      res = res + acc * coeffs%c1(4,6)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      reg4 = acc * momenta(6,1)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,2)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(5,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(5,1)
      acc = acc + momenta(2,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(3,1)
      acc = acc + momenta(2,1)*momenta(3,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg4 = acc * momenta(4,1)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      reg3 = acc * momenta(2,2)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc + reg3
      reg2 = acc * momenta(5,2)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(6,2)
      acc = acc * momenta(2,2)
      acc = acc * momenta(5,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(1,4)
      acc = momenta(2,1)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,1)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,1)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,1)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(4,0)*momenta(6,1)
      acc = acc + momenta(4,1)*momenta(6,0)
      reg4 = acc * momenta(2,3)
      acc = momenta(2,0)*momenta(4,1)
      acc = acc + momenta(2,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,1)
      acc = acc + momenta(4,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,1)
      acc = acc + momenta(3,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,1)
      acc = acc + momenta(3,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(6,1)
      acc = acc + momenta(1,1)*momenta(6,0)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,1)
      acc = acc + momenta(1,1)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,1)
      acc = acc + momenta(1,1)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,1)
      acc = acc + momenta(1,1)*momenta(3,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg3 = acc * momenta(6,0)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(5,0)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,0)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      reg4 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,0)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(4,0)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg4 = acc * momenta(3,0)
      acc = momenta(2,0)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,0)
      acc = acc * momenta(3,2)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(3,0)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,0)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,0)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,0)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,0)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,0)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,0)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,0)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      reg3 = acc * momenta(5,1)
      acc = momenta(2,2)*momenta(5,3)
      acc = acc + momenta(2,3)*momenta(5,2)
      reg4 = acc * momenta(6,1)
      acc = momenta(5,2)*momenta(6,3)
      acc = acc + momenta(5,3)*momenta(6,2)
      acc = acc * momenta(2,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(4,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(4,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(4,1)*momenta(5,2)
      acc = acc + momenta(4,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(2,1)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,1)
      reg4 = acc * momenta(3,2)
      acc = momenta(2,2)*momenta(6,3)
      acc = acc + momenta(2,3)*momenta(6,2)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(3,1)*momenta(5,2)
      acc = acc + momenta(3,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(3,1)*momenta(4,2)
      acc = acc + momenta(3,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(6,2)
      acc = acc + momenta(1,2)*momenta(6,1)
      reg4 = acc * momenta(2,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc * momenta(6,3)
      acc = acc + reg4
      reg3 = acc * momenta(5,3)
      acc = momenta(1,1)*momenta(5,2)
      acc = acc + momenta(1,2)*momenta(5,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(6,3)
      acc = acc * momenta(2,3)
      acc = acc * momenta(5,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 30.0_ki
      res = res + acc * coeffs%c3(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_a_tensor_6
!****f* src/interface/tens_comb/contract_b_tensor_6
! NAME
!
! Function contract_b_tensor_6
!
! USAGE
!
!  ans = contract_b_tensor_6(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an B-type tensor
!  of rank 6 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_b_tensor_6(coeffs, momenta) result(res)
   ! generated by: write_function_b_tensor_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   real(ki) :: reg4
   real(ki) :: reg5
   real(ki) :: reg2
   real(ki) :: reg3
   real(ki) :: reg0
   real(ki) :: reg1
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = coeffs%c1(1,2)
      res = res - coeffs%c1(2,2)
      res = res - coeffs%c1(3,2)
      res = res - coeffs%c1(4,2)
   case(1)
      acc = -momenta(1,0)
      res = acc * coeffs%c2(1,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(2,2)
      acc = -momenta(1,0)
      res = res + acc * coeffs%c2(3,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(4,2)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c2(5,2)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(6,2)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c1(1,3)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c1(2,3)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c1(3,3)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c1(4,3)
      acc = momenta(1,1)
      res = res + acc * coeffs%c2(1,6)
      acc = momenta(1,2)
      res = res + acc * coeffs%c2(2,6)
      acc = momenta(1,3)
      res = res + acc * coeffs%c2(3,6)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c2(4,6)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(5,6)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c2(6,6)
   case(2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = acc * coeffs%c3(1,5)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,5)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,5)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,5)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,3)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,3)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,11)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,11)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,11)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,11)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,10)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,10)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,10)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,10)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,10)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,10)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,2)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,2)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,2)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(1,7)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(2,7)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(3,7)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(4,7)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(5,7)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(6,7)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(1,7)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(2,7)
      acc = -momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(3,7)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(4,7)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(5,7)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(6,7)
      acc = 6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c1(1,4)
      acc = -6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,4)
      acc = -6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,4)
      acc = -6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,4)
   case(3)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = 3*momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = acc * coeffs%c2(1,11)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = 3*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,11)
      acc = 3*momenta(2,0)*momenta(3,3)
      acc = acc + 3*momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,11)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,11)
      acc = -3*momenta(2,1)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,11)
      acc = -3*momenta(2,2)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,11)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(1,11)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(2,11)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c2(3,11)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(4,11)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(5,11)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(6,11)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,17)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,17)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,17)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,17)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,3)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,3)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,3)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,11)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,4)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,8)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,8)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,8)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,8)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,4)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,4)
      acc = -6*momenta(2,0)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,4)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,4)
      acc = -6*momenta(2,1)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,4)
      acc = -6*momenta(2,2)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,4)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,6)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,6)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,6)
      acc = -momenta(2,1)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,6)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,6)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,6)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,6)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,6)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(3,0)
      res = res + acc * coeffs%c1(1,5)
      acc = -10*momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c1(2,5)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c1(3,5)
      acc = -10*momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c1(4,5)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)
      res = res + acc * coeffs%c2(1,8)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(2,8)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(3,8)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)
      res = res + acc * coeffs%c2(4,8)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(5,8)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)
      res = res + acc * coeffs%c2(6,8)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,8)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,8)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,8)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,8)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,8)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,8)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,12)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,12)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,12)
      acc = -momenta(2,2)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,2)
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,12)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,12)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,12)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,12)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,12)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,1)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(1,13)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(2,13)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(3,13)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(4,13)
      acc = -6*momenta(1,1)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(5,13)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c2(6,13)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,14)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,14)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,14)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,14)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(1,14)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(2,14)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(3,14)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,3)
      acc = acc + reg0
      acc = acc / 3.0_ki
      res = res + acc * coeffs%c3(4,14)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c4(1,7)
   case(4)
      acc = momenta(1,1)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = acc * coeffs%c4(1,14)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -momenta(3,2)*momenta(4,3)
      acc = acc - momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,14)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(3,0)*momenta(4,1)
      acc = acc - 3*momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(2,0)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(3,1)
      acc = acc - 3*momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = -3*momenta(2,0)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(2,0)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(3,0)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = -3*momenta(2,1)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = -3*momenta(2,0)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,6)
      acc = 10*momenta(1,0)*momenta(2,1)
      acc = acc + 10*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,15)
      acc = 10*momenta(1,0)*momenta(2,2)
      acc = acc + 10*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,15)
      acc = 10*momenta(1,0)*momenta(2,3)
      acc = acc + 10*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 10*momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,15)
      acc = -10*momenta(1,1)*momenta(4,2)
      acc = acc - 10*momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -10*momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -10*momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,15)
      acc = -10*momenta(1,1)*momenta(4,3)
      acc = acc - 10*momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -10*momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -10*momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,15)
      acc = -10*momenta(1,2)*momenta(2,3)
      acc = acc - 10*momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,15)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,16)
      acc = momenta(1,1)*momenta(4,3)
      acc = acc + momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,16)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,16)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,16)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,1)*momenta(4,2)
      acc = acc - 3*momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,16)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,16)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,2)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,16)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(2,2)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,16)
      acc = -momenta(2,1)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -momenta(1,0)*momenta(3,1)
      acc = acc - momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -momenta(3,0)*momenta(4,1)
      acc = acc - momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = momenta(2,1)*momenta(3,2)
      acc = acc + momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,13)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -momenta(3,1)*momenta(4,3)
      acc = acc - momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,13)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = 3*momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = 3*momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,12)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = 3*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,12)
      acc = 3*momenta(2,0)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = 3*momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,12)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,12)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,12)
      acc = -3*momenta(2,2)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,12)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,12)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,12)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,12)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,12)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,12)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,12)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -6*momenta(2,0)*momenta(4,1)
      acc = acc - 6*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(4,2)
      acc = acc - 6*momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -6*momenta(2,0)*momenta(3,2)
      acc = acc - 6*momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,10)
      acc = -6*momenta(2,0)*momenta(4,1)
      acc = acc - 6*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(4,3)
      acc = acc - 6*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -6*momenta(2,0)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,10)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -6*momenta(3,0)*momenta(4,3)
      acc = acc - 6*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,10)
      acc = -6*momenta(2,1)*momenta(3,2)
      acc = acc - 6*momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = -6*momenta(1,1)*momenta(3,2)
      acc = acc - 6*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -6*momenta(3,1)*momenta(4,3)
      acc = acc - 6*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,10)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -momenta(3,0)*momenta(4,3)
      acc = acc - momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,9)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(4,3)
      acc = acc - momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,9)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(1,1)*momenta(4,3)
      acc = acc + 3*momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(4,3)
      acc = acc + 3*momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = 3*momenta(2,1)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = 3*momenta(2,0)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,15)
      acc = -10*momenta(1,0)*momenta(4,1)
      acc = acc - 10*momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -10*momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -10*momenta(1,1)*momenta(2,1)*momenta(3,0)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(1,5)
      acc = -10*momenta(1,0)*momenta(2,2)
      acc = acc - 10*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(2,5)
      acc = -10*momenta(2,0)*momenta(4,3)
      acc = acc - 10*momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = -10*momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -10*momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(3,5)
      acc = -10*momenta(1,1)*momenta(2,2)
      acc = acc - 10*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -10*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(4,5)
      acc = -10*momenta(2,1)*momenta(4,3)
      acc = acc - 10*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = -10*momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -10*momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(5,5)
      acc = -10*momenta(2,2)*momenta(4,3)
      acc = acc - 10*momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = -10*momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -10*momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c2(6,5)
      acc = momenta(2,1)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(3,1)*momenta(4,3)
      acc = acc + momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = momenta(1,1)*momenta(4,2)
      acc = acc + momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = momenta(1,1)*momenta(3,2)
      acc = acc + momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,12)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -momenta(3,1)*momenta(4,2)
      acc = acc - momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,12)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(3,0)*momenta(4,1)
      acc = acc + 3*momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,18)
      acc = 3*momenta(2,1)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = 3*momenta(2,0)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(3,1)
      acc = acc + 3*momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,18)
      acc = 3*momenta(2,0)*momenta(3,3)
      acc = acc + 3*momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = 3*momenta(2,0)*momenta(4,3)
      acc = acc + 3*momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = 3*momenta(3,0)*momenta(4,3)
      acc = acc + 3*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 3*momenta(1,0)*momenta(4,2)
      acc = acc + 3*momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = 3*momenta(1,0)*momenta(3,2)
      acc = acc + 3*momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,18)
      acc = -3*momenta(2,1)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,18)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,18)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,1)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,18)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,18)
      acc = -momenta(1,1)*momenta(4,2)
      acc = acc - momenta(1,2)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,2)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,18)
      acc = -momenta(2,0)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(3,0)*momenta(4,3)
      acc = acc - momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -momenta(1,0)*momenta(4,2)
      acc = acc - momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -momenta(1,0)*momenta(3,2)
      acc = acc - momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,8)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -momenta(2,0)*momenta(4,1)
      acc = acc - momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(4,2)
      acc = acc - momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -momenta(2,0)*momenta(3,2)
      acc = acc - momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c4(1,8)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = 3*momenta(1,1)*momenta(4,2)
      acc = acc + 3*momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = 3*momenta(2,0)*momenta(3,2)
      acc = acc + 3*momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,19)
      acc = 3*momenta(2,0)*momenta(4,1)
      acc = acc + 3*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = 3*momenta(1,1)*momenta(4,3)
      acc = acc + 3*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(4,1)
      acc = acc + 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = 3*momenta(2,0)*momenta(3,3)
      acc = acc + 3*momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,19)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = 3*momenta(3,0)*momenta(4,3)
      acc = acc + 3*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,19)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,19)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,2)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,19)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,19)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,3)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,19)
      acc = -momenta(1,1)*momenta(4,3)
      acc = acc - momenta(1,3)*momenta(4,1)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,19)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,0)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = -momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(3,3)
      acc = -momenta(2,3)*momenta(3,0)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = -momenta(2,1)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = -momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,2)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(2,0)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(2,0)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,0)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(3,0)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = -3*momenta(2,1)*momenta(3,2)
      acc = acc - 3*momenta(2,2)*momenta(3,1)
      reg1 = acc * momenta(1,3)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,1)*momenta(3,2)
      acc = acc - 3*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = 6*momenta(1,0)*momenta(4,1)
      acc = acc + 6*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = 6*momenta(2,0)*momenta(4,1)
      acc = acc + 6*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = 6*momenta(3,1)*momenta(4,2)
      acc = acc + 6*momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,20)
      acc = 6*momenta(2,0)*momenta(4,1)
      acc = acc + 6*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 6*momenta(1,0)*momenta(4,1)
      acc = acc + 6*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = 6*momenta(3,1)*momenta(4,3)
      acc = acc + 6*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,20)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = 6*momenta(1,2)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = 6*momenta(3,2)*momenta(4,3)
      acc = acc + 6*momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,20)
      acc = -6*momenta(1,1)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(4,3)
      acc = acc - 6*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,1)*momenta(4,2)
      acc = acc - 6*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -6*momenta(2,2)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,20)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = -3*momenta(2,0)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,10)
      acc = 6*momenta(1,0)*momenta(4,1)
      acc = acc + 6*momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = 6*momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,1)
      acc = acc + 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,14)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = 6*momenta(1,0)*momenta(2,2)
      acc = acc + 6*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = 6*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,14)
      acc = 6*momenta(2,0)*momenta(4,3)
      acc = acc + 6*momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = 6*momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = 6*momenta(1,0)*momenta(2,3)
      acc = acc + 6*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = 6*momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,14)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,14)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = -6*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = -6*momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,14)
      acc = -6*momenta(2,2)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = -6*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,14)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c2(1,14)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c2(2,14)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c2(3,14)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c2(4,14)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c2(5,14)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c2(6,14)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,1)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(1,13)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,1)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(2,13)
      acc = momenta(2,2)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(3,13)
      acc = -momenta(2,2)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,2)
      reg1 = acc * momenta(3,3)
      acc = -momenta(2,3)*momenta(3,2)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(1,2)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      acc = acc + reg0
      acc = acc / 4.0_ki
      res = res + acc * coeffs%c3(4,13)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,1)*momenta(4,2)
      acc = acc - 3*momenta(3,2)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,13)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,1)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,13)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,0)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(4,0)
      acc = -3*momenta(3,2)*momenta(4,3)
      acc = acc - 3*momenta(3,3)*momenta(4,2)
      acc = acc * momenta(1,0)
      acc = acc * momenta(2,0)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,13)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(4,2)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc * momenta(4,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(3,1)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -3*momenta(1,1)*momenta(4,2)
      acc = acc - 3*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(3,3)
      acc = acc + reg2
      reg1 = acc * momenta(2,1)
      acc = -3*momenta(2,2)*momenta(3,3)
      acc = acc - 3*momenta(2,3)*momenta(3,2)
      acc = acc * momenta(1,1)
      acc = acc * momenta(4,1)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,13)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(1,3)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      reg3 = acc * momenta(4,3)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg3
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -3*momenta(1,1)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,1)
      reg3 = acc * momenta(3,0)
      acc = -3*momenta(1,0)*momenta(4,1)
      acc = acc - 3*momenta(1,1)*momenta(4,0)
      reg4 = acc * momenta(3,3)
      acc = -3*momenta(1,0)*momenta(4,3)
      acc = acc - 3*momenta(1,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg4
      acc = acc + reg3
      reg2 = acc * momenta(2,2)
      acc = -3*momenta(2,1)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,1)
      reg4 = acc * momenta(3,0)
      acc = -3*momenta(2,0)*momenta(4,1)
      acc = acc - 3*momenta(2,1)*momenta(4,0)
      reg5 = acc * momenta(3,3)
      acc = -3*momenta(2,0)*momenta(4,3)
      acc = acc - 3*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg5
      acc = acc + reg4
      acc = acc * momenta(1,2)
      acc = acc + reg2
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 24.0_ki
      res = res + acc * coeffs%c4(1,3)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,1)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,15)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = momenta(2,1)*momenta(4,3)
      acc = acc + momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,15)
      acc = momenta(2,2)*momenta(3,3)
      acc = acc + momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = momenta(1,2)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,15)
      acc = -momenta(2,2)*momenta(3,3)
      acc = acc - momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = -momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,15)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,15)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,15)
      acc = -momenta(2,0)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,15)
      acc = -momenta(2,1)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = -momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -momenta(2,1)*momenta(4,3)
      acc = acc - momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = -momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,15)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(1,15)
      acc = -momenta(1,0)*momenta(4,1)
      acc = acc - momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,1)
      acc = acc - momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(2,15)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = -momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,0)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(3,15)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = -momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c3(4,15)
      acc = momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c2(1,9)
      acc = momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c2(2,9)
      acc = momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c2(3,9)
      acc = -momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c2(4,9)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c2(5,9)
      acc = -momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c2(6,9)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      reg1 = acc * momenta(2,1)
      acc = -6*momenta(1,1)*momenta(2,0)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -6*momenta(1,0)*momenta(2,0)*momenta(4,1)
      acc = acc + reg2
      acc = acc * momenta(3,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(1,9)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg1 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(2,0)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,0)
      acc = acc + reg2
      acc = acc * momenta(4,0)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(2,9)
      acc = -6*momenta(2,0)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,0)
      reg1 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,3)*momenta(4,3)
      acc = acc + reg1
      reg0 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,0)
      reg2 = acc * momenta(4,0)
      acc = -6*momenta(1,0)*momenta(2,0)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(3,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(3,9)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,1)
      acc = -6*momenta(1,1)*momenta(2,1)*momenta(3,2)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,1)
      acc = acc + reg2
      acc = acc * momenta(4,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(4,9)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,1)
      acc = -6*momenta(2,1)*momenta(3,3)*momenta(4,1)
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,3)
      acc = -6*momenta(2,3)*momenta(3,1)*momenta(4,3)
      acc = acc + reg2
      acc = acc * momenta(1,1)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(5,9)
      acc = -6*momenta(2,2)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,2)
      reg1 = acc * momenta(1,3)
      acc = -6*momenta(1,2)*momenta(2,3)*momenta(3,3)
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,2)*momenta(2,3)
      acc = acc - 6*momenta(1,3)*momenta(2,2)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,2)*momenta(2,2)*momenta(3,3)
      acc = acc + reg2
      acc = acc * momenta(4,3)
      acc = acc + reg0
      acc = acc / 6.0_ki
      res = res + acc * coeffs%c2(6,9)
      acc = 15*momenta(1,0)*momenta(2,0)*momenta(3,0)*momenta(4,0)
      res = res + acc * coeffs%c1(1,6)
      acc = -15*momenta(1,1)*momenta(2,1)*momenta(3,1)*momenta(4,1)
      res = res + acc * coeffs%c1(2,6)
      acc = -15*momenta(1,2)*momenta(2,2)*momenta(3,2)*momenta(4,2)
      res = res + acc * coeffs%c1(3,6)
      acc = -15*momenta(1,3)*momenta(2,3)*momenta(3,3)*momenta(4,3)
      res = res + acc * coeffs%c1(4,6)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg1 = acc * momenta(3,0)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(4,2)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      reg2 = acc * momenta(2,2)
      acc = -6*momenta(2,0)*momenta(4,1)
      acc = acc - 6*momenta(2,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc + reg2
      reg1 = acc * momenta(3,2)
      acc = -6*momenta(3,0)*momenta(4,1)
      acc = acc - 6*momenta(3,1)*momenta(4,0)
      acc = acc * momenta(1,2)
      acc = acc * momenta(2,2)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(1,4)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg1 = acc * momenta(3,0)
      acc = -6*momenta(2,0)*momenta(4,1)
      acc = acc - 6*momenta(2,1)*momenta(4,0)
      reg2 = acc * momenta(3,3)
      acc = -6*momenta(2,0)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,0)
      acc = acc * momenta(3,1)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,1)
      acc = acc - 6*momenta(1,1)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,0)*momenta(4,1)
      acc = acc - 6*momenta(1,1)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -6*momenta(1,0)*momenta(3,1)
      acc = acc - 6*momenta(1,1)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = -6*momenta(2,0)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,0)
      reg1 = acc * momenta(4,2)
      acc = -6*momenta(2,0)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,0)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(3,0)*momenta(4,3)
      acc = acc - 6*momenta(3,3)*momenta(4,0)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,0)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,0)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,0)*momenta(4,2)
      acc = acc - 6*momenta(1,2)*momenta(4,0)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -6*momenta(1,0)*momenta(3,2)
      acc = acc - 6*momenta(1,2)*momenta(3,0)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = -6*momenta(2,1)*momenta(3,3)
      acc = acc - 6*momenta(2,3)*momenta(3,1)
      reg1 = acc * momenta(4,2)
      acc = -6*momenta(2,1)*momenta(4,3)
      acc = acc - 6*momenta(2,3)*momenta(4,1)
      reg2 = acc * momenta(3,2)
      acc = -6*momenta(3,1)*momenta(4,3)
      acc = acc - 6*momenta(3,3)*momenta(4,1)
      acc = acc * momenta(2,2)
      acc = acc + reg2
      acc = acc + reg1
      reg0 = acc * momenta(1,3)
      acc = -6*momenta(1,1)*momenta(2,2)
      acc = acc - 6*momenta(1,2)*momenta(2,1)
      reg2 = acc * momenta(4,3)
      acc = -6*momenta(1,1)*momenta(4,2)
      acc = acc - 6*momenta(1,2)*momenta(4,1)
      acc = acc * momenta(2,3)
      acc = acc + reg2
      reg1 = acc * momenta(3,3)
      acc = -6*momenta(1,1)*momenta(3,2)
      acc = acc - 6*momenta(1,2)*momenta(3,1)
      acc = acc * momenta(2,3)
      acc = acc * momenta(4,3)
      acc = acc + reg1
      acc = acc + reg0
      acc = acc / 12.0_ki
      res = res + acc * coeffs%c3(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_b_tensor_6
!****f* src/interface/tens_comb/contract_c_tensor_6
! NAME
!
! Function contract_c_tensor_6
!
! USAGE
!
!  ans = contract_c_tensor_6(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an C-type tensor
!  of rank 6 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_c_tensor_6(coeffs, momenta) result(res)
   ! generated by: write_function_c_tensor_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      res = -coeffs%c2(1,7)
      res = res - coeffs%c2(2,7)
      res = res - coeffs%c2(3,7)
      res = res + coeffs%c2(4,7)
      res = res + coeffs%c2(5,7)
      res = res + coeffs%c2(6,7)
      acc = 3
      res = res + acc * coeffs%c1(1,4)
      acc = 3
      res = res + acc * coeffs%c1(2,4)
      acc = 3
      res = res + acc * coeffs%c1(3,4)
      acc = 3
      res = res + acc * coeffs%c1(4,4)
   case(1)
      acc = -3*momenta(1,0)
      res = acc * coeffs%c2(1,11)
      acc = -3*momenta(1,0)
      res = res + acc * coeffs%c2(2,11)
      acc = -3*momenta(1,0)
      res = res + acc * coeffs%c2(3,11)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(4,11)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(5,11)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(6,11)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(1,4)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(2,4)
      acc = 3*momenta(1,0)
      res = res + acc * coeffs%c2(3,4)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(4,4)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(5,4)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(6,4)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(1,6)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(2,6)
      acc = momenta(1,0)
      res = res + acc * coeffs%c3(3,6)
      acc = momenta(1,1)
      res = res + acc * coeffs%c3(4,6)
      acc = 15*momenta(1,0)
      res = res + acc * coeffs%c1(1,5)
      acc = 15*momenta(1,1)
      res = res + acc * coeffs%c1(2,5)
      acc = 15*momenta(1,2)
      res = res + acc * coeffs%c1(3,5)
      acc = 15*momenta(1,3)
      res = res + acc * coeffs%c1(4,5)
      acc = -3*momenta(1,1)
      res = res + acc * coeffs%c2(1,8)
      acc = -3*momenta(1,2)
      res = res + acc * coeffs%c2(2,8)
      acc = -3*momenta(1,3)
      res = res + acc * coeffs%c2(3,8)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(4,8)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(5,8)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(6,8)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c3(1,12)
      acc = -momenta(1,1)
      res = res + acc * coeffs%c3(2,12)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c3(3,12)
      acc = momenta(1,2)
      res = res + acc * coeffs%c3(4,12)
      acc = 3*momenta(1,1)
      res = res + acc * coeffs%c2(1,13)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(2,13)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(3,13)
      acc = 3*momenta(1,2)
      res = res + acc * coeffs%c2(4,13)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(5,13)
      acc = 3*momenta(1,3)
      res = res + acc * coeffs%c2(6,13)
      acc = -momenta(1,2)
      res = res + acc * coeffs%c3(1,14)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c3(2,14)
      acc = -momenta(1,3)
      res = res + acc * coeffs%c3(3,14)
      acc = momenta(1,3)
      res = res + acc * coeffs%c3(4,14)
   case(2)
      acc = -momenta(1,2)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = acc * coeffs%c4(1,14)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,9)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,9)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,9)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,9)
      acc = 15*momenta(1,0)*momenta(2,1)
      acc = acc + 15*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,15)
      acc = 15*momenta(1,0)*momenta(2,2)
      acc = acc + 15*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,15)
      acc = 15*momenta(1,0)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,15)
      acc = 15*momenta(1,1)*momenta(2,2)
      acc = acc + 15*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,15)
      acc = 15*momenta(1,1)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,15)
      acc = 15*momenta(1,2)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,15)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,16)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,16)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,16)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,16)
      acc = momenta(1,0)*momenta(2,1)
      acc = acc + momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c4(1,5)
      acc = -momenta(1,1)*momenta(2,3)
      acc = acc - momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c4(1,13)
      acc = -9*momenta(1,0)*momenta(2,1)
      acc = acc - 9*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,12)
      acc = -9*momenta(1,0)*momenta(2,2)
      acc = acc - 9*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,12)
      acc = -9*momenta(1,0)*momenta(2,3)
      acc = acc - 9*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,12)
      acc = 9*momenta(1,1)*momenta(2,2)
      acc = acc + 9*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,12)
      acc = 9*momenta(1,1)*momenta(2,3)
      acc = acc + 9*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,12)
      acc = 9*momenta(1,2)*momenta(2,3)
      acc = acc + 9*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,12)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,10)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,10)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,10)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,10)
      acc = momenta(1,0)*momenta(2,3)
      acc = acc + momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c4(1,9)
      acc = 15*momenta(1,0)*momenta(2,1)
      acc = acc + 15*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(1,5)
      acc = 15*momenta(1,0)*momenta(2,2)
      acc = acc + 15*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(2,5)
      acc = 15*momenta(1,0)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(3,5)
      acc = 15*momenta(1,1)*momenta(2,2)
      acc = acc + 15*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(4,5)
      acc = 15*momenta(1,1)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(5,5)
      acc = 15*momenta(1,2)*momenta(2,3)
      acc = acc + 15*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c2(6,5)
      acc = -momenta(1,1)*momenta(2,2)
      acc = acc - momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c4(1,12)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,18)
      acc = -3*momenta(1,0)*momenta(2,1)
      acc = acc - 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,18)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,18)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,18)
      acc = momenta(1,0)*momenta(2,2)
      acc = acc + momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c4(1,8)
      acc = -3*momenta(1,0)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,19)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,19)
      acc = -3*momenta(1,0)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,19)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,19)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,7)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,7)
      acc = 3*momenta(1,0)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,7)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,7)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,20)
      acc = 3*momenta(1,1)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,20)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,20)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,20)
      acc = 3*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(1,14)
      acc = 3*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(2,14)
      acc = 3*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(3,14)
      acc = 3*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(4,14)
      acc = 3*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(5,14)
      acc = 3*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(6,14)
      acc = -6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(1,14)
      acc = -6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(2,14)
      acc = -6*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(3,14)
      acc = 6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(4,14)
      acc = 6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(5,14)
      acc = 6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(6,14)
      acc = -3*momenta(1,1)*momenta(2,2)
      acc = acc - 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,13)
      acc = -3*momenta(1,1)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,13)
      acc = -3*momenta(1,2)*momenta(2,3)
      acc = acc - 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,13)
      acc = 3*momenta(1,2)*momenta(2,3)
      acc = acc + 3*momenta(1,3)*momenta(2,2)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,13)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c3(1,15)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c3(2,15)
      acc = -momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c3(3,15)
      acc = momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c3(4,15)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c3(1,15)
      acc = -momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c3(2,15)
      acc = -momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c3(3,15)
      acc = momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c3(4,15)
      acc = momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c3(1,15)
      acc = momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c3(2,15)
      acc = momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c3(3,15)
      acc = momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c3(4,15)
      acc = 3*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(1,9)
      acc = 3*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(2,9)
      acc = 3*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c2(3,9)
      acc = 3*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(4,9)
      acc = 3*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(5,9)
      acc = 3*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(6,9)
      acc = -6*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c2(1,9)
      acc = -6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(2,9)
      acc = -6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(3,9)
      acc = 6*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c2(4,9)
      acc = 6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(5,9)
      acc = 6*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c2(6,9)
      acc = 45*momenta(1,0)*momenta(2,0)
      res = res + acc * coeffs%c1(1,6)
      acc = 45*momenta(1,1)*momenta(2,1)
      res = res + acc * coeffs%c1(2,6)
      acc = 45*momenta(1,2)*momenta(2,2)
      res = res + acc * coeffs%c1(3,6)
      acc = 45*momenta(1,3)*momenta(2,3)
      res = res + acc * coeffs%c1(4,6)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(1,4)
      acc = 3*momenta(1,0)*momenta(2,1)
      acc = acc + 3*momenta(1,1)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(2,4)
      acc = 3*momenta(1,0)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,0)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(3,4)
      acc = 3*momenta(1,1)*momenta(2,2)
      acc = acc + 3*momenta(1,2)*momenta(2,1)
      acc = acc / 2.0_ki
      res = res + acc * coeffs%c3(4,4)
   case default
      res = 0.0_ki
   end select
end  function contract_c_tensor_6
!****f* src/interface/tens_comb/contract_d_tensor_6
! NAME
!
! Function contract_d_tensor_6
!
! USAGE
!
!  ans = contract_d_tensor_6(coeffs, momenta)
!
! DESCRIPTION
!
!  Contracts the a set of coefficients with an D-type tensor
!  of rank 6 constructed from a given set of momenta.
!
! INPUTS
!
!  * coeffs  -- coefficients of type(coeff_type_6)
!  * momenta -- real array of dimension(:,3) containing the
!               momenta r_i of the loop propagators
!
! RETURN VALUE
!
!  The result of the contraction which is a complex number
!
! SIDE EFFECTS
!
!  None
!
! EXAMPLE
!
!
!*****
pure function contract_d_tensor_6(coeffs, momenta) result(res)
   ! generated by: write_function_d_tensor_contract
   implicit none
   type(coeff_type_6), intent(in) :: coeffs
   real(ki), dimension(:,0:), intent(in), optional :: momenta
   complex(ki) :: res
   integer :: rk
   real(ki) :: acc
   if (present(momenta)) then
      rk = size(momenta, 1)
   else
      rk = 0
   end if
   select case(rk)
   case(0)
      acc = -3
      res = acc * coeffs%c2(1,14)
      acc = -3
      res = res + acc * coeffs%c2(2,14)
      acc = -3
      res = res + acc * coeffs%c2(3,14)
      acc = -3
      res = res + acc * coeffs%c2(4,14)
      acc = -3
      res = res + acc * coeffs%c2(5,14)
      acc = -3
      res = res + acc * coeffs%c2(6,14)
      res = res + coeffs%c3(1,15)
      res = res + coeffs%c3(2,15)
      res = res + coeffs%c3(3,15)
      res = res - coeffs%c3(4,15)
      acc = 3
      res = res + acc * coeffs%c2(1,9)
      acc = 3
      res = res + acc * coeffs%c2(2,9)
      acc = 3
      res = res + acc * coeffs%c2(3,9)
      acc = -3
      res = res + acc * coeffs%c2(4,9)
      acc = -3
      res = res + acc * coeffs%c2(5,9)
      acc = -3
      res = res + acc * coeffs%c2(6,9)
      acc = 15
      res = res + acc * coeffs%c1(1,6)
      acc = -15
      res = res + acc * coeffs%c1(2,6)
      acc = -15
      res = res + acc * coeffs%c1(3,6)
      acc = -15
      res = res + acc * coeffs%c1(4,6)
   case default
      res = 0.0_ki
   end select
end  function contract_d_tensor_6
end module tens_comb
